searchData={"items":[{"type":"module","title":"Credo","doc":"Credo builds upon four building blocks:\n\n- `Credo.CLI` - everything related to the command line interface (CLI), which orchestrates the analysis\n- `Credo.Execution` - a struct which is handed down the pipeline during analysis\n- `Credo.Check` - the default Credo checks\n- `Credo.Code` - all analysis tools used by Credo during analysis","ref":"Credo.html"},{"type":"function","title":"Credo.run/1","doc":"Runs Credo with the given `argv` and returns its final `Credo.Execution` struct.\n\nExample:\n\n    iex> exec = Credo.run([\"--only\", \"Readability\"])\n    iex> issues = Credo.Execution.get_issues(exec)\n    iex> Enum.count(issues) > 0\n    true","ref":"Credo.html#run/1"},{"type":"function","title":"Credo.run/2","doc":"","ref":"Credo.html#run/2"},{"type":"function","title":"Credo.version/0","doc":"","ref":"Credo.html#version/0"},{"type":"task","title":"mix credo","doc":"Run code analysis (use `--help` for options)","ref":"Mix.Tasks.Credo.html"},{"type":"task","title":"mix credo.gen.check","doc":"Generate a new custom check for Credo","ref":"Mix.Tasks.Credo.Gen.Check.html"},{"type":"task","title":"mix credo.gen.config","doc":"Generate a new config for Credo","ref":"Mix.Tasks.Credo.Gen.Config.html"},{"type":"behaviour","title":"Credo.Check","doc":"`Check` modules represent the checks which are run during Credo's analysis.\n\nExample:\n\n    defmodule MyCheck do\n      use Credo.Check, category: :warning, base_priority: :high\n\n      def run(%SourceFile{} = source_file, params) do\n        #\n      end\n    end\n\nThe check can be configured by passing the following\noptions to `use Credo.Check`:\n\n- `:base_priority`  Sets the checks's base priority (`:low`, `:normal`, `:high`, `:higher` or `:ignore`).\n- `:category`       Sets the check's category (`:consistency`, `:design`, `:readability`, `:refactor`  or `:warning`).\n- `:elixir_version` Sets the check's version requirement for Elixir (defaults to `>= 0.0.1`).\n- `:explanations`   Sets explanations displayed for the check, e.g.\n\n    ```elixir\n    [\n      check: \"...\",\n      params: [\n        param1: \"Your favorite number\",\n        param2: \"Online/Offline mode\"\n      ]\n    ]\n    ```\n\n- `:param_defaults` Sets the default values for the check's params (e.g. `[param1: 42, param2: \"offline\"]`)\n- `:tags`           Sets the tags for this check (list of atoms, e.g. `[:tag1, :tag2]`)\n\nPlease also note that these options to `use Credo.Check` are just a convenience to implement the `Credo.Check`\nbehaviour. You can implement any of these by hand:\n\n    defmodule MyCheck do\n      use Credo.Check\n\n      def category, do: :warning\n\n      def base_priority, do: :high\n\n      def explanations do\n        [\n          check: \"...\",\n          params: [\n            param1: \"Your favorite number\",\n            param2: \"Online/Offline mode\"\n          ]\n        ]\n      end\n\n      def param_defaults, do: [param1: 42, param2: \"offline\"]\n\n      def run(%SourceFile{} = source_file, params) do\n        #\n      end\n    end\n\nThe `run/2` function of a Check module takes two parameters: a source file and a list of parameters for the check.\nIt has to return a list of found issues.","ref":"Credo.Check.html"},{"type":"callback","title":"Credo.Check.base_priority/0","doc":"Returns the base priority for the check.\n\nThis can be one of `:higher`, `:high`, `:normal`, `:low` or `:ignore`\n(technically it can also be  or an integer, but these are internal representations although that is not recommended).","ref":"Credo.Check.html#c:base_priority/0"},{"type":"callback","title":"Credo.Check.category/0","doc":"Returns the category for the check.","ref":"Credo.Check.html#c:category/0"},{"type":"callback","title":"Credo.Check.docs_uri/0","doc":"Returns the docs URL for the check.","ref":"Credo.Check.html#c:docs_uri/0"},{"type":"callback","title":"Credo.Check.elixir_version/0","doc":"Returns the required Elixir version for the check.","ref":"Credo.Check.html#c:elixir_version/0"},{"type":"callback","title":"Credo.Check.exit_status/0","doc":"Returns the exit status for the check.","ref":"Credo.Check.html#c:exit_status/0"},{"type":"function","title":"Credo.Check.explanation_for/2","doc":"","ref":"Credo.Check.html#explanation_for/2"},{"type":"callback","title":"Credo.Check.explanations/0","doc":"Returns the explanations for the check and params as a keyword list.","ref":"Credo.Check.html#c:explanations/0"},{"type":"callback","title":"Credo.Check.format_issue/2","doc":"","ref":"Credo.Check.html#c:format_issue/2"},{"type":"function","title":"Credo.Check.format_issue/3","doc":"format_issue takes an issue_meta and returns an issue.\nThe resulting issue can be made more explicit by passing the following\noptions to `format_issue/2`:\n\n- `:priority`     Sets the issue's priority.\n- `:trigger`      Sets the issue's trigger, i.e. the text causing the issue (see `Credo.Check.Warning.IoInspect`).\n- `:line_no`      Sets the issue's line number. Tries to find `column` if `:trigger` is supplied.\n- `:column`       Sets the issue's column.\n- `:exit_status`  Sets the issue's exit_status.\n- `:severity`     Sets the issue's severity.","ref":"Credo.Check.html#format_issue/3"},{"type":"callback","title":"Credo.Check.id/0","doc":"Returns an ID that can be used to identify the check.","ref":"Credo.Check.html#c:id/0"},{"type":"callback","title":"Credo.Check.param_defaults/0","doc":"Returns the default values for the check's params as a keyword list.","ref":"Credo.Check.html#c:param_defaults/0"},{"type":"callback","title":"Credo.Check.run/2","doc":"","ref":"Credo.Check.html#c:run/2"},{"type":"callback","title":"Credo.Check.run_on_all?/0","doc":"Returns whether or not this check runs on all source files.","ref":"Credo.Check.html#c:run_on_all?/0"},{"type":"callback","title":"Credo.Check.run_on_all_source_files/3","doc":"Runs the current check on all `source_files` by calling `run_on_source_file/3`.\n\nIf you are developing a check that has to run on all source files, you can overwrite `run_on_all_source_files/3`:\n\n    defmodule MyCheck do\n      use Credo.Check\n\n      def run_on_all_source_files(exec, source_files, params) do\n        issues =\n          source_files\n          |> do_something_crazy()\n          |> do_something_crazier()\n\n        append_issues_and_timings(exec, issues)\n\n        :ok\n      end\n    end\n\nCheck out Credo's checks from the consistency category for examples of these kinds of checks.","ref":"Credo.Check.html#c:run_on_all_source_files/3"},{"type":"callback","title":"Credo.Check.run_on_source_file/3","doc":"Runs the current check on a single `source_file` and appends the resulting issues to the current `exec`.","ref":"Credo.Check.html#c:run_on_source_file/3"},{"type":"callback","title":"Credo.Check.tags/0","doc":"Returns the tags for the check.","ref":"Credo.Check.html#c:tags/0"},{"type":"function","title":"Credo.Check.to_exit_status/1","doc":"Converts a given category to an exit status","ref":"Credo.Check.html#to_exit_status/1"},{"type":"module","title":"Credo.Plugin","doc":"Plugins are module which can provide additional functionality to Credo.\n\nA plugin is basically just a module that provides an `init/1` callback.\n\n    defmodule CredoDemoPlugin do\n      def init(exec) do\n        # but what do we do here??\n        exec\n      end\n    end\n\nThe `Credo.Plugin` module provides a number of functions for extending Credo's core features.\n\n    defmodule CredoDemoPlugin do\n      @config_file File.read!(\".credo.exs\")\n\n      import Credo.Plugin\n\n      def init(exec) do\n        exec\n        |> register_default_config(@config_file)\n        |> register_command(\"demo\", CredoDemoPlugin.DemoCommand)\n        |> register_cli_switch(:castle, :string, :X)\n        |> prepend_task(:set_default_command, CredoDemoPlugin.SetDemoAsDefaultCommand)\n      end\n    end","ref":"Credo.Plugin.html"},{"type":"function","title":"Credo.Plugin.append_task/3","doc":"Appends a `Credo.Execution.Task` module to Credo's main execution pipeline.\n\nCredo's execution pipeline consists of several steps, each with a group of tasks, which you can hook into.\n\nAppending tasks to these steps is easy:\n\n    # credo_demo_plugin.ex\n    defmodule CredoDemoPlugin do\n      import Credo.Plugin\n\n      def init(exec) do\n        append_task(exec, :set_default_command, CredoDemoPlugin.SetDemoAsDefaultCommand)\n      end\n    end\n\nNote how `Credo.Plugin.append_task/3` takes two arguments after the `Credo.Execution` struct: the name of the group to be modified and the module that should be executed.\n\nThe group names of Credo's main pipeline are:\n\n- `:parse_cli_options`\n- `:initialize_plugins`\n- `:determine_command`\n- `:set_default_command`\n- `:initialize_command`\n- `:parse_cli_options_final`\n- `:validate_cli_options`\n- `:convert_cli_options_to_config`\n- `:resolve_config`\n- `:validate_config`\n- `:run_command`\n- `:halt_execution`\n\n\nThe module appended to these groups should use `Credo.Execution.Task`:\n\n    # credo_demo_plugin/set_demo_as_default_command.ex\n    defmodule CredoDemoPlugin.SetDemoAsDefaultCommand do\n      use Credo.Execution.Task\n\n      alias Credo.CLI.Options\n\n      def call(exec, _opts) do\n        set_command(exec, exec.cli_options.command || \"demo\")\n      end\n\n      defp set_command(exec, command) do\n        %Execution{exec | cli_options: %Options{exec.cli_options | command: command}}\n      end\n    end\n\nThis example would have the effect that typing `mix credo` would no longer run the built-in `Suggest` command, but rather our plugin's `Demo` command.","ref":"Credo.Plugin.html#append_task/3"},{"type":"function","title":"Credo.Plugin.append_task/4","doc":"Appends a `Credo.Execution.Task` module to the execution pipeline of an existing Command.\n\nCredo's commands can also have an execution pipeline of their own, which is executed when the command is used and which you can hook into as well.\n\nAppending tasks to these steps is easy:\n\n    # credo_demo_plugin.ex\n    defmodule CredoDemoPlugin do\n      import Credo.Plugin\n\n      def init(exec) do\n        append_task(exec, Credo.CLI.Command.Suggest.SuggestCommand, :print_after_analysis, CredoDemoPlugin.WriteFile)\n      end\n    end\n\nNote how `Credo.Plugin.append_task/4` takes three arguments after the `Credo.Execution` struct: the pipeline and the name of the group to be modified and the module that should be executed.\n\nHere are the pipeline keys and group names:\n\n- `Credo.CLI.Command.Suggest.SuggestCommand` (run via `mix credo suggest`)\n  - `:load_and_validate_source_files`\n  - `:prepare_analysis`\n  - `:print_before_analysis`\n  - `:run_analysis`\n  - `:filter_issues`\n  - `:print_after_analysis`\n\n- `Credo.CLI.Command.List.ListCommand` (run via `mix credo list`)\n  - `:load_and_validate_source_files`\n  - `:prepare_analysis`\n  - `:print_before_analysis`\n  - `:run_analysis`\n  - `:filter_issues`\n  - `:print_after_analysis`\n\n- `Credo.CLI.Command.Diff.DiffCommand` (run via `mix credo diff`)\n  - `:load_and_validate_source_files`\n  - `:prepare_analysis`\n  - `:print_previous_analysis`\n  - `:run_analysis`\n  - `:filter_issues`\n  - `:print_after_analysis`\n  - `:filter_issues_for_exit_status`\n\n- `Credo.CLI.Command.Info.InfoCommand` (run via `mix credo info`)\n  - `:load_and_validate_source_files`\n  - `:prepare_analysis`\n  - `:print_info`\n\n\nThe module appended to these groups should use `Credo.Execution.Task`:\n\n    # credo_demo_plugin/write_file.ex\n    defmodule CredoDemoPlugin.WriteFile do\n      use Credo.Execution.Task\n\n      alias Credo.CLI.Options\n\n      def call(exec, _opts) do\n        issue_count = exec |> Execution.get_issues() |> Enum.count\n        File.write!(\"demo.json\", ~q({\"issue_count\": #{issue_count}}))\n\n        exec\n      end\n    end\n\nThis example would have the effect that running `mix credo suggest` would write the issue count in a JSON file.","ref":"Credo.Plugin.html#append_task/4"},{"type":"function","title":"Credo.Plugin.prepend_task/3","doc":"Prepends a `Credo.Execution.Task` module to Credo's main execution pipeline.\n\nCredo's execution pipeline consists of several steps, each with a group of tasks, which you can hook into.\n\nPrepending tasks to these steps is easy:\n\n    # credo_demo_plugin.ex\n    defmodule CredoDemoPlugin do\n      import Credo.Plugin\n\n      def init(exec) do\n        prepend_task(exec, :set_default_command, CredoDemoPlugin.SetDemoAsDefaultCommand)\n      end\n    end\n\nNote how `Credo.Plugin.prepend_task/3` takes two arguments after the `Credo.Execution` struct: the name of the group to be modified and the module that should be executed.\n\nThe group names of Credo's main pipeline are:\n\n- `:parse_cli_options`\n- `:initialize_plugins`\n- `:determine_command`\n- `:set_default_command`\n- `:initialize_command`\n- `:parse_cli_options_final`\n- `:validate_cli_options`\n- `:convert_cli_options_to_config`\n- `:resolve_config`\n- `:validate_config`\n- `:run_command`\n- `:halt_execution`\n\n\nThe module prepended to these groups should use `Credo.Execution.Task`:\n\n    # credo_demo_plugin/set_demo_as_default_command.ex\n    defmodule CredoDemoPlugin.SetDemoAsDefaultCommand do\n      use Credo.Execution.Task\n\n      alias Credo.CLI.Options\n\n      def call(exec, _opts) do\n        set_command(exec, exec.cli_options.command || \"demo\")\n      end\n\n      defp set_command(exec, command) do\n        %Execution{exec | cli_options: %Options{exec.cli_options | command: command}}\n      end\n    end\n\nThis example would have the effect that typing `mix credo` would no longer run the built-in `Suggest` command, but rather our plugin's `Demo` command.","ref":"Credo.Plugin.html#prepend_task/3"},{"type":"function","title":"Credo.Plugin.prepend_task/4","doc":"Prepends a `Credo.Execution.Task` module to the execution pipeline of an existing Command.\n\nCredo's commands can also have an execution pipeline of their own, which is executed when the command is used and which you can hook into as well.\n\nPrepending tasks to these steps is easy:\n\n    # credo_demo_plugin.ex\n    defmodule CredoDemoPlugin do\n      import Credo.Plugin\n\n      def init(exec) do\n        prepend_task(exec, Credo.CLI.Command.Suggest.SuggestCommand, :print_after_analysis, CredoDemoPlugin.WriteFile)\n      end\n    end\n\nNote how `Credo.Plugin.prepend_task/4` takes three arguments after the `Credo.Execution` struct: the pipeline and the name of the group to be modified and the module that should be executed.\n\nHere are the pipeline keys and group names:\n\n- `Credo.CLI.Command.Suggest.SuggestCommand` (run via `mix credo suggest`)\n  - `:load_and_validate_source_files`\n  - `:prepare_analysis`\n  - `:print_before_analysis`\n  - `:run_analysis`\n  - `:filter_issues`\n  - `:print_after_analysis`\n\n- `Credo.CLI.Command.List.ListCommand` (run via `mix credo list`)\n  - `:load_and_validate_source_files`\n  - `:prepare_analysis`\n  - `:print_before_analysis`\n  - `:run_analysis`\n  - `:filter_issues`\n  - `:print_after_analysis`\n\n- `Credo.CLI.Command.Diff.DiffCommand` (run via `mix credo diff`)\n  - `:load_and_validate_source_files`\n  - `:prepare_analysis`\n  - `:print_previous_analysis`\n  - `:run_analysis`\n  - `:filter_issues`\n  - `:print_after_analysis`\n  - `:filter_issues_for_exit_status`\n\n- `Credo.CLI.Command.Info.InfoCommand` (run via `mix credo info`)\n  - `:load_and_validate_source_files`\n  - `:prepare_analysis`\n  - `:print_info`\n\n\nThe module prepended to these groups should use `Credo.Execution.Task`:\n\n    # credo_demo_plugin/write_file.ex\n    defmodule CredoDemoPlugin.WriteFile do\n      use Credo.Execution.Task\n\n      alias Credo.CLI.Options\n\n      def call(exec, _opts) do\n        issue_count = exec |> Execution.get_issues() |> Enum.count\n        File.write!(\"demo.json\", ~q({\"issue_count\": #{issue_count}}))\n\n        exec\n      end\n    end\n\nThis example would have the effect that running `mix credo suggest` would write the issue count in a JSON file.","ref":"Credo.Plugin.html#prepend_task/4"},{"type":"function","title":"Credo.Plugin.register_cli_switch/5","doc":"Adds a CLI switch to Credo.\n\nFor demo purposes, we are writing a command called `demo` (see `register_command/3`):\n\n    # credo_demo_plugin/demo_command.ex\n    defmodule CredoDemoPlugin do\n      import Credo.Plugin\n\n      def init(exec) do\n        exec\n        |> register_command(\"demo\", CredoDemoPlugin.DemoCommand)\n      end\n    end\n\n    # credo_demo_plugin/demo_command.ex\n    defmodule CredoDemoPlugin.DemoCommand do\n      alias Credo.CLI.Output.UI\n      alias Credo.Execution\n\n      def call(exec, _) do\n        castle = Execution.get_plugin_param(exec, CredoDemoPlugin, :castle)\n\n        UI.puts(\"By the power of #{castle}!\")\n\n        exec\n      end\n    end\n\nSince Plugins can be configured by params in `.credo.exs`, we can add the `:castle` param:\n\n    # .credo.exs\n    {CredoDemoPlugin, [castle: \"Grayskull\"]}\n\nAnd get the following output:\n\n```bash\n$ mix credo demo\nBy the power of Grayskull!\n```\n\nPlugins can provide custom CLI options as well, so we can do something like:\n\n```bash\n$ mix credo demo --castle Winterfell\nUnknown switch: --castle\n```\n\nRegistering a custom CLI switch for this is easy:\n\n    defmodule CredoDemoPlugin do\n      import Credo.Plugin\n\n      def init(exec) do\n        exec\n        |> register_command(\"demo\", CredoDemoPlugin.DemoCommand)\n        |> register_cli_switch(:castle, :string, :X)\n      end\n    end\n\nEvery registered CLI switch is automatically converted into a plugin param of the same name, which is why we get the following output:\n\n```bash\n$ mix credo demo --castle Winterfell\nBy the power of Winterfell!\n\n$ mix credo demo -X Camelot\nBy the power of Camelot!\n```\n\nPlugin authors can also provide a function to control the plugin param's name and value more granularly:\n\n    defmodule CredoDemoPlugin do\n      import Credo.Plugin\n\n      def init(exec) do\n        register_cli_switch(exec, :kastle, :string, :X, fn(switch_value) ->\n          {:castle, String.upcase(switch_value)}\n        end)\n      end\n    end\n\nAnd get the following output:\n\n```bash\n$ mix credo demo --kastle Winterfell\nBy the power of WINTERFELL!\n```","ref":"Credo.Plugin.html#register_cli_switch/5"},{"type":"function","title":"Credo.Plugin.register_command/3","doc":"Registers and initializes a Command module with a given `name`.","ref":"Credo.Plugin.html#register_command/3"},{"type":"function","title":"Add new commands - Credo.Plugin.register_command/3","doc":"Commands are just modules with a call function and adding new commands is easy.\n\n    # credo_demo_plugin.ex\n    defmodule CredoDemoPlugin do\n      import Credo.Plugin\n\n      def init(exec) do\n        register_command(exec, \"demo\", CredoDemoPlugin.DemoCommand)\n      end\n    end\n\n    # credo_demo_plugin/demo_command.ex\n    defmodule CredoDemoPlugin.DemoCommand do\n      alias Credo.CLI.Output.UI\n      alias Credo.Execution\n\n      def call(exec, _) do\n        castle = Execution.get_plugin_param(exec, CredoDemoPlugin, :castle)\n\n        UI.puts(\"By the power of #{castle}!\")\n\n        exec\n      end\n    end\n\nUsers can use this command by typing\n\n```bash\n$ mix credo demo\nBy the power of !\n```","ref":"Credo.Plugin.html#register_command/3-add-new-commands"},{"type":"function","title":"Override an existing command - Credo.Plugin.register_command/3","doc":"Since commands are just modules with a call function, overriding existing commands is easy.\n\n    defmodule CredoDemoPlugin do\n      import Credo.Plugin\n\n      def init(exec) do\n        register_command(exec, \"explain\", CredoDemoPlugin.MyBetterExplainCommand)\n      end\n    end\n\nThis example would have the effect that typing `mix credo lib/my_file.ex:42` would no longer run the built-in `Explain` command, but rather our plugin's `MyBetterExplain` command.","ref":"Credo.Plugin.html#register_command/3-override-an-existing-command"},{"type":"function","title":"Credo.Plugin.register_default_config/2","doc":"Registers the contents of a config file.\n\nThis registers the contents of a config file as default config, loading it after Credo's own default config but before the [config files loaded from the current working directory](config_file.html#transitive-configuration-files).\n\n    defmodule CredoDemoPlugin do\n      @config_file File.read!(\".credo.exs\")\n\n      import Credo.Plugin\n\n      def init(exec) do\n        register_default_config(exec, @config_file)\n      end\n    end","ref":"Credo.Plugin.html#register_default_config/2"},{"type":"module","title":"Credo.Execution","doc":"Every run of Credo is configured via an `Credo.Execution` struct, which is created and\nmanipulated via the `Credo.Execution` module.","ref":"Credo.Execution.html"},{"type":"function","title":"Credo.Execution.__struct__/0","doc":"The `Credo.Execution` struct is created and manipulated via the `Credo.Execution` module.","ref":"Credo.Execution.html#__struct__/0"},{"type":"function","title":"Credo.Execution.build/1","doc":"Builds an Execution struct for the the given `argv`.","ref":"Credo.Execution.html#build/1"},{"type":"function","title":"Credo.Execution.checks/1","doc":"Returns the checks that should be run for a given `exec` struct.\n\nTakes all checks from the `checks:` field of the exec, matches those against\nany patterns to include or exclude certain checks given via the command line.","ref":"Credo.Execution.html#checks/1"},{"type":"function","title":"Credo.Execution.ensure_execution_struct/2","doc":"Ensures that the given `value` is a `%Credo.Execution{}` struct, raises an error otherwise.\n\nExample:\n\n    exec\n    |> mod.init()\n    |> Credo.Execution.ensure_execution_struct(\"#{mod}.init/1\")","ref":"Credo.Execution.html#ensure_execution_struct/2"},{"type":"function","title":"Credo.Execution.get_assign/3","doc":"Returns the assign with the given `name` for the given `exec` struct (or return the given `default` value).\n\n    Credo.Execution.get_assign(exec, \"foo\")\n    # => nil\n\n    Credo.Execution.get_assign(exec, \"foo\", 42)\n    # => 42","ref":"Credo.Execution.html#get_assign/3"},{"type":"function","title":"Credo.Execution.get_command/2","doc":"Returns the `Credo.CLI.Command` module for the given `name`.\n\n    Credo.Execution.get_command(exec, \"explain\")\n    # => Credo.CLI.Command.Explain.ExplainCommand","ref":"Credo.Execution.html#get_command/2"},{"type":"function","title":"Credo.Execution.get_command_name/1","doc":"Returns the name of the command, which should be run by the given execution.\n\n    Credo.Execution.get_command_name(exec)\n    # => \"suggest\"","ref":"Credo.Execution.html#get_command_name/1"},{"type":"function","title":"Credo.Execution.get_given_cli_switch/2","doc":"Returns the value for the given `switch_name`.\n\n    Credo.Execution.get_given_cli_switch(exec, \"foo\")\n    # => \"bar\"","ref":"Credo.Execution.html#get_given_cli_switch/2"},{"type":"function","title":"Credo.Execution.get_issues/1","doc":"Returns all issues for the given `exec` struct.","ref":"Credo.Execution.html#get_issues/1"},{"type":"function","title":"Credo.Execution.get_issues/2","doc":"Returns all issues for the given `exec` struct that relate to the given `filename`.","ref":"Credo.Execution.html#get_issues/2"},{"type":"function","title":"Credo.Execution.get_issues_grouped_by_filename/1","doc":"Returns all issues grouped by filename for the given `exec` struct.","ref":"Credo.Execution.html#get_issues_grouped_by_filename/1"},{"type":"function","title":"Credo.Execution.get_plugin_param/3","doc":"Returns the `Credo.Plugin` module's param value.\n\n    Credo.Execution.get_command(exec, CredoDemoPlugin, \"foo\")\n    # => nil\n\n    Credo.Execution.get_command(exec, CredoDemoPlugin, \"foo\", 42)\n    # => 42","ref":"Credo.Execution.html#get_plugin_param/3"},{"type":"function","title":"Credo.Execution.get_result/3","doc":"Returns the result with the given `name` for the given `exec` struct (or return the given `default` value).\n\n    Credo.Execution.get_result(exec, \"foo\")\n    # => nil\n\n    Credo.Execution.get_result(exec, \"foo\", 42)\n    # => 42","ref":"Credo.Execution.html#get_result/3"},{"type":"function","title":"Credo.Execution.get_source_files/1","doc":"Returns all source files for the given `exec` struct.\n\n    Credo.Execution.get_source_files(exec)\n    # => [%SourceFile ,\n    #     %SourceFile ]","ref":"Credo.Execution.html#get_source_files/1"},{"type":"function","title":"Credo.Execution.get_valid_command_names/1","doc":"Returns all valid command names.\n\n    Credo.Execution.get_valid_command_names(exec)\n    # => [\"categories\", \"diff\", \"explain\", \"gen.check\", \"gen.config\", \"help\", \"info\",\n    #     \"list\", \"suggest\", \"version\"]","ref":"Credo.Execution.html#get_valid_command_names/1"},{"type":"function","title":"Credo.Execution.halt/1","doc":"Halts further execution of the pipeline meaning all subsequent steps are skipped.\n\nThe `error` callback is called for the current Task.\n\n    defmodule FooTask do\n      use Credo.Execution.Task\n\n      def call(exec) do\n        Execution.halt(exec)\n      end\n\n      def error(exec) do\n        IO.puts(\"Execution has been halted!\")\n\n        exec\n      end\n    end","ref":"Credo.Execution.html#halt/1"},{"type":"function","title":"Credo.Execution.halt/2","doc":"Halts further execution of the pipeline using the given `halt_message`.\n\nThe `error` callback is called for the current Task.\nIf the callback is not implemented, Credo outputs the given `halt_message`.\n\n    defmodule FooTask do\n      use Credo.Execution.Task\n\n      def call(exec) do\n        Execution.halt(exec, \"Execution has been halted!\")\n      end\n    end","ref":"Credo.Execution.html#halt/2"},{"type":"function","title":"Credo.Execution.put_assign/3","doc":"Puts the given `value` with the given `name` as assign into the given `exec` struct and returns the struct.\n\n    Credo.Execution.put_assign(exec, \"foo\", 42)\n    # => %Credo.Execution{...}","ref":"Credo.Execution.html#put_assign/3"},{"type":"function","title":"Credo.Execution.put_issues/2","doc":"Sets the issues for the given `exec` struct, overwriting any existing issues.","ref":"Credo.Execution.html#put_issues/2"},{"type":"function","title":"Credo.Execution.put_pipeline/3","doc":"Puts a given `pipeline` in `exec` under `pipeline_key`.\n\nA pipeline is a keyword list of named groups. Each named group is a list of `Credo.Execution.Task` modules:\n\n    Execution.put_pipeline(exec, :my_pipeline_key,\n      load_things: [ MyProject.LoadThings ],\n      run_analysis: [ MyProject.Run ],\n      print_results: [ MyProject.PrintResults ]\n    )\n\nA named group can also be a list of two-element tuples, consisting of a `Credo.Execution.Task` module and a\nkeyword list of options, which are passed to the Task module's `call/2` function:\n\n    Execution.put_pipeline(exec, :my_pipeline_key,\n      load_things: [ {MyProject.LoadThings, []} ],\n      run_analysis: [ {MyProject.Run, [foo: \"bar\"]} ],\n      print_results: [ {MyProject.PrintResults, []} ]\n    )","ref":"Credo.Execution.html#put_pipeline/3"},{"type":"function","title":"Credo.Execution.put_result/3","doc":"Puts the given `value` with the given `name` as result into the given `exec` struct.\n\n    Credo.Execution.put_result(exec, \"foo\", 42)\n    # => %Credo.Execution{...}","ref":"Credo.Execution.html#put_result/3"},{"type":"function","title":"Credo.Execution.run_pipeline/2","doc":"Runs the pipeline with the given `pipeline_key` and returns the result `Credo.Execution` struct.\n\n    Execution.run_pipeline(exec, :my_pipeline_key)\n    # => %Credo.Execution{...}","ref":"Credo.Execution.html#run_pipeline/2"},{"type":"function","title":"Credo.Execution.set_strict/1","doc":"Sets the exec values which `strict` implies (if applicable).","ref":"Credo.Execution.html#set_strict/1"},{"type":"function","title":"Credo.Execution.tags_for_check/2","doc":"Returns the tags for a given `check` and its `params`.","ref":"Credo.Execution.html#tags_for_check/2"},{"type":"module","title":"Credo.Issue","doc":"`Issue` structs represent all issues found during the code analysis.","ref":"Credo.Issue.html"},{"type":"type","title":"Credo.Issue.t/0","doc":"","ref":"Credo.Issue.html#t:t/0"},{"type":"module","title":"Credo.IssueMeta","doc":"IssueMeta provides helper functions for meta information which a check wants\nto pass to the `issue_for(...)` function, i.e. the current SourceFile and check\nparams (by default).","ref":"Credo.IssueMeta.html"},{"type":"function","title":"Credo.IssueMeta.for/2","doc":"Returns an issue meta tuple for the given `source_file` and `check_params`.","ref":"Credo.IssueMeta.html#for/2"},{"type":"function","title":"Credo.IssueMeta.params/1","doc":"Returns the check params for the given `issue_meta`.","ref":"Credo.IssueMeta.html#params/1"},{"type":"function","title":"Credo.IssueMeta.source_file/1","doc":"Returns the source file for the given `issue_meta`.","ref":"Credo.IssueMeta.html#source_file/1"},{"type":"type","title":"Credo.IssueMeta.t/0","doc":"","ref":"Credo.IssueMeta.html#t:t/0"},{"type":"module","title":"Credo.SourceFile","doc":"`SourceFile` structs represent a source file in the codebase.","ref":"Credo.SourceFile.html"},{"type":"function","title":"Credo.SourceFile.ast/1","doc":"Returns the AST for the given `source_file`.","ref":"Credo.SourceFile.html#ast/1"},{"type":"function","title":"Credo.SourceFile.column/3","doc":"Returns the column of the given `trigger` inside the given line.\n\nNOTE: Both `line_no` and the returned index are 1-based.","ref":"Credo.SourceFile.html#column/3"},{"type":"function","title":"Credo.SourceFile.line_at/2","doc":"Returns the line at the given `line_no`.\n\nNOTE: `line_no` is a 1-based index.","ref":"Credo.SourceFile.html#line_at/2"},{"type":"function","title":"Credo.SourceFile.line_at/4","doc":"Returns the snippet at the given `line_no` between `column1` and `column2`.\n\nNOTE: `line_no` is a 1-based index.","ref":"Credo.SourceFile.html#line_at/4"},{"type":"function","title":"Credo.SourceFile.lines/1","doc":"Returns the lines of source code for the given `source_file`.","ref":"Credo.SourceFile.html#lines/1"},{"type":"function","title":"Credo.SourceFile.parse/2","doc":"Returns a `SourceFile` struct for the given `source` code and `filename`.","ref":"Credo.SourceFile.html#parse/2"},{"type":"function","title":"Credo.SourceFile.source/1","doc":"Returns the source code for the given `source_file`.","ref":"Credo.SourceFile.html#source/1"},{"type":"function","title":"Credo.SourceFile.source_and_filename/2","doc":"Returns the source code and filename for the given `source_file_or_source`.","ref":"Credo.SourceFile.html#source_and_filename/2"},{"type":"function","title":"Credo.SourceFile.timed_out/1","doc":"","ref":"Credo.SourceFile.html#timed_out/1"},{"type":"type","title":"Credo.SourceFile.t/0","doc":"","ref":"Credo.SourceFile.html#t:t/0"},{"type":"module","title":"Credo.Code","doc":"`Credo.Code` contains a lot of utility or helper functions that deal with the\nanalysis of - you guessed it - code.\n\nWhenever a function serves a general purpose in this area, e.g. getting the\nvalue of a module attribute inside a given module, we want to extract that\nfunction and put it in the `Credo.Code` namespace, so others can utilize them\nwithout reinventing the wheel.","ref":"Credo.Code.html"},{"type":"function","title":"Credo.Code.ast/1","doc":"Returns an AST for a given `String` or `Credo.SourceFile`.","ref":"Credo.Code.html#ast/1"},{"type":"function","title":"Credo.Code.clean_charlists_strings_and_sigils/1","doc":"Takes a SourceFile and returns its source code stripped of all Strings and\nSigils.","ref":"Credo.Code.html#clean_charlists_strings_and_sigils/1"},{"type":"function","title":"Credo.Code.clean_charlists_strings_sigils_and_comments/2","doc":"Takes a SourceFile and returns its source code stripped of all Strings, Sigils\nand code comments.","ref":"Credo.Code.html#clean_charlists_strings_sigils_and_comments/2"},{"type":"function","title":"Credo.Code.contains_child?/2","doc":"Returns true if the given `child` AST node is part of the larger\n`parent` AST node.","ref":"Credo.Code.html#contains_child?/2"},{"type":"function","title":"Credo.Code.postwalk/3","doc":"Postwalks a given `Credo.SourceFile`'s AST or a given AST.\n\nTechnically this is just a wrapper around `Macro.postwalk/3`.","ref":"Credo.Code.html#postwalk/3"},{"type":"function","title":"Credo.Code.prewalk/3","doc":"Prewalks a given `Credo.SourceFile`'s AST or a given AST.\n\nTechnically this is just a wrapper around `Macro.prewalk/3`.","ref":"Credo.Code.html#prewalk/3"},{"type":"function","title":"Credo.Code.remove_metadata/1","doc":"Returns an AST without its metadata.","ref":"Credo.Code.html#remove_metadata/1"},{"type":"function","title":"Credo.Code.to_lines/1","doc":"Converts a String or `Credo.SourceFile` into a List of tuples of `{line_no, line}`.","ref":"Credo.Code.html#to_lines/1"},{"type":"function","title":"Credo.Code.to_tokens/1","doc":"Converts a String or `Credo.SourceFile` into a List of tokens using the `:elixir_tokenizer`.","ref":"Credo.Code.html#to_tokens/1"},{"type":"function","title":"Credo.Code.to_tokens/2","doc":"","ref":"Credo.Code.html#to_tokens/2"},{"type":"module","title":"Credo.Code.Block","doc":"This module provides helper functions to analyse blocks, e.g. the block taken\nby the `if` macro.","ref":"Credo.Code.Block.html"},{"type":"function","title":"Credo.Code.Block.after_block?/1","doc":"Returns true if the given `ast` has an after block.","ref":"Credo.Code.Block.html#after_block?/1"},{"type":"function","title":"Credo.Code.Block.after_block_for/1","doc":"Returns a tuple {:ok, after_block} or nil for a given AST node.","ref":"Credo.Code.Block.html#after_block_for/1"},{"type":"function","title":"Credo.Code.Block.after_block_for!/1","doc":"Returns the after: block of a given AST node.","ref":"Credo.Code.Block.html#after_block_for!/1"},{"type":"function","title":"Credo.Code.Block.all_blocks_for!/1","doc":"Returns the do: block of a given AST node.","ref":"Credo.Code.Block.html#all_blocks_for!/1"},{"type":"function","title":"Credo.Code.Block.calls_in_catch_block/1","doc":"Returns the children of the `catch` block of the given AST node.","ref":"Credo.Code.Block.html#calls_in_catch_block/1"},{"type":"function","title":"Credo.Code.Block.calls_in_do_block/1","doc":"Returns the children of the `do` block of the given AST node.","ref":"Credo.Code.Block.html#calls_in_do_block/1"},{"type":"function","title":"Credo.Code.Block.calls_in_rescue_block/1","doc":"Returns the children of the `rescue` block of the given AST node.","ref":"Credo.Code.Block.html#calls_in_rescue_block/1"},{"type":"function","title":"Credo.Code.Block.catch_block?/1","doc":"Returns true if the given `ast` has an catch block.","ref":"Credo.Code.Block.html#catch_block?/1"},{"type":"function","title":"Credo.Code.Block.catch_block_for/1","doc":"Returns a tuple {:ok, catch_block} or nil for a given AST node.","ref":"Credo.Code.Block.html#catch_block_for/1"},{"type":"function","title":"Credo.Code.Block.catch_block_for!/1","doc":"Returns the catch: block of a given AST node.","ref":"Credo.Code.Block.html#catch_block_for!/1"},{"type":"function","title":"Credo.Code.Block.do_block?/1","doc":"Returns true if the given `ast` has a do block.","ref":"Credo.Code.Block.html#do_block?/1"},{"type":"function","title":"Credo.Code.Block.do_block_for/1","doc":"Returns a tuple {:ok, do_block} or nil for a given AST node.","ref":"Credo.Code.Block.html#do_block_for/1"},{"type":"function","title":"Credo.Code.Block.do_block_for!/1","doc":"Returns the do: block of a given AST node.","ref":"Credo.Code.Block.html#do_block_for!/1"},{"type":"function","title":"Credo.Code.Block.else_block?/1","doc":"Returns true if the given `ast` has an else block.","ref":"Credo.Code.Block.html#else_block?/1"},{"type":"function","title":"Credo.Code.Block.else_block_for/1","doc":"Returns a tuple {:ok, else_block} or nil for a given AST node.","ref":"Credo.Code.Block.html#else_block_for/1"},{"type":"function","title":"Credo.Code.Block.else_block_for!/1","doc":"Returns the `else` block of a given AST node.","ref":"Credo.Code.Block.html#else_block_for!/1"},{"type":"function","title":"Credo.Code.Block.rescue_block?/1","doc":"Returns true if the given `ast` has an rescue block.","ref":"Credo.Code.Block.html#rescue_block?/1"},{"type":"function","title":"Credo.Code.Block.rescue_block_for/1","doc":"Returns a tuple {:ok, rescue_block} or nil for a given AST node.","ref":"Credo.Code.Block.html#rescue_block_for/1"},{"type":"function","title":"Credo.Code.Block.rescue_block_for!/1","doc":"Returns the rescue: block of a given AST node.","ref":"Credo.Code.Block.html#rescue_block_for!/1"},{"type":"module","title":"Credo.Code.Charlists","doc":"This module lets you strip charlists from source code.","ref":"Credo.Code.Charlists.html"},{"type":"function","title":"Credo.Code.Charlists.replace_with_spaces/5","doc":"Replaces all characters inside charlists with the equivalent amount of\nwhite-space.","ref":"Credo.Code.Charlists.html#replace_with_spaces/5"},{"type":"module","title":"Credo.Code.Heredocs","doc":"This module lets you strip heredocs from source code.","ref":"Credo.Code.Heredocs.html"},{"type":"function","title":"Credo.Code.Heredocs.replace_with_spaces/5","doc":"Replaces all characters inside heredocs\nwith the equivalent amount of white-space.","ref":"Credo.Code.Heredocs.html#replace_with_spaces/5"},{"type":"module","title":"Credo.Code.Module","doc":"This module provides helper functions to analyse modules, return the defined\nfunctions or module attributes.","ref":"Credo.Code.Module.html"},{"type":"function","title":"Credo.Code.Module.aliases/1","doc":"Returns the list of aliases defined in a given module source code.","ref":"Credo.Code.Module.html#aliases/1"},{"type":"function","title":"Credo.Code.Module.analyze/1","doc":"","ref":"Credo.Code.Module.html#analyze/1"},{"type":"function","title":"Credo.Code.Module.attribute/2","doc":"Reads an attribute from a module's `ast`","ref":"Credo.Code.Module.html#attribute/2"},{"type":"function","title":"Credo.Code.Module.def_arity/1","doc":"Returns the arity of the given function definition `ast`","ref":"Credo.Code.Module.html#def_arity/1"},{"type":"function","title":"Credo.Code.Module.def_count/1","doc":"Returns the function/macro count for the given module's AST","ref":"Credo.Code.Module.html#def_count/1"},{"type":"function","title":"Credo.Code.Module.def_name/1","doc":"Returns the name of the function/macro defined in the given `ast`","ref":"Credo.Code.Module.html#def_name/1"},{"type":"function","title":"Credo.Code.Module.def_name_with_op/1","doc":"Returns the {fun_name, op} tuple of the function/macro defined in the given `ast`","ref":"Credo.Code.Module.html#def_name_with_op/1"},{"type":"function","title":"Credo.Code.Module.def_name_with_op/2","doc":"","ref":"Credo.Code.Module.html#def_name_with_op/2"},{"type":"function","title":"Credo.Code.Module.def_names/1","doc":"Returns the name of the functions/macros for the given module's `ast`","ref":"Credo.Code.Module.html#def_names/1"},{"type":"function","title":"Credo.Code.Module.def_names_with_op/1","doc":"Returns the name of the functions/macros for the given module's `ast`","ref":"Credo.Code.Module.html#def_names_with_op/1"},{"type":"function","title":"Credo.Code.Module.def_names_with_op/2","doc":"Returns the name of the functions/macros for the given module's `ast` if it has the given `arity`.","ref":"Credo.Code.Module.html#def_names_with_op/2"},{"type":"function","title":"Credo.Code.Module.defs/1","doc":"","ref":"Credo.Code.Module.html#defs/1"},{"type":"function","title":"Credo.Code.Module.exception?/1","doc":"","ref":"Credo.Code.Module.html#exception?/1"},{"type":"function","title":"Credo.Code.Module.modules/1","doc":"Returns the list of modules used in a given module source code.","ref":"Credo.Code.Module.html#modules/1"},{"type":"function","title":"Credo.Code.Module.name/1","doc":"Returns the name of a module's given ast node.","ref":"Credo.Code.Module.html#name/1"},{"type":"type","title":"Credo.Code.Module.location/0","doc":"","ref":"Credo.Code.Module.html#t:location/0"},{"type":"type","title":"Credo.Code.Module.module_part/0","doc":"","ref":"Credo.Code.Module.html#t:module_part/0"},{"type":"module","title":"Credo.Code.Name","doc":"This module provides helper functions to process names of functions, module\nattributes and modules.","ref":"Credo.Code.Name.html"},{"type":"function","title":"Credo.Code.Name.first/1","doc":"","ref":"Credo.Code.Name.html#first/1"},{"type":"function","title":"Credo.Code.Name.full/1","doc":"","ref":"Credo.Code.Name.html#full/1"},{"type":"function","title":"Credo.Code.Name.last/1","doc":"","ref":"Credo.Code.Name.html#last/1"},{"type":"function","title":"Credo.Code.Name.no_case?/1","doc":"","ref":"Credo.Code.Name.html#no_case?/1"},{"type":"function","title":"Credo.Code.Name.parts_count/1","doc":"","ref":"Credo.Code.Name.html#parts_count/1"},{"type":"function","title":"Credo.Code.Name.pascal_case?/1","doc":"","ref":"Credo.Code.Name.html#pascal_case?/1"},{"type":"function","title":"Credo.Code.Name.snake_case?/1","doc":"","ref":"Credo.Code.Name.html#snake_case?/1"},{"type":"function","title":"Credo.Code.Name.snake_case?/2","doc":"","ref":"Credo.Code.Name.html#snake_case?/2"},{"type":"function","title":"Credo.Code.Name.split_pascal_case/1","doc":"","ref":"Credo.Code.Name.html#split_pascal_case/1"},{"type":"module","title":"Credo.Code.Parameters","doc":"This module provides helper functions to analyse the parameters taken by a\nfunction.","ref":"Credo.Code.Parameters.html"},{"type":"function","title":"Credo.Code.Parameters.count/1","doc":"Returns the parameter count for the given function's AST","ref":"Credo.Code.Parameters.html#count/1"},{"type":"function","title":"Credo.Code.Parameters.names/1","doc":"Returns the names of all parameters for the given function's AST","ref":"Credo.Code.Parameters.html#names/1"},{"type":"module","title":"Credo.Code.ParserError","doc":"This is an internal `Issue` raised by Credo when it finds itself unable to\nparse the source code in a file.","ref":"Credo.Code.ParserError.html"},{"type":"module","title":"Credo.Code.Scope","doc":"This module provides helper functions to determine the scope name at a certain\npoint in the analysed code.","ref":"Credo.Code.Scope.html"},{"type":"function","title":"Credo.Code.Scope.mod_name/1","doc":"Returns the module part of a scope.\n\n    iex> Credo.Code.Scope.mod_name(\"Credo.Code\")\n    \"Credo.Code\"\n\n    iex> Credo.Code.Scope.mod_name(\"Credo.Code.ast\")\n    \"Credo.Code\"","ref":"Credo.Code.Scope.html#mod_name/1"},{"type":"function","title":"Credo.Code.Scope.name/2","doc":"Returns the scope for the given line as a tuple consisting of the call to\ndefine the scope (`:defmodule`, `:def`, `:defp` or `:defmacro`) and the\nname of the scope.\n\nExamples:\n\n    {:defmodule, \"Foo.Bar\"}\n    {:def, \"Foo.Bar.baz\"}","ref":"Credo.Code.Scope.html#name/2"},{"type":"module","title":"Credo.Code.Sigils","doc":"This module lets you strip sigils from source code.","ref":"Credo.Code.Sigils.html"},{"type":"function","title":"Credo.Code.Sigils.replace_with_spaces/5","doc":"Replaces all characters inside all sigils with the equivalent amount of\nwhite-space.","ref":"Credo.Code.Sigils.html#replace_with_spaces/5"},{"type":"module","title":"Credo.Code.Strings","doc":"This module lets you strip strings from source code.","ref":"Credo.Code.Strings.html"},{"type":"function","title":"Credo.Code.Strings.replace_with_spaces/5","doc":"Replaces all characters inside string literals and string sigils\nwith the equivalent amount of white-space.","ref":"Credo.Code.Strings.html#replace_with_spaces/5"},{"type":"module","title":"Credo.Code.Token","doc":"This module provides helper functions to analyse tokens returned by `Credo.Code.to_tokens/1`.","ref":"Credo.Code.Token.html"},{"type":"function","title":"Credo.Code.Token.eol?/1","doc":"Returns `true` if the given `token` contains a line break.","ref":"Credo.Code.Token.html#eol?/1"},{"type":"function","title":"Credo.Code.Token.position/1","doc":"Returns the position of a token in the form\n\n    {line_no_start, col_start, line_no_end, col_end}","ref":"Credo.Code.Token.html#position/1"},{"type":"function","title":"Credo.Code.Token.to_col_end/3","doc":"","ref":"Credo.Code.Token.html#to_col_end/3"},{"type":"module","title":"Credo.Code.TokenAstCorrelation","doc":"","ref":"Credo.Code.TokenAstCorrelation.html"},{"type":"function","title":"Credo.Code.TokenAstCorrelation.find_tokens_in_ast/2","doc":"","ref":"Credo.Code.TokenAstCorrelation.html#find_tokens_in_ast/2"},{"type":"module","title":"Credo.Test.Case","doc":"Conveniences for testing Credo custom checks and plugins.\n\nThis module can be used in your test cases, like this:\n\n    use Credo.Test.Case\n\nUsing this module will:\n\n* import all the functions from this module\n* make the test case `:async` by default (use `use Credo.Test.Case, async: false` to opt out)","ref":"Credo.Test.Case.html"},{"type":"module","title":"Testing custom checks - Credo.Test.Case","doc":"Suppose we have a custom check in our project that checks whether or not\nthe \"FooBar rules\" are applied (one of those *very* project-specific things).\n\n    defmodule MyProject.MyCustomChecks.FooBar do\n      use Credo.Check, category: :warning, base_priority: :high\n\n      def run(%SourceFile{} = source_file, params) do\n        # ... implement all the \"FooBar rules\" ...\n      end\n    end\n\nWhen we want to test this check, we can use `Credo.Test.Case` for convenience:\n\n    defmodule MyProject.MyCustomChecks.FooBarTest do\n      use Credo.Test.Case\n\n      alias MyProject.MyCustomChecks.FooBar\n\n      test \"it should NOT report expected code\" do\n        \"\"\"\n        defmodule CredoSampleModule do\n          # ... some good Elixir code ...\n        end\n        \"\"\"\n        |> to_source_file()\n        |> run_check(FooBar)\n        |> refute_issues()\n      end\n\n      test \"it should report code that violates the FooBar rule\" do\n        \"\"\"\n        defmodule CredoSampleModule do\n          # ... some Elixir code that violates the FooBar rule ...\n        end\n        \"\"\"\n        |> to_source_file()\n        |> run_check(FooBar)\n        |> assert_issues()\n      end\n    end\n\nThis is as simple and mundane as it looks (which is a good thing):\nWe have two tests: one for the good case, one for the bad case.\nIn each, we create a source file representation from a heredoc, run our custom check and assert/refute the issues\nwe expect.","ref":"Credo.Test.Case.html#module-testing-custom-checks"},{"type":"module","title":"Asserting found issues - Credo.Test.Case","doc":"Once we get to know domain a little better, we can add more tests, typically testing for other bad cases in which\nour check should produce issues.\n\nNote that there are two assertion functions for this: `assert_issue/2` and `assert_issues/2`, where the first one\nensures that there is a single issue and the second asserts that there are at least two issues.\n\nBoth functions take an optional `callback` as their second parameter, which is called with the `issue` or the\nlist of `issues` found, which makes it convenient  to check for the issues properties ...\n\n    \"\"\"\n    # ... any Elixir code ...\n    \"\"\"\n    |> to_source_file()\n    |> run_check(FooBar)\n    |> assert_issue(fn issue -> assert issue.trigger == \"foo\" end)\n\n... or properties of the list of issues:\n\n    \"\"\"\n    # ... any Elixir code ...\n    \"\"\"\n    |> to_source_file()\n    |> run_check(FooBar)\n    |> assert_issue(fn issues -> assert Enum.count(issues) == 3 end)","ref":"Credo.Test.Case.html#module-asserting-found-issues"},{"type":"module","title":"Testing checks that analyse multiple source files - Credo.Test.Case","doc":"For checks that analyse multiple source files, like Credo's consistency checks, we can use `to_source_files/1` to\ncreate\n\n    [\n      \"\"\"\n      # source file 1\n      \"\"\",\n      \"\"\"\n      # source file 2\n      \"\"\"\n    ]\n    |> to_source_files()\n    |> run_check(FooBar)\n    |> refute_issues()\n\nIf our check needs named source files, we can always use `to_source_file/2` to create individually named source\nfiles and combine them into a list:\n\n    source_file1 =\n      \"\"\"\n      # source file 1\n      \"\"\"\n      |> to_source_file(\"foo.ex\")\n\n    source_file2 =\n      \"\"\"\n      # source file 2\n      \"\"\"\n      |> to_source_file(\"bar.ex\")\n\n    [source_file1, source_file2]\n    |> run_check(FooBar)\n    |> assert_issue(fn issue -> assert issue.filename == \"foo.ex\" end)","ref":"Credo.Test.Case.html#module-testing-checks-that-analyse-multiple-source-files"},{"type":"function","title":"Credo.Test.Case.assert_issue/2","doc":"Asserts the presence of a single issue.","ref":"Credo.Test.Case.html#assert_issue/2"},{"type":"function","title":"Credo.Test.Case.assert_issues/2","doc":"Asserts the presence of more than one issue.","ref":"Credo.Test.Case.html#assert_issues/2"},{"type":"function","title":"Credo.Test.Case.refute_issues/1","doc":"Refutes the presence of any issues.","ref":"Credo.Test.Case.html#refute_issues/1"},{"type":"function","title":"Credo.Test.Case.run_check/3","doc":"Runs the given `check` on the given `source_file` using the given `params`.\n\n    \"x = 5\"\n    |> to_source_file()\n    |> run_check(MyProject.MyCheck, foo_parameter: \"bar\")","ref":"Credo.Test.Case.html#run_check/3"},{"type":"function","title":"Credo.Test.Case.to_source_file/1","doc":"Converts the given `source` string to a `%SourceFile{}`.\n\n    \"x = 5\"\n    |> to_source_file()","ref":"Credo.Test.Case.html#to_source_file/1"},{"type":"function","title":"Credo.Test.Case.to_source_file/2","doc":"Converts the given `source` string to a `%SourceFile{}` with the given `filename`.\n\n    \"x = 5\"\n    |> to_source_file(\"simple.ex\")","ref":"Credo.Test.Case.html#to_source_file/2"},{"type":"function","title":"Credo.Test.Case.to_source_files/1","doc":"Converts the given `list` of source code strings to a list of `%SourceFile{}` structs.\n\n    [\"x = 5\", \"y = 6\"]\n    |> to_source_files()","ref":"Credo.Test.Case.html#to_source_files/1"},{"type":"module","title":"Credo.Test.CheckRunner","doc":"","ref":"Credo.Test.CheckRunner.html"},{"type":"function","title":"Credo.Test.CheckRunner.run_check/3","doc":"","ref":"Credo.Test.CheckRunner.html#run_check/3"},{"type":"module","title":"Credo.Test.SourceFiles","doc":"","ref":"Credo.Test.SourceFiles.html"},{"type":"function","title":"Credo.Test.SourceFiles.to_source_file/1","doc":"","ref":"Credo.Test.SourceFiles.html#to_source_file/1"},{"type":"function","title":"Credo.Test.SourceFiles.to_source_file/2","doc":"","ref":"Credo.Test.SourceFiles.html#to_source_file/2"},{"type":"function","title":"Credo.Test.SourceFiles.to_source_files/1","doc":"","ref":"Credo.Test.SourceFiles.html#to_source_files/1"},{"type":"module","title":"Credo.Check.ConfigComment","doc":"`ConfigComment` structs represent comments which follow control Credo's behaviour.\n\nThe following comments are supported:\n\n    # credo:disable-for-this-file\n    # credo:disable-for-next-line\n    # credo:disable-for-previous-line\n    # credo:disable-for-lines:","ref":"Credo.Check.ConfigComment.html"},{"type":"function","title":"Credo.Check.ConfigComment.ignores_issue?/2","doc":"Returns `true` if the given `issue` should be ignored based on the given `config_comment`","ref":"Credo.Check.ConfigComment.html#ignores_issue?/2"},{"type":"function","title":"Credo.Check.ConfigComment.new/3","doc":"Returns a `ConfigComment` struct based on the given parameters.","ref":"Credo.Check.ConfigComment.html#new/3"},{"type":"module","title":"Credo.Check.Params","doc":"This module provides functions for handling parameters (\"params\") given to\nchecks through `.credo.exs` (i.e. the `Credo.ConfigFile`).","ref":"Credo.Check.Params.html"},{"type":"function","title":"Credo.Check.Params.get/3","doc":"Returns the given `field`'s `params` value.\n\nExample:\n\n    defmodule SamepleCheck do\n      def param_defaults do\n        [foo: \"bar\"]\n      end\n    end\n\n    iex> Credo.Check.Params.get([], :foo, SamepleCheck)\n    \"bar\"\n    iex> Credo.Check.Params.get([foo: \"baz\"], :foo, SamepleCheck)\n    \"baz\"","ref":"Credo.Check.Params.html#get/3"},{"type":"module","title":"Credo.Check.Design.AliasUsage","doc":"This check has a base priority of `normal` and works with any version of Elixir.","ref":"Credo.Check.Design.AliasUsage.html"},{"type":"module","title":"Explanation - Credo.Check.Design.AliasUsage","doc":"Functions from other modules should be used via an alias if the module's\nnamespace is not top-level.\n\nWhile this is completely fine:\n\n    defmodule MyApp.Web.Search do\n      def twitter_mentions do\n        MyApp.External.TwitterAPI.search(...)\n      end\n    end\n\n... you might want to refactor it to look like this:\n\n    defmodule MyApp.Web.Search do\n      alias MyApp.External.TwitterAPI\n\n      def twitter_mentions do\n        TwitterAPI.search(...)\n      end\n    end\n\nThe thinking behind this is that you can see the dependencies of your module\nat a glance. So if you are attempting to build a medium to large project,\nthis can help you to get your boundaries/layers/contracts right.\n\nAs always: This is just a suggestion. Check the configuration options for\ntweaking or disabling this check.","ref":"Credo.Check.Design.AliasUsage.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Design.AliasUsage","doc":"Use the following parameters to configure this check:\n\n### `:excluded_namespaces`\n\n  List of namespaces to be excluded for this check.\n\n*This parameter defaults to* `[\"File\", \"IO\", \"Inspect\", \"Kernel\", \"Macro\", \"Supervisor\", \"Task\", \"Version\"]`.\n\n### `:excluded_lastnames`\n\n  List of lastnames to be excluded for this check.\n\n*This parameter defaults to* `[\"Access\", \"Agent\", \"Application\", \"Atom\", \"Base\", \"Behaviour\", \"Bitwise\", \"Code\", \"Date\", \"DateTime\", \"Dict\", \"Enum\", \"Exception\", \"File\", \"Float\", \"GenEvent\", \"GenServer\", \"HashDict\", \"HashSet\", \"Integer\", \"IO\", \"Kernel\", \"Keyword\", \"List\", \"Macro\", \"Map\", \"MapSet\", \"Module\", \"NaiveDateTime\", \"Node\", \"OptionParser\", \"Path\", \"Port\", \"Process\", \"Protocol\", \"Range\", \"Record\", \"Regex\", \"Registry\", \"Set\", \"Stream\", \"String\", \"StringIO\", \"Supervisor\", \"System\", \"Task\", \"Time\", \"Tuple\", \"URI\", \"Version\"]`.\n\n### `:if_nested_deeper_than`\n\n  Only raise an issue if a module is nested deeper than this.\n\n*This parameter defaults to* `0`.\n\n### `:if_called_more_often_than`\n\n  Only raise an issue if a module is called more often than this.\n\n*This parameter defaults to* `0`.\n\n### `:if_referenced`\n\n  Raise an issue if a module is referenced by name, e.g. as an argument in a function call.\n\n*This parameter defaults to* `false`.\n\n### `:only`\n\n  Regex or a list of regexes that specifies which modules to include for this check.\n  \n  `excluded_namespaces` and `excluded_lastnames` take precedence over this parameter.\n  \n\n*This parameter defaults to* `nil`.","ref":"Credo.Check.Design.AliasUsage.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Design.AliasUsage","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Design.AliasUsage.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Design.DuplicatedCode","doc":"This check has a base priority of `higher` and works with any version of Elixir.","ref":"Credo.Check.Design.DuplicatedCode.html"},{"type":"module","title":"Explanation - Credo.Check.Design.DuplicatedCode","doc":"Code should not be copy-pasted in a codebase when there is room to abstract\nthe copied functionality in a meaningful way.\n\nThat said, you should by no means \"ABSTRACT ALL THE THINGS!\".\n\nSometimes it can serve a purpose to have code be explicit in two places, even\nif it means the snippets are nearly identical. A good example for this are\nDatabase Adapters in a project like Ecto, where you might have nearly\nidentical functions for things like `order_by` or `limit` in both the\nPostgres and MySQL adapters.\n\nIn this case, introducing an `AbstractAdapter` just to avoid code duplication\nmight cause more trouble down the line than having a bit of duplicated code.\n\nLike all `Software Design` issues, this is just advice and might not be\napplicable to your project/situation.","ref":"Credo.Check.Design.DuplicatedCode.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Design.DuplicatedCode","doc":"Use the following parameters to configure this check:\n\n### `:mass_threshold`\n\n  The minimum mass which a part of code has to have to qualify for this check.\n\n*This parameter defaults to* `40`.\n\n### `:nodes_threshold`\n\n  The number of nodes that need to be found to raise an issue.\n\n*This parameter defaults to* `2`.\n\n### `:excluded_macros`\n\n  List of macros to be excluded for this check.\n\n*This parameter defaults to* `[]`.","ref":"Credo.Check.Design.DuplicatedCode.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Design.DuplicatedCode","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Design.DuplicatedCode.html#module-general-parameters"},{"type":"function","title":"Credo.Check.Design.DuplicatedCode.add_masses/1","doc":"","ref":"Credo.Check.Design.DuplicatedCode.html#add_masses/1"},{"type":"function","title":"Credo.Check.Design.DuplicatedCode.calculate_hashes/4","doc":"Calculates hash values for all sub nodes in a given +ast+.\n\nReturns a map with the hashes as keys and the nodes as values.","ref":"Credo.Check.Design.DuplicatedCode.html#calculate_hashes/4"},{"type":"function","title":"Credo.Check.Design.DuplicatedCode.create_issue?/2","doc":"","ref":"Credo.Check.Design.DuplicatedCode.html#create_issue?/2"},{"type":"function","title":"Credo.Check.Design.DuplicatedCode.line_no_for/1","doc":"","ref":"Credo.Check.Design.DuplicatedCode.html#line_no_for/1"},{"type":"function","title":"Credo.Check.Design.DuplicatedCode.mass/1","doc":"Returns the mass (count of instructions) for an AST.","ref":"Credo.Check.Design.DuplicatedCode.html#mass/1"},{"type":"function","title":"Credo.Check.Design.DuplicatedCode.prune_hashes/2","doc":"Takes a map of hashes to nodes and prunes those nodes that are just\nsubnodes of others in the same set.\n\nReturns the resulting map.","ref":"Credo.Check.Design.DuplicatedCode.html#prune_hashes/2"},{"type":"function","title":"Credo.Check.Design.DuplicatedCode.to_hash/1","doc":"Returns a hash-value for a given +ast+.","ref":"Credo.Check.Design.DuplicatedCode.html#to_hash/1"},{"type":"module","title":"Credo.Check.Design.SkipTestWithoutComment","doc":"This check has a base priority of `normal` and works with any version of Elixir.","ref":"Credo.Check.Design.SkipTestWithoutComment.html"},{"type":"module","title":"Explanation - Credo.Check.Design.SkipTestWithoutComment","doc":"Skipped tests should have a comment documenting why the test is skipped.\n\nTests are often skipped using `@tag :skip` when some issue arises that renders\nthe test temporarily broken or unable to run. This temporary skip often becomes\na permanent one because the reason for the test being skipped is not documented.\n\nA comment should exist on the line prior to the skip tag describing why the test is\nskipped.\n\nExample:\n\n    # john: skipping this since our credentials expired, working on getting new ones\n    @tag :skip\n    test \"vendor api returns data\" do\n      # ...\n    end\n\nWhile the pure existence of a comment does not change anything per se, a thoughtful\ncomment can improve the odds for future iteration on the issue.","ref":"Credo.Check.Design.SkipTestWithoutComment.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Design.SkipTestWithoutComment","doc":"*There are no specific parameters for this check.*","ref":"Credo.Check.Design.SkipTestWithoutComment.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Design.SkipTestWithoutComment","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Design.SkipTestWithoutComment.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Design.TagFIXME","doc":"This check has a base priority of `high` and works with any version of Elixir.","ref":"Credo.Check.Design.TagFIXME.html"},{"type":"module","title":"Explanation - Credo.Check.Design.TagFIXME","doc":"FIXME comments are used to indicate places where source code needs fixing.\n\nExample:\n\n    # FIXME: this does no longer work, research new API url\n    defp fun do\n      # ...\n    end\n\nThe premise here is that FIXME should indeed be fixed as soon as possible and\nare therefore reported by Credo.\n\nLike all `Software Design` issues, this is just advice and might not be\napplicable to your project/situation.","ref":"Credo.Check.Design.TagFIXME.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Design.TagFIXME","doc":"Use the following parameters to configure this check:\n\n### `:include_doc`\n\n  Set to `true` to also include tags from @doc attributes.\n\n*This parameter defaults to* `true`.","ref":"Credo.Check.Design.TagFIXME.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Design.TagFIXME","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Design.TagFIXME.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Design.TagTODO","doc":"This check has a base priority of `0` and works with any version of Elixir.","ref":"Credo.Check.Design.TagTODO.html"},{"type":"module","title":"Explanation - Credo.Check.Design.TagTODO","doc":"TODO comments are used to remind yourself of source code related things.\n\nExample:\n\n    # TODO: move this to a Helper module\n    defp fun do\n      # ...\n    end\n\nThe premise here is that TODO should be dealt with in the near future and\nare therefore reported by Credo.\n\nLike all `Software Design` issues, this is just advice and might not be\napplicable to your project/situation.","ref":"Credo.Check.Design.TagTODO.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Design.TagTODO","doc":"Use the following parameters to configure this check:\n\n### `:include_doc`\n\n  Set to `true` to also include tags from @doc attributes.\n\n*This parameter defaults to* `true`.","ref":"Credo.Check.Design.TagTODO.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Design.TagTODO","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Design.TagTODO.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Readability.AliasAs","doc":"This check has a base priority of `low` and works with any version of Elixir.","ref":"Credo.Check.Readability.AliasAs.html"},{"type":"module","title":"Explanation - Credo.Check.Readability.AliasAs","doc":"Aliases which are not completely renamed using the `:as` option are easier to follow.\n\n    # preferred\n\n    def MyModule do\n      alias MyApp.Module1\n\n      def my_function(foo) do\n        Module1.run(foo)\n      end\n    end\n\n    # NOT preferred\n\n    def MyModule do\n      alias MyApp.Module1, as: M1\n\n      def my_function(foo) do\n        # what the heck is `M1`?\n        M1.run(foo)\n      end\n    end\n\nLike all `Readability` issues, this one is not a technical concern.\nBut you can improve the odds of others reading and liking your code by making\nit easier to follow.","ref":"Credo.Check.Readability.AliasAs.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Readability.AliasAs","doc":"*There are no specific parameters for this check.*","ref":"Credo.Check.Readability.AliasAs.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Readability.AliasAs","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Readability.AliasAs.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Readability.AliasOrder","doc":"This check has a base priority of `low` and works with any version of Elixir.","ref":"Credo.Check.Readability.AliasOrder.html"},{"type":"module","title":"Explanation - Credo.Check.Readability.AliasOrder","doc":"Alphabetically ordered lists are more easily scannable by the reader.\n\n    # preferred\n\n    alias ModuleA\n    alias ModuleB\n    alias ModuleC\n\n    # NOT preferred\n\n    alias ModuleA\n    alias ModuleC\n    alias ModuleB\n\nAlias should be alphabetically ordered among their group:\n\n    # preferred\n\n    alias ModuleC\n    alias ModuleD\n\n    alias ModuleA\n    alias ModuleB\n\n    # NOT preferred\n\n    alias ModuleC\n    alias ModuleD\n\n    alias ModuleB\n    alias ModuleA\n\nLike all `Readability` issues, this one is not a technical concern.\nBut you can improve the odds of others reading and liking your code by making\nit easier to follow.","ref":"Credo.Check.Readability.AliasOrder.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Readability.AliasOrder","doc":"*There are no specific parameters for this check.*","ref":"Credo.Check.Readability.AliasOrder.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Readability.AliasOrder","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Readability.AliasOrder.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Readability.BlockPipe","doc":"This check has a base priority of `0` and works with any version of Elixir.","ref":"Credo.Check.Readability.BlockPipe.html"},{"type":"module","title":"Explanation - Credo.Check.Readability.BlockPipe","doc":"Pipes (`|>`) should not be used with blocks.\n\nThe code in this example ...\n\n    list\n    |> Enum.take(5)\n    |> Enum.sort()\n    |> case do\n      [[_h | _t] | _] -> true\n      _ -> false\n    end\n\n... should be refactored to look like this:\n\n    maybe_nested_lists =\n      list\n      |> Enum.take(5)\n      |> Enum.sort()\n\n    case maybe_nested_lists do\n      [[_h | _t] | _] -> true\n      _ -> false\n    end\n\n... or create a new function:\n\n    list\n    |> Enum.take(5)\n    |> Enum.sort()\n    |> contains_nested_list?()\n\nPiping to blocks may be harder to read because it can be said that it obscures intentions\nand increases cognitive load on the reader. Instead, prefer introducing variables to your code or\nnew functions when it may be a sign that your function is getting too complicated and/or has too many concerns.\n\nLike all `Readability` issues, this one is not a technical concern, but you can improve the odds of others reading\nand understanding the intent of your code by making it easier to follow.","ref":"Credo.Check.Readability.BlockPipe.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Readability.BlockPipe","doc":"Use the following parameters to configure this check:\n\n### `:exclude`\n\n  Do not raise an issue for these macros and functions.\n\n*This parameter defaults to* `[]`.","ref":"Credo.Check.Readability.BlockPipe.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Readability.BlockPipe","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Readability.BlockPipe.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Readability.FunctionNames","doc":"This check has a base priority of `high` and works with any version of Elixir.","ref":"Credo.Check.Readability.FunctionNames.html"},{"type":"module","title":"Explanation - Credo.Check.Readability.FunctionNames","doc":"Function, macro, and guard names are always written in snake_case in Elixir.\n\n    # snake_case\n\n    def handle_incoming_message(message) do\n    end\n\n    # not snake_case\n\n    def handleIncomingMessage(message) do\n    end\n\nLike all `Readability` issues, this one is not a technical concern.\nBut you can improve the odds of others reading and liking your code by making\nit easier to follow.","ref":"Credo.Check.Readability.FunctionNames.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Readability.FunctionNames","doc":"Use the following parameters to configure this check:\n\n### `:allow_acronyms`\n\n  Allows acronyms like HTTP or OTP in function names.\n\n*This parameter defaults to* `false`.","ref":"Credo.Check.Readability.FunctionNames.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Readability.FunctionNames","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Readability.FunctionNames.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Readability.ImplTrue","doc":"This check has a base priority of `normal` and works with any version of Elixir.","ref":"Credo.Check.Readability.ImplTrue.html"},{"type":"module","title":"Explanation - Credo.Check.Readability.ImplTrue","doc":"When implementing behaviour callbacks, `@impl true` indicates that a function implements a callback, but\na better way is to note the actual behaviour being implemented, for example `@impl MyBehaviour`. This\nnot only improves readability, but adds extra validation in cases where multiple behaviours are implemented\nin a single module.\n\nInstead of:\n\n    @impl true\n    def my_funcion() do\n      ...\n\nuse:\n\n    @impl MyBehaviour\n    def my_funcion() do\n      ...","ref":"Credo.Check.Readability.ImplTrue.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Readability.ImplTrue","doc":"*There are no specific parameters for this check.*","ref":"Credo.Check.Readability.ImplTrue.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Readability.ImplTrue","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Readability.ImplTrue.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Readability.LargeNumbers","doc":"This check has a base priority of `high` and works with any version of Elixir.","ref":"Credo.Check.Readability.LargeNumbers.html"},{"type":"module","title":"Explanation - Credo.Check.Readability.LargeNumbers","doc":"Numbers can contain underscores for readability purposes.\nThese do not affect the value of the number, but can help read large numbers\nmore easily.\n\n    141592654 # how large is this number?\n\n    141_592_654 # ah, it's in the hundreds of millions!\n\nLike all `Readability` issues, this one is not a technical concern.\nBut you can improve the odds of others reading and liking your code by making\nit easier to follow.","ref":"Credo.Check.Readability.LargeNumbers.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Readability.LargeNumbers","doc":"Use the following parameters to configure this check:\n\n### `:only_greater_than`\n\n  The check only reports numbers greater than this.\n\n*This parameter defaults to* `9999`.\n\n### `:trailing_digits`\n\n  The check allows for the given number of trailing digits (can be a number, range or list)\n\n*This parameter defaults to* `[]`.","ref":"Credo.Check.Readability.LargeNumbers.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Readability.LargeNumbers","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Readability.LargeNumbers.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Readability.MaxLineLength","doc":"This check has a base priority of `low` and works with any version of Elixir.","ref":"Credo.Check.Readability.MaxLineLength.html"},{"type":"module","title":"Explanation - Credo.Check.Readability.MaxLineLength","doc":"Checks for the length of lines.\n\nIgnores function definitions and (multi-)line strings by default.","ref":"Credo.Check.Readability.MaxLineLength.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Readability.MaxLineLength","doc":"Use the following parameters to configure this check:\n\n### `:max_length`\n\n  The maximum number of characters a line may consist of.\n\n*This parameter defaults to* `120`.\n\n### `:ignore_definitions`\n\n  Set to `true` to ignore lines including function definitions.\n\n*This parameter defaults to* `true`.\n\n### `:ignore_specs`\n\n  Set to `true` to ignore lines including `@spec`s.\n\n*This parameter defaults to* `false`.\n\n### `:ignore_sigils`\n\n  Set to `true` to ignore lines that are sigils, e.g. regular expressions.\n\n*This parameter defaults to* `true`.\n\n### `:ignore_strings`\n\n  Set to `true` to ignore lines that are strings or in heredocs.\n\n*This parameter defaults to* `true`.\n\n### `:ignore_urls`\n\n  Set to `true` to ignore lines that contain urls.\n\n*This parameter defaults to* `true`.","ref":"Credo.Check.Readability.MaxLineLength.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Readability.MaxLineLength","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Readability.MaxLineLength.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Readability.ModuleAttributeNames","doc":"This check has a base priority of `high` and works with any version of Elixir.","ref":"Credo.Check.Readability.ModuleAttributeNames.html"},{"type":"module","title":"Explanation - Credo.Check.Readability.ModuleAttributeNames","doc":"Module attribute names are always written in snake_case in Elixir.\n\n# snake_case\n\n@inbox_name \"incoming\"\n\n# not snake_case\n\n@inboxName \"incoming\"\n\nLike all `Readability` issues, this one is not a technical concern.\nBut you can improve the odds of others reading and liking your code by making\nit easier to follow.","ref":"Credo.Check.Readability.ModuleAttributeNames.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Readability.ModuleAttributeNames","doc":"*There are no specific parameters for this check.*","ref":"Credo.Check.Readability.ModuleAttributeNames.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Readability.ModuleAttributeNames","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Readability.ModuleAttributeNames.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Readability.ModuleDoc","doc":"This check has a base priority of `0` and works with any version of Elixir.","ref":"Credo.Check.Readability.ModuleDoc.html"},{"type":"module","title":"Explanation - Credo.Check.Readability.ModuleDoc","doc":"Every module should contain comprehensive documentation.\n\n    # preferred\n\n    defmodule MyApp.Web.Search do\n      @moduledoc \"\"\"\n      This module provides a public API for all search queries originating\n      in the web layer.\n      \"\"\"\n    end\n\n    # also okay: explicitly say there is no documentation\n\n    defmodule MyApp.Web.Search do\n      @moduledoc false\n    end\n\nMany times a sentence or two in plain english, explaining why the module\nexists, will suffice. Documenting your train of thought this way will help\nboth your co-workers and your future-self.\n\nOther times you will want to elaborate even further and show some\nexamples of how the module's functions can and should be used.\n\nIn some cases however, you might not want to document things about a module,\ne.g. it is part of a private API inside your project. Since Elixir prefers\nexplicitness over implicit behaviour, you should \"tag\" these modules with\n\n    @moduledoc false\n\nto make it clear that there is no intention in documenting it.","ref":"Credo.Check.Readability.ModuleDoc.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Readability.ModuleDoc","doc":"Use the following parameters to configure this check:\n\n### `:ignore_names`\n\n  All modules matching this regex (or list of regexes) will be ignored.\n\n*This parameter defaults to* `[~r/(\\.\\w+Controller|\\.Endpoint|\\.\\w+Live(\\.\\w+)?|\\.Repo|\\.Router|\\.\\w+Socket|\\.\\w+View|\\.\\w+HTML|\\.\\w+JSON|\\.Telemetry|\\.Layouts|\\.Mailer)$/]`.","ref":"Credo.Check.Readability.ModuleDoc.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Readability.ModuleDoc","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Readability.ModuleDoc.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Readability.ModuleNames","doc":"This check has a base priority of `high` and works with any version of Elixir.","ref":"Credo.Check.Readability.ModuleNames.html"},{"type":"module","title":"Explanation - Credo.Check.Readability.ModuleNames","doc":"Module names are always written in PascalCase in Elixir.\n\n    # PascalCase\n\n    defmodule MyApp.WebSearchController do\n      # ...\n    end\n\n    # not PascalCase\n\n    defmodule MyApp.Web_searchController do\n      # ...\n    end\n\nLike all `Readability` issues, this one is not a technical concern.\nBut you can improve the odds of other reading and liking your code by making\nit easier to follow.","ref":"Credo.Check.Readability.ModuleNames.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Readability.ModuleNames","doc":"Use the following parameters to configure this check:\n\n### `:ignore`\n\n  List of ignored module names and patterns e.g. `[~r/Sample_Module/, \"Credo.Sample_Module\"]`\n\n*This parameter defaults to* `[]`.","ref":"Credo.Check.Readability.ModuleNames.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Readability.ModuleNames","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Readability.ModuleNames.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Readability.MultiAlias","doc":"This check has a base priority of `low` and works with any version of Elixir.","ref":"Credo.Check.Readability.MultiAlias.html"},{"type":"module","title":"Explanation - Credo.Check.Readability.MultiAlias","doc":"Multi alias expansion makes module uses harder to search for in large code bases.\n\n    # preferred\n\n    alias Module.Foo\n    alias Module.Bar\n\n    # NOT preferred\n\n    alias Module.{Foo, Bar}\n\nLike all `Readability` issues, this one is not a technical concern.\nBut you can improve the odds of others reading and liking your code by making\nit easier to follow.","ref":"Credo.Check.Readability.MultiAlias.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Readability.MultiAlias","doc":"*There are no specific parameters for this check.*","ref":"Credo.Check.Readability.MultiAlias.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Readability.MultiAlias","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Readability.MultiAlias.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Readability.NestedFunctionCalls","doc":"This check has a base priority of `0` and works with any version of Elixir.","ref":"Credo.Check.Readability.NestedFunctionCalls.html"},{"type":"module","title":"Explanation - Credo.Check.Readability.NestedFunctionCalls","doc":"A function call should not be nested inside another function call.\n\nSo while this is fine:\n\n    Enum.shuffle([1,2,3])\n\nThe code in this example ...\n\n    Enum.shuffle(Enum.uniq([1,2,3,3]))\n\n... should be refactored to look like this:\n\n    [1,2,3,3]\n    |> Enum.uniq()\n    |> Enum.shuffle()\n\nNested function calls make the code harder to read. Instead, break the\nfunction calls out into a pipeline.\n\nLike all `Readability` issues, this one is not a technical concern.\nBut you can improve the odds of others reading and liking your code by making\nit easier to follow.","ref":"Credo.Check.Readability.NestedFunctionCalls.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Readability.NestedFunctionCalls","doc":"Use the following parameters to configure this check:\n\n### `:min_pipeline_length`\n\n  Set a minimum pipeline length\n\n*This parameter defaults to* `2`.","ref":"Credo.Check.Readability.NestedFunctionCalls.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Readability.NestedFunctionCalls","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Readability.NestedFunctionCalls.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Readability.NestedFunctionCalls.PipeHelper","doc":"This module exists to contain logic for the cannot_be_in_pipline?/3 helper\nfunction. This function was originally copied from the\nCredo.Check.Refactor.PipeChainStart module's valid_chain_start?/3 function.\nBoth functions are identical.","ref":"Credo.Check.Readability.NestedFunctionCalls.PipeHelper.html"},{"type":"function","title":"Credo.Check.Readability.NestedFunctionCalls.PipeHelper.cannot_be_in_pipeline?/3","doc":"","ref":"Credo.Check.Readability.NestedFunctionCalls.PipeHelper.html#cannot_be_in_pipeline?/3"},{"type":"function","title":"Credo.Check.Readability.NestedFunctionCalls.PipeHelper.valid_chain_start_function_call?/3","doc":"","ref":"Credo.Check.Readability.NestedFunctionCalls.PipeHelper.html#valid_chain_start_function_call?/3"},{"type":"module","title":"Credo.Check.Readability.OneArityFunctionInPipe","doc":"This check has a base priority of `low` and works with any version of Elixir.","ref":"Credo.Check.Readability.OneArityFunctionInPipe.html"},{"type":"module","title":"Explanation - Credo.Check.Readability.OneArityFunctionInPipe","doc":"Use parentheses for one-arity functions when using the pipe operator (|>).\n\n    # not preferred\n    some_string |> String.downcase |> String.trim\n\n    # preferred\n    some_string |> String.downcase() |> String.trim()\n\nLike all `Readability` issues, this one is not a technical concern.\nBut you can improve the odds of others reading and liking your code by making\nit easier to follow.","ref":"Credo.Check.Readability.OneArityFunctionInPipe.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Readability.OneArityFunctionInPipe","doc":"*There are no specific parameters for this check.*","ref":"Credo.Check.Readability.OneArityFunctionInPipe.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Readability.OneArityFunctionInPipe","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Readability.OneArityFunctionInPipe.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Readability.OnePipePerLine","doc":"This check has a base priority of `0` and works with any version of Elixir.","ref":"Credo.Check.Readability.OnePipePerLine.html"},{"type":"module","title":"Explanation - Credo.Check.Readability.OnePipePerLine","doc":"Don't use multiple pipes (|>) in the same line.\nEach function in the pipe should be in it's own line.\n\n    # preferred\n\n    foo\n    |> bar()\n    |> baz()\n\n    # NOT preferred\n\n    foo |> bar() |> baz()\n\nThe code in this example ...\n\n    1 |> Integer.to_string() |> String.to_integer()\n\n... should be refactored to look like this:\n\n    1\n    |> Integer.to_string()\n    |> String.to_integer()\n\nLike all `Readability` issues, this one is not a technical concern.\nBut you can improve the odds of others reading and liking your code by making\nit easier to follow.","ref":"Credo.Check.Readability.OnePipePerLine.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Readability.OnePipePerLine","doc":"*There are no specific parameters for this check.*","ref":"Credo.Check.Readability.OnePipePerLine.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Readability.OnePipePerLine","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Readability.OnePipePerLine.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Readability.ParenthesesInCondition","doc":"This check has a base priority of `high` and works with any version of Elixir.","ref":"Credo.Check.Readability.ParenthesesInCondition.html"},{"type":"module","title":"Explanation - Credo.Check.Readability.ParenthesesInCondition","doc":"Because `if` and `unless` are macros, the preferred style is to not use\nparentheses around conditions.\n\n    # preferred\n\n    if valid?(username) do\n      # ...\n    end\n\n    # NOT preferred\n\n    if( valid?(username) ) do\n      # ...\n    end\n\nLike all `Readability` issues, this one is not a technical concern.\nBut you can improve the odds of others reading and liking your code by making\nit easier to follow.","ref":"Credo.Check.Readability.ParenthesesInCondition.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Readability.ParenthesesInCondition","doc":"*There are no specific parameters for this check.*","ref":"Credo.Check.Readability.ParenthesesInCondition.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Readability.ParenthesesInCondition","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Readability.ParenthesesInCondition.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Readability.ParenthesesOnZeroArityDefs","doc":"This check has a base priority of `low` and works with any version of Elixir.","ref":"Credo.Check.Readability.ParenthesesOnZeroArityDefs.html"},{"type":"module","title":"Explanation - Credo.Check.Readability.ParenthesesOnZeroArityDefs","doc":"Either use parentheses or not when defining a function with no arguments.\n\nBy default, this check enforces no parentheses, so zero-arity function\nand macro definitions should look like this:\n\n    def summer? do\n      # ...\n    end\n\nIf the `:parens` param is set to `true` for this check, then the check\nenforces zero-arity function and macro definitions to have parens:\n\n    def summer?() do\n      # ...\n    end\n\nLike all `Readability` issues, this one is not a technical concern.\nBut you can improve the odds of others reading and liking your code by making\nit easier to follow.","ref":"Credo.Check.Readability.ParenthesesOnZeroArityDefs.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Readability.ParenthesesOnZeroArityDefs","doc":"*There are no specific parameters for this check.*","ref":"Credo.Check.Readability.ParenthesesOnZeroArityDefs.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Readability.ParenthesesOnZeroArityDefs","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Readability.ParenthesesOnZeroArityDefs.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Readability.PipeIntoAnonymousFunctions","doc":"This check has a base priority of `low` and works with any version of Elixir.","ref":"Credo.Check.Readability.PipeIntoAnonymousFunctions.html"},{"type":"module","title":"Explanation - Credo.Check.Readability.PipeIntoAnonymousFunctions","doc":"Avoid piping into anonymous functions.\n\nThe code in this example ...\n\n    def my_fun(foo) do\n      foo\n      |> (fn i -> i * 2 end).()\n      |> my_other_fun()\n    end\n\n... should be refactored to define a private function:\n\n    def my_fun(foo) do\n      foo\n      |> times_2()\n      |> my_other_fun()\n    end\n\n    defp timex_2(i), do: i * 2\n\nLike all `Readability` issues, this one is not a technical concern.\nBut you can improve the odds of others reading and liking your code by making\nit easier to follow.","ref":"Credo.Check.Readability.PipeIntoAnonymousFunctions.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Readability.PipeIntoAnonymousFunctions","doc":"*There are no specific parameters for this check.*","ref":"Credo.Check.Readability.PipeIntoAnonymousFunctions.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Readability.PipeIntoAnonymousFunctions","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Readability.PipeIntoAnonymousFunctions.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Readability.PredicateFunctionNames","doc":"This check has a base priority of `high` and works with any version of Elixir.","ref":"Credo.Check.Readability.PredicateFunctionNames.html"},{"type":"module","title":"Explanation - Credo.Check.Readability.PredicateFunctionNames","doc":"Predicate functions/macros should be named accordingly:\n\n* For functions, they should end in a question mark.\n\n    # preferred\n\n    defp user?(cookie) do\n    end\n\n    defp has_attachment?(mail) do\n    end\n\n    # NOT preferred\n\n    defp is_user?(cookie) do\n    end\n\n    defp is_user(cookie) do\n    end\n\n* For guard-safe macros they should have the prefix `is_` and not end in a question mark.\n\n    # preferred\n\n    defmacro is_user(cookie) do\n    end\n\n    # NOT preferred\n\n    defmacro is_user?(cookie) do\n    end\n\n    defmacro user?(cookie) do\n    end\n\nLike all `Readability` issues, this one is not a technical concern.\nBut you can improve the odds of others reading and liking your code by making\nit easier to follow.","ref":"Credo.Check.Readability.PredicateFunctionNames.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Readability.PredicateFunctionNames","doc":"*There are no specific parameters for this check.*","ref":"Credo.Check.Readability.PredicateFunctionNames.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Readability.PredicateFunctionNames","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Readability.PredicateFunctionNames.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Readability.PreferImplicitTry","doc":"This check has a base priority of `low` and works with any version of Elixir.","ref":"Credo.Check.Readability.PreferImplicitTry.html"},{"type":"module","title":"Explanation - Credo.Check.Readability.PreferImplicitTry","doc":"Prefer using an implicit `try` rather than explicit `try` if you try to rescue\nanything the function does.\n\nFor example, this:\n\n    def failing_function(first) do\n      try do\n        to_string(first)\n      rescue\n        _ -> :rescued\n      end\n    end\n\nCan be rewritten without `try` as below:\n\n    def failing_function(first) do\n      to_string(first)\n    rescue\n      _ -> :rescued\n    end\n\nLike all `Readability` issues, this one is not a technical concern.\nThe code will behave identical in both ways.","ref":"Credo.Check.Readability.PreferImplicitTry.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Readability.PreferImplicitTry","doc":"*There are no specific parameters for this check.*","ref":"Credo.Check.Readability.PreferImplicitTry.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Readability.PreferImplicitTry","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Readability.PreferImplicitTry.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Readability.PreferUnquotedAtoms","doc":"This check has a base priority of `high` and requires Elixir `< 1.7.0-dev`.","ref":"Credo.Check.Readability.PreferUnquotedAtoms.html"},{"type":"module","title":"Explanation - Credo.Check.Readability.PreferUnquotedAtoms","doc":"Prefer unquoted atoms unless quotes are necessary.\nThis is helpful because a quoted atom can be easily mistaken for a string.\n\n    # preferred\n\n    :x\n    [x: 1]\n    %{x: 1}\n\n    # NOT preferred\n\n    :\"x\"\n    [\"x\": 1]\n    %{\"x\": 1}\n\nThe primary case where this can become an issue is when using atoms or\nstrings for keys in a Map or Keyword list.\n\nFor example, this:\n\n    %{\"x\": 1}\n\nCan easily be mistaken for this:\n\n    %{\"x\" => 1}\n\nBecause a string key cannot be used to access a value with the equivalent\natom key, this can lead to subtle bugs which are hard to discover.\n\nLike all `Readability` issues, this one is not a technical concern.\nThe code will behave identical in both ways.","ref":"Credo.Check.Readability.PreferUnquotedAtoms.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Readability.PreferUnquotedAtoms","doc":"*There are no specific parameters for this check.*","ref":"Credo.Check.Readability.PreferUnquotedAtoms.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Readability.PreferUnquotedAtoms","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Readability.PreferUnquotedAtoms.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Readability.RedundantBlankLines","doc":"This check has a base priority of `low` and works with any version of Elixir.","ref":"Credo.Check.Readability.RedundantBlankLines.html"},{"type":"module","title":"Explanation - Credo.Check.Readability.RedundantBlankLines","doc":"Files should not have two or more consecutive blank lines.\n\nLike all `Readability` issues, this one is not a technical concern.\nBut you can improve the odds of others reading and liking your code by making\nit easier to follow.","ref":"Credo.Check.Readability.RedundantBlankLines.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Readability.RedundantBlankLines","doc":"Use the following parameters to configure this check:\n\n### `:max_blank_lines`\n\n  The maximum number of tolerated consecutive blank lines.\n\n*This parameter defaults to* `1`.","ref":"Credo.Check.Readability.RedundantBlankLines.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Readability.RedundantBlankLines","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Readability.RedundantBlankLines.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Readability.Semicolons","doc":"This check has a base priority of `high` and works with any version of Elixir.","ref":"Credo.Check.Readability.Semicolons.html"},{"type":"module","title":"Explanation - Credo.Check.Readability.Semicolons","doc":"Don't use ; to separate statements and expressions.\nStatements and expressions should be separated by lines.\n\n    # preferred\n\n    a = 1\n    b = 2\n\n    # NOT preferred\n\n    a = 1; b = 2\n\nLike all `Readability` issues, this one is not a technical concern.\nBut you can improve the odds of others reading and liking your code by making\nit easier to follow.","ref":"Credo.Check.Readability.Semicolons.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Readability.Semicolons","doc":"*There are no specific parameters for this check.*","ref":"Credo.Check.Readability.Semicolons.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Readability.Semicolons","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Readability.Semicolons.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Readability.SeparateAliasRequire","doc":"This check has a base priority of `low` and works with any version of Elixir.","ref":"Credo.Check.Readability.SeparateAliasRequire.html"},{"type":"module","title":"Explanation - Credo.Check.Readability.SeparateAliasRequire","doc":"All instances of `alias` should be consecutive within a file.\nLikewise, all instances of `require` should be consecutive within a file.\n\nFor example:\n\n    defmodule Foo do\n      require Logger\n      alias Foo.Bar\n\n      alias Foo.Baz\n      require Integer\n\n      # ...\n    end\n\nshould be changed to:\n\n    defmodule Foo do\n      require Integer\n      require Logger\n\n      alias Foo.Bar\n      alias Foo.Baz\n\n      # ...\n    end\n\nLike all `Readability` issues, this one is not a technical concern.\nBut you can improve the odds of others reading and liking your code by making\nit easier to follow.","ref":"Credo.Check.Readability.SeparateAliasRequire.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Readability.SeparateAliasRequire","doc":"*There are no specific parameters for this check.*","ref":"Credo.Check.Readability.SeparateAliasRequire.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Readability.SeparateAliasRequire","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Readability.SeparateAliasRequire.html#module-general-parameters"},{"type":"function","title":"Credo.Check.Readability.SeparateAliasRequire.message/1","doc":"","ref":"Credo.Check.Readability.SeparateAliasRequire.html#message/1"},{"type":"module","title":"Credo.Check.Readability.SingleFunctionToBlockPipe","doc":"This check has a base priority of `0` and works with any version of Elixir.","ref":"Credo.Check.Readability.SingleFunctionToBlockPipe.html"},{"type":"module","title":"Explanation - Credo.Check.Readability.SingleFunctionToBlockPipe","doc":"A single pipe (`|>`) should not be used to pipe into blocks.\n\nThe code in this example ...\n\n    list\n    |> length()\n    |> case do\n      0 -> :none\n      1 -> :one\n      _ -> :many\n    end\n\n... should be refactored to look like this:\n\n    case length(list) do\n      0 -> :none\n      1 -> :one\n      _ -> :many\n    end\n\nIf you want to disallow piping into blocks all together, use\n`Credo.Check.Readability.BlockPipe`.","ref":"Credo.Check.Readability.SingleFunctionToBlockPipe.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Readability.SingleFunctionToBlockPipe","doc":"*There are no specific parameters for this check.*","ref":"Credo.Check.Readability.SingleFunctionToBlockPipe.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Readability.SingleFunctionToBlockPipe","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Readability.SingleFunctionToBlockPipe.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Readability.SinglePipe","doc":"This check has a base priority of `high` and works with any version of Elixir.","ref":"Credo.Check.Readability.SinglePipe.html"},{"type":"module","title":"Explanation - Credo.Check.Readability.SinglePipe","doc":"Pipes (`|>`) should only be used when piping data through multiple calls.\n\nSo while this is fine:\n\n    list\n    |> Enum.take(5)\n    |> Enum.shuffle\n    |> evaluate()\n\nThe code in this example ...\n\n    list\n    |> evaluate()\n\n... should be refactored to look like this:\n\n    evaluate(list)\n\nUsing a single |> to invoke functions makes the code harder to read. Instead,\nwrite a function call when a pipeline is only one function long.\n\nLike all `Readability` issues, this one is not a technical concern.\nBut you can improve the odds of others reading and liking your code by making\nit easier to follow.","ref":"Credo.Check.Readability.SinglePipe.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Readability.SinglePipe","doc":"Use the following parameters to configure this check:\n\n### `:allow_0_arity_functions`\n\n  Allow 0-arity functions\n\n*This parameter defaults to* `false`.","ref":"Credo.Check.Readability.SinglePipe.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Readability.SinglePipe","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Readability.SinglePipe.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Readability.SpaceAfterCommas","doc":"This check has a base priority of `0` and works with any version of Elixir.","ref":"Credo.Check.Readability.SpaceAfterCommas.html"},{"type":"module","title":"Explanation - Credo.Check.Readability.SpaceAfterCommas","doc":"You can use white-space after commas to make items of lists,\ntuples and other enumerations easier to separate from one another.\n\n    # preferred\n\n    alias Project.{Alpha, Beta}\n\n    def some_func(first, second, third) do\n      list = [1, 2, 3, 4, 5]\n      # ...\n    end\n\n    # NOT preferred - items are harder to separate\n\n    alias Project.{Alpha,Beta}\n\n    def some_func(first,second,third) do\n      list = [1,2,3,4,5]\n      # ...\n    end\n\nLike all `Readability` issues, this one is not a technical concern.\nBut you can improve the odds of others reading and liking your code by making\nit easier to follow.","ref":"Credo.Check.Readability.SpaceAfterCommas.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Readability.SpaceAfterCommas","doc":"*There are no specific parameters for this check.*","ref":"Credo.Check.Readability.SpaceAfterCommas.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Readability.SpaceAfterCommas","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Readability.SpaceAfterCommas.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Readability.Specs","doc":"This check has a base priority of `0` and works with any version of Elixir.","ref":"Credo.Check.Readability.Specs.html"},{"type":"module","title":"Explanation - Credo.Check.Readability.Specs","doc":"Functions, callbacks and macros need typespecs.\n\nAdding typespecs gives tools like Dialyzer more information when performing\nchecks for type errors in function calls and definitions.\n\n    @spec add(integer, integer) :: integer\n    def add(a, b), do: a + b\n\nFunctions with multiple arities need to have a spec defined for each arity:\n\n    @spec foo(integer) :: boolean\n    @spec foo(integer, integer) :: boolean\n    def foo(a), do: a > 0\n    def foo(a, b), do: a > b\n\nThe check only considers whether the specification is present, it doesn't\nperform any actual type checking.\n\nLike all `Readability` issues, this one is not a technical concern.\nBut you can improve the odds of others reading and liking your code by making\nit easier to follow.","ref":"Credo.Check.Readability.Specs.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Readability.Specs","doc":"Use the following parameters to configure this check:\n\n### `:include_defp`\n\n  Include private functions.\n\n*This parameter defaults to* `false`.","ref":"Credo.Check.Readability.Specs.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Readability.Specs","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Readability.Specs.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Readability.StrictModuleLayout","doc":"This check has a base priority of `low` and works with any version of Elixir.","ref":"Credo.Check.Readability.StrictModuleLayout.html"},{"type":"module","title":"Explanation - Credo.Check.Readability.StrictModuleLayout","doc":"Provide module parts in a required order.\n\n    # preferred\n\n    defmodule MyMod do\n      @moduledoc \"moduledoc\"\n      use Foo\n      import Bar\n      alias Baz\n      require Qux\n    end\n\nLike all `Readability` issues, this one is not a technical concern.\nBut you can improve the odds of others reading and liking your code by making\nit easier to follow.","ref":"Credo.Check.Readability.StrictModuleLayout.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Readability.StrictModuleLayout","doc":"Use the following parameters to configure this check:\n\n### `:order`\n\n  List of atoms identifying the desired order of module parts.\n  \n  Supported values are:\n  \n  - `:moduledoc` - `@moduledoc` module attribute\n  - `:shortdoc` - `@shortdoc` module attribute\n  - `:behaviour` - `@behaviour` module attribute\n  - `:use` - `use` expression\n  - `:import` - `import` expression\n  - `:alias` - `alias` expression\n  - `:require` - `require` expression\n  - `:defstruct` - `defstruct` expression\n  - `:opaque` - `@opaque` module attribute\n  - `:type` - `@type` module attribute\n  - `:typep` - `@typep` module attribute\n  - `:callback` - `@callback` module attribute\n  - `:macrocallback` - `@macrocallback` module attribute\n  - `:optional_callbacks` - `@optional_callbacks` module attribute\n  - `:module_attribute` - other module attribute\n  - `:public_fun` - public function\n  - `:private_fun` - private function or a public function marked with `@doc false`\n  - `:public_macro` - public macro\n  - `:private_macro` - private macro or a public macro marked with `@doc false`\n  - `:callback_impl` - public function or macro marked with `@impl`\n  - `:public_guard` - public guard\n  - `:private_guard` - private guard or a public guard marked with `@doc false`\n  - `:module` - inner module definition (`defmodule` expression inside a module)\n  \n  Notice that the desired order always starts from the top.\n  \n  For example, if you provide the order `~w/public_fun private_fun/a`,\n  it means that everything else (e.g. `@moduledoc`) must appear after\n  function definitions.\n  \n\n*This parameter defaults to* `[:shortdoc, :moduledoc, :behaviour, :use, :import, :alias, :require]`.\n\n### `:ignore`\n\n  List of atoms identifying the module parts which are not checked, and may\n  therefore appear anywhere in the module. Supported values are the same as\n  in the `:order` param.\n  \n\n*This parameter defaults to* `[]`.\n\n### `:ignore_module_attributes`\n\n  List of atoms identifying the module attributes which are not checked, and may\n  therefore appear anywhere in the module. Useful for custom DSLs that use attributes\n  before function heads.\n  \n  For example, if you provide `~w/trace/a`, all `@trace` attributes will be ignored.\n  \n\n*This parameter defaults to* `[]`.","ref":"Credo.Check.Readability.StrictModuleLayout.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Readability.StrictModuleLayout","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Readability.StrictModuleLayout.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Readability.StringSigils","doc":"This check has a base priority of `low` and works with any version of Elixir.","ref":"Credo.Check.Readability.StringSigils.html"},{"type":"module","title":"Explanation - Credo.Check.Readability.StringSigils","doc":"If you used quoted strings that contain quotes, you might want to consider\nswitching to the use of sigils instead.\n\n    # okay\n\n    \" #\\{text} \"\n\n    # not okay, lots of escaped quotes\n\n    \" #\\{text} \"\n\n    # refactor to\n\n    ~S( #\\{text} )\n\nThis allows us to remove the noise which results from the need to escape\nquotes within quotes.\n\nLike all `Readability` issues, this one is not a technical concern.\nBut you can improve the odds of others reading and liking your code by making\nit easier to follow.","ref":"Credo.Check.Readability.StringSigils.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Readability.StringSigils","doc":"Use the following parameters to configure this check:\n\n### `:maximum_allowed_quotes`\n\n  The maximum amount of escaped quotes you want to tolerate.\n\n*This parameter defaults to* `3`.","ref":"Credo.Check.Readability.StringSigils.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Readability.StringSigils","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Readability.StringSigils.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Readability.TrailingBlankLine","doc":"This check has a base priority of `low` and works with any version of Elixir.","ref":"Credo.Check.Readability.TrailingBlankLine.html"},{"type":"module","title":"Explanation - Credo.Check.Readability.TrailingBlankLine","doc":"Files should end in a trailing blank line.\n\nThis is mostly for historical reasons: every text file should end with a \\n,\nor newline since this acts as `eol` or the end of the line character.\n\nSee also: http://pubs.opengroup.org/onlinepubs/9699919799/basedefs/V1_chap03.html#tag_03_206\n\nMost text editors ensure this \"final newline\" automatically.","ref":"Credo.Check.Readability.TrailingBlankLine.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Readability.TrailingBlankLine","doc":"*There are no specific parameters for this check.*","ref":"Credo.Check.Readability.TrailingBlankLine.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Readability.TrailingBlankLine","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Readability.TrailingBlankLine.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Readability.TrailingWhiteSpace","doc":"This check has a base priority of `low` and works with any version of Elixir.","ref":"Credo.Check.Readability.TrailingWhiteSpace.html"},{"type":"module","title":"Explanation - Credo.Check.Readability.TrailingWhiteSpace","doc":"There should be no white-space (i.e. tabs, spaces) at the end of a line.\n\nMost text editors provide a way to remove them automatically.","ref":"Credo.Check.Readability.TrailingWhiteSpace.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Readability.TrailingWhiteSpace","doc":"Use the following parameters to configure this check:\n\n### `:ignore_strings`\n\n  Set to `false` to check lines that are strings or in heredocs\n\n*This parameter defaults to* `true`.","ref":"Credo.Check.Readability.TrailingWhiteSpace.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Readability.TrailingWhiteSpace","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Readability.TrailingWhiteSpace.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Readability.UnnecessaryAliasExpansion","doc":"This check has a base priority of `low` and works with any version of Elixir.","ref":"Credo.Check.Readability.UnnecessaryAliasExpansion.html"},{"type":"module","title":"Explanation - Credo.Check.Readability.UnnecessaryAliasExpansion","doc":"Alias expansion is useful but when aliasing a single module,\nit can be harder to read with unnecessary braces.\n\n    # preferred\n\n    alias ModuleA.Foo\n    alias ModuleA.{Foo, Bar}\n\n    # NOT preferred\n\n    alias ModuleA.{Foo}\n\nLike all `Readability` issues, this one is not a technical concern.\nBut you can improve the odds of others reading and liking your code by making\nit easier to follow.","ref":"Credo.Check.Readability.UnnecessaryAliasExpansion.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Readability.UnnecessaryAliasExpansion","doc":"*There are no specific parameters for this check.*","ref":"Credo.Check.Readability.UnnecessaryAliasExpansion.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Readability.UnnecessaryAliasExpansion","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Readability.UnnecessaryAliasExpansion.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Readability.VariableNames","doc":"This check has a base priority of `high` and works with any version of Elixir.","ref":"Credo.Check.Readability.VariableNames.html"},{"type":"module","title":"Explanation - Credo.Check.Readability.VariableNames","doc":"Variable names are always written in snake_case in Elixir.\n\n    # snake_case:\n\n    incoming_result = handle_incoming_message(message)\n\n    # not snake_case\n\n    incomingResult = handle_incoming_message(message)\n\nLike all `Readability` issues, this one is not a technical concern.\nBut you can improve the odds of others reading and liking your code by making\nit easier to follow.","ref":"Credo.Check.Readability.VariableNames.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Readability.VariableNames","doc":"*There are no specific parameters for this check.*","ref":"Credo.Check.Readability.VariableNames.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Readability.VariableNames","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Readability.VariableNames.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Readability.WithCustomTaggedTuple","doc":"This check has a base priority of `low` and works with any version of Elixir.","ref":"Credo.Check.Readability.WithCustomTaggedTuple.html"},{"type":"module","title":"Explanation - Credo.Check.Readability.WithCustomTaggedTuple","doc":"Avoid using custom tags for error reporting from `with` macros.\n\nThis code injects placeholder tags such as `:resource` and `:authz` for the purpose of error\nreporting.\n\n    with {:resource, {:ok, resource}}   {:error, :not_found}\n      {:authz, _} -> {:error, :unauthorized}\n    end\n\nInstead, extract each validation into a separate helper function which returns error\ninformation immediately:\n\n    defp find_resource(user) do\n      with :error <- Resource.fetch(user), do: {:error, :not_found}\n    end\n\n    defp authorize(resource, user) do\n      with :error <- Resource.authorize(resource, user), do: {:error, :unauthorized}\n    end\n\nAt this point, the validation chain in `with` becomes clearer and easier to understand:\n\n    with {:ok, resource} <- find_resource(user),\n         :ok <- authorize(resource, user),\n         do: do_something(user)\n\nLike all `Readability` issues, this one is not a technical concern.\nBut you can improve the odds of others reading and liking your code by making\nit easier to follow.","ref":"Credo.Check.Readability.WithCustomTaggedTuple.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Readability.WithCustomTaggedTuple","doc":"*There are no specific parameters for this check.*","ref":"Credo.Check.Readability.WithCustomTaggedTuple.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Readability.WithCustomTaggedTuple","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Readability.WithCustomTaggedTuple.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Readability.WithSingleClause","doc":"This check has a base priority of `0` and works with any version of Elixir.","ref":"Credo.Check.Readability.WithSingleClause.html"},{"type":"module","title":"Explanation - Credo.Check.Readability.WithSingleClause","doc":"`with` statements are useful when you need to chain a sequence\nof pattern matches, stopping at the first one that fails.\n\nIf the `with` has a single pattern matching clause and no `else`\nbranch, it means that if the clause doesn't match than the whole\n`with` will return the value of that clause.\n\nHowever, if that `with` has also an `else` clause, then you're using `with` exactly\nlike a `case` and a `case` should be used instead.\n\nTake this code:\n\n    with {:ok, user}  \n        raise \"DB is down!\"\n\n      {:error, reason} ->\n        raise \"error: #{inspect(reason)}\"\n    end\n\nIt can be rewritten with a clearer use of `case`:\n\n    case User.create(make_ref()) do\n      {:ok, user} ->\n        user\n\n      {:error, :db_down} ->\n        raise \"DB is down!\"\n\n      {:error, reason} ->\n        raise \"error: #{inspect(reason)}\"\n    end\n\nLike all `Readability` issues, this one is not a technical concern.\nBut you can improve the odds of others reading and liking your code by making\nit easier to follow.","ref":"Credo.Check.Readability.WithSingleClause.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Readability.WithSingleClause","doc":"*There are no specific parameters for this check.*","ref":"Credo.Check.Readability.WithSingleClause.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Readability.WithSingleClause","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Readability.WithSingleClause.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Refactor.ABCSize","doc":"This check has a base priority of `0` and works with any version of Elixir.","ref":"Credo.Check.Refactor.ABCSize.html"},{"type":"module","title":"Explanation - Credo.Check.Refactor.ABCSize","doc":"The ABC size describes a metric based on assignments, branches and conditions.\n\nA high ABC size is a hint that a function might be doing \"more\" than it\nshould.\n\nAs always: Take any metric with a grain of salt. Since this one was originally\nintroduced for C, C++ and Java, we still have to see whether or not this can\nbe a useful metric in a declarative language like Elixir.","ref":"Credo.Check.Refactor.ABCSize.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Refactor.ABCSize","doc":"Use the following parameters to configure this check:\n\n### `:max_size`\n\n  The maximum ABC size a function should have.\n\n*This parameter defaults to* `30`.\n\n### `:excluded_functions`\n\n  All functions listed will be ignored.\n\n*This parameter defaults to* `[]`.","ref":"Credo.Check.Refactor.ABCSize.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Refactor.ABCSize","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Refactor.ABCSize.html#module-general-parameters"},{"type":"function","title":"Credo.Check.Refactor.ABCSize.abc_size_for/2","doc":"Returns the ABC size for the block inside the given AST, which is expected\nto represent a function or macro definition.\n\n    iex> {:def, [line: 1],\n    ...>   [\n    ...>     {:first_fun, [line: 1], nil},\n    ...>     [do: {:=, [line: 2], [{:x, [line: 2], nil}, 1]}]\n    ...>   ]\n    ...> } |> Credo.Check.Refactor.ABCSize.abc_size\n    1.0","ref":"Credo.Check.Refactor.ABCSize.html#abc_size_for/2"},{"type":"module","title":"Credo.Check.Refactor.AppendSingleItem","doc":"This check has a base priority of `low` and works with any version of Elixir.","ref":"Credo.Check.Refactor.AppendSingleItem.html"},{"type":"module","title":"Explanation - Credo.Check.Refactor.AppendSingleItem","doc":"When building up large lists, it is faster to prepend than\nappend. Therefore: It is sometimes best to prepend to the list\nduring iteration and call Enum.reverse/1 at the end, as it is quite\nfast.\n\nExample:\n\n    list = list_so_far ++ [new_item]\n\n    # refactoring it like this can make the code faster:\n\n    list = [new_item] ++ list_so_far\n    # ...\n    Enum.reverse(list)","ref":"Credo.Check.Refactor.AppendSingleItem.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Refactor.AppendSingleItem","doc":"*There are no specific parameters for this check.*","ref":"Credo.Check.Refactor.AppendSingleItem.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Refactor.AppendSingleItem","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Refactor.AppendSingleItem.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Refactor.Apply","doc":"This check has a base priority of `low` and works with any version of Elixir.","ref":"Credo.Check.Refactor.Apply.html"},{"type":"module","title":"Explanation - Credo.Check.Refactor.Apply","doc":"Prefer calling functions directly if the number of arguments is known\nat compile time instead of using `apply/2` and `apply/3`.\n\nExample:\n\n    # preferred\n\n    fun.(arg_1, arg_2, ..., arg_n)\n\n    module.function(arg_1, arg_2, ..., arg_n)\n\n    # NOT preferred\n\n    apply(fun, [arg_1, arg_2, ..., arg_n])\n\n    apply(module, :function, [arg_1, arg_2, ..., arg_n])","ref":"Credo.Check.Refactor.Apply.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Refactor.Apply","doc":"*There are no specific parameters for this check.*","ref":"Credo.Check.Refactor.Apply.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Refactor.Apply","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Refactor.Apply.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Refactor.CaseTrivialMatches","doc":"This check has a base priority of `0` and works with any version of Elixir.","ref":"Credo.Check.Refactor.CaseTrivialMatches.html"},{"type":"module","title":"Explanation - Credo.Check.Refactor.CaseTrivialMatches","doc":"PLEASE NOTE: This check is deprecated as it might do more harm than good.\n\nRelated discussion: https://github.com/rrrene/credo/issues/65","ref":"Credo.Check.Refactor.CaseTrivialMatches.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Refactor.CaseTrivialMatches","doc":"*There are no specific parameters for this check.*","ref":"Credo.Check.Refactor.CaseTrivialMatches.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Refactor.CaseTrivialMatches","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Refactor.CaseTrivialMatches.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Refactor.CondStatements","doc":"This check has a base priority of `0` and works with any version of Elixir.","ref":"Credo.Check.Refactor.CondStatements.html"},{"type":"module","title":"Explanation - Credo.Check.Refactor.CondStatements","doc":"Each cond statement should have 3 or more statements including the\n\"always true\" statement.\n\nConsider an `if`/`else` construct if there is only one condition and the\n\"always true\" statement, since it will more accessible to programmers\nnew to the codebase (and possibly new to Elixir).\n\nExample:\n\n    cond do\n      x == y -> 0\n      true -> 1\n    end\n\n    # should be written as\n\n    if x == y do\n      0\n    else\n      1\n    end","ref":"Credo.Check.Refactor.CondStatements.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Refactor.CondStatements","doc":"*There are no specific parameters for this check.*","ref":"Credo.Check.Refactor.CondStatements.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Refactor.CondStatements","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Refactor.CondStatements.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Refactor.CyclomaticComplexity","doc":"This check has a base priority of `0` and works with any version of Elixir.","ref":"Credo.Check.Refactor.CyclomaticComplexity.html"},{"type":"module","title":"Explanation - Credo.Check.Refactor.CyclomaticComplexity","doc":"Cyclomatic complexity (CC) is a software complexity metric closely\ncorrelated with coding errors.\n\nIf a function feels like it's gotten too complex, it more often than not also\nhas a high CC value. So, if anything, this is useful to convince team members\nand bosses of a need to refactor parts of the code based on \"objective\"\nmetrics.","ref":"Credo.Check.Refactor.CyclomaticComplexity.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Refactor.CyclomaticComplexity","doc":"Use the following parameters to configure this check:\n\n### `:max_complexity`\n\n  The maximum cyclomatic complexity a function should have.\n\n*This parameter defaults to* `9`.","ref":"Credo.Check.Refactor.CyclomaticComplexity.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Refactor.CyclomaticComplexity","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Refactor.CyclomaticComplexity.html#module-general-parameters"},{"type":"function","title":"Credo.Check.Refactor.CyclomaticComplexity.complexity_for/1","doc":"Returns the Cyclomatic Complexity score for the block inside the given AST,\nwhich is expected to represent a function or macro definition.\n\n    iex> {:def, [line: 1],\n    ...>   [\n    ...>     {:first_fun, [line: 1], nil},\n    ...>     [do: {:=, [line: 2], [{:x, [line: 2], nil}, 1]}]\n    ...>   ]\n    ...> } |> Credo.Check.Refactor.CyclomaticComplexity.complexity_for\n    1.0","ref":"Credo.Check.Refactor.CyclomaticComplexity.html#complexity_for/1"},{"type":"module","title":"Credo.Check.Refactor.DoubleBooleanNegation","doc":"This check has a base priority of `low` and works with any version of Elixir.","ref":"Credo.Check.Refactor.DoubleBooleanNegation.html"},{"type":"module","title":"Explanation - Credo.Check.Refactor.DoubleBooleanNegation","doc":"Having double negations in your code can obscure the parameter's original value.\n\n    # NOT preferred\n\n    !!var\n\nThis will return `false` for `false` and `nil`, and `true` for anything else.\n\nAt first this seems like an extra clever shorthand to cast anything truthy to\n`true` and anything non-truthy to `false`. But in most scenarios you want to\nbe explicit about your input parameters (because it is easier to reason about\nedge-cases, code-paths and tests).\nAlso: `nil` and `false` do mean two different things.\n\nA scenario where you want this kind of flexibility, however, is parsing\nexternal data, e.g. a third party JSON-API where a value is sometimes `null`\nand sometimes `false` and you want to normalize that before handing it down\nin your program.\n\nIn these case, you would be better off making the cast explicit by introducing\na helper function:\n\n    # preferred\n\n    defp present?(nil), do: false\n    defp present?(false), do: false\n    defp present?(_), do: true\n\nThis makes your code more explicit than relying on the implications of `!!`.","ref":"Credo.Check.Refactor.DoubleBooleanNegation.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Refactor.DoubleBooleanNegation","doc":"*There are no specific parameters for this check.*","ref":"Credo.Check.Refactor.DoubleBooleanNegation.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Refactor.DoubleBooleanNegation","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Refactor.DoubleBooleanNegation.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Refactor.EnumHelpers","doc":"","ref":"Credo.Check.Refactor.EnumHelpers.html"},{"type":"function","title":"Credo.Check.Refactor.EnumHelpers.traverse/8","doc":"","ref":"Credo.Check.Refactor.EnumHelpers.html#traverse/8"},{"type":"module","title":"Credo.Check.Refactor.FilterCount","doc":"This check has a base priority of `high` and works with any version of Elixir.","ref":"Credo.Check.Refactor.FilterCount.html"},{"type":"module","title":"Explanation - Credo.Check.Refactor.FilterCount","doc":"`Enum.count/2` is more efficient than `Enum.filter/2 |> Enum.count/1`.\n\nThis should be refactored:\n\n    [1, 2, 3, 4, 5]\n    |> Enum.filter(fn x -> rem(x, 3) == 0 end)\n    |> Enum.count()\n\nto look like this:\n\n    Enum.count([1, 2, 3, 4, 5], fn x -> rem(x, 3) == 0 end)\n\nThe reason for this is performance, because the two separate calls\nto `Enum.filter/2` and `Enum.count/1` require two iterations whereas\n`Enum.count/2` performs the same work in one pass.","ref":"Credo.Check.Refactor.FilterCount.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Refactor.FilterCount","doc":"*There are no specific parameters for this check.*","ref":"Credo.Check.Refactor.FilterCount.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Refactor.FilterCount","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Refactor.FilterCount.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Refactor.FilterFilter","doc":"This check has a base priority of `0` and works with any version of Elixir.","ref":"Credo.Check.Refactor.FilterFilter.html"},{"type":"module","title":"Explanation - Credo.Check.Refactor.FilterFilter","doc":"One `Enum.filter/2` is more efficient than `Enum.filter/2 |> Enum.filter/2`.\n\nThis should be refactored:\n\n    [\"a\", \"b\", \"c\"]\n    |> Enum.filter(&String.contains?(&1, \"x\"))\n    |> Enum.filter(&String.contains?(&1, \"a\"))\n\nto look like this:\n\n    Enum.filter([\"a\", \"b\", \"c\"], fn letter ->\n      String.contains?(letter, \"x\") && String.contains?(letter, \"a\")\n    end)\n\nThe reason for this is performance, because the two separate calls\nto `Enum.filter/2` require two iterations whereas doing the\nfunctions in the single `Enum.filter/2` only requires one.","ref":"Credo.Check.Refactor.FilterFilter.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Refactor.FilterFilter","doc":"*There are no specific parameters for this check.*","ref":"Credo.Check.Refactor.FilterFilter.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Refactor.FilterFilter","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Refactor.FilterFilter.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Refactor.FilterReject","doc":"This check has a base priority of `0` and works with any version of Elixir.","ref":"Credo.Check.Refactor.FilterReject.html"},{"type":"module","title":"Explanation - Credo.Check.Refactor.FilterReject","doc":"One `Enum.filter/2` is more efficient than `Enum.filter/2 |> Enum.reject/2`.\n\nThis should be refactored:\n\n    [\"a\", \"b\", \"c\"]\n    |> Enum.filter(&String.contains?(&1, \"x\"))\n    |> Enum.reject(&String.contains?(&1, \"a\"))\n\nto look like this:\n\n    Enum.filter([\"a\", \"b\", \"c\"], fn letter ->\n      String.contains?(letter, \"x\") && !String.contains?(letter, \"a\")\n    end)\n\nThe reason for this is performance, because the two calls to\n`Enum.reject/2` and `Enum.filter/2` require two iterations whereas\ndoing the functions in the single `Enum.filter/2` only requires one.","ref":"Credo.Check.Refactor.FilterReject.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Refactor.FilterReject","doc":"*There are no specific parameters for this check.*","ref":"Credo.Check.Refactor.FilterReject.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Refactor.FilterReject","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Refactor.FilterReject.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Refactor.FunctionArity","doc":"This check has a base priority of `0` and works with any version of Elixir.","ref":"Credo.Check.Refactor.FunctionArity.html"},{"type":"module","title":"Explanation - Credo.Check.Refactor.FunctionArity","doc":"A function can take as many parameters as needed, but even in a functional\nlanguage there can be too many parameters.\n\nCan optionally ignore private functions (check configuration options).","ref":"Credo.Check.Refactor.FunctionArity.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Refactor.FunctionArity","doc":"Use the following parameters to configure this check:\n\n### `:max_arity`\n\n  The maximum number of parameters which a function should take.\n\n*This parameter defaults to* `8`.\n\n### `:ignore_defp`\n\n  Set to `true` to ignore private functions.\n\n*This parameter defaults to* `false`.","ref":"Credo.Check.Refactor.FunctionArity.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Refactor.FunctionArity","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Refactor.FunctionArity.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Refactor.IoPuts","doc":"This check has a base priority of `0` and works with any version of Elixir.","ref":"Credo.Check.Refactor.IoPuts.html"},{"type":"module","title":"Explanation - Credo.Check.Refactor.IoPuts","doc":"Prefer using Logger statements over using `IO.puts/1`.\n\nThis is a situational check.\n\nAs such, it might be a great help for e.g. Phoenix projects, but\na clear mismatch for CLI projects.","ref":"Credo.Check.Refactor.IoPuts.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Refactor.IoPuts","doc":"*There are no specific parameters for this check.*","ref":"Credo.Check.Refactor.IoPuts.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Refactor.IoPuts","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Refactor.IoPuts.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Refactor.LongQuoteBlocks","doc":"This check has a base priority of `high` and works with any version of Elixir.","ref":"Credo.Check.Refactor.LongQuoteBlocks.html"},{"type":"module","title":"Explanation - Credo.Check.Refactor.LongQuoteBlocks","doc":"Long `quote` blocks are generally an indication that too much is done inside\nthem.\n\nLet's look at why this is problematic:\n\n    defmodule MetaCommand do\n      def __using__(opts \\\\ []) do\n        modes = opts[:modes]\n        command_name = opts[:command_name]\n\n        quote do\n          def run(filename) do\n            contents =\n              if File.exists?(filename) do\n                {:ok, file} = File.open(filename, unquote(modes))\n                {:ok, contents} = IO.read(file, :line)\n                File.close(file)\n                contents\n              else\n                \"\"\n              end\n\n            case contents do\n              \"\" ->\n                # ...\n              unquote(command_name)   rest ->\n                # ...\n            end\n          end\n\n          # ...\n        end\n      end\n    end\n\nA cleaner solution would be to call \"regular\" functions outside the\n`quote` block to perform the actual work.\n\n    defmodule MyMetaCommand do\n      def __using__(opts \\\\ []) do\n        modes = opts[:modes]\n        command_name = opts[:command_name]\n\n        quote do\n          def run(filename) do\n            MyMetaCommand.run_on_file(filename, unquote(modes), unquote(command_name))\n          end\n\n          # ...\n        end\n      end\n\n      def run_on_file(filename, modes, command_name) do\n        contents =\n          # actual implementation\n      end\n    end\n\nThis way it is easier to reason about what is actually happening. And to debug\nit.","ref":"Credo.Check.Refactor.LongQuoteBlocks.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Refactor.LongQuoteBlocks","doc":"Use the following parameters to configure this check:\n\n### `:max_line_count`\n\n  The maximum number of lines a quote block should be allowed to have.\n\n*This parameter defaults to* `150`.\n\n### `:ignore_comments`\n\n  Ignores comments when counting the lines of a `quote` block.\n\n*This parameter defaults to* `false`.","ref":"Credo.Check.Refactor.LongQuoteBlocks.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Refactor.LongQuoteBlocks","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Refactor.LongQuoteBlocks.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Refactor.MapInto","doc":"This check has a base priority of `high` and requires Elixir `< 1.8.0`.","ref":"Credo.Check.Refactor.MapInto.html"},{"type":"module","title":"Explanation - Credo.Check.Refactor.MapInto","doc":"`Enum.into/3` is more efficient than `Enum.map/2 |> Enum.into/2`.\n\nThis should be refactored:\n\n    [:apple, :banana, :carrot]\n    |> Enum.map(&({&1, to_string(&1)}))\n    |> Enum.into(%{})\n\nto look like this:\n\n    Enum.into([:apple, :banana, :carrot], %{}, &({&1, to_string(&1)}))\n\nThe reason for this is performance, because the separate calls to\n`Enum.map/2` and `Enum.into/2` require two iterations whereas\n`Enum.into/3` only requires one.\n\n**NOTE**: This check is only available in Elixir < 1.8 since performance\nimprovements have since made this check obsolete.","ref":"Credo.Check.Refactor.MapInto.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Refactor.MapInto","doc":"*There are no specific parameters for this check.*","ref":"Credo.Check.Refactor.MapInto.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Refactor.MapInto","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Refactor.MapInto.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Refactor.MapJoin","doc":"This check has a base priority of `high` and works with any version of Elixir.","ref":"Credo.Check.Refactor.MapJoin.html"},{"type":"module","title":"Explanation - Credo.Check.Refactor.MapJoin","doc":"`Enum.map_join/3` is more efficient than `Enum.map/2 |> Enum.join/2`.\n\nThis should be refactored:\n\n    [\"a\", \"b\", \"c\"]\n    |> Enum.map(&String.upcase/1)\n    |> Enum.join(\", \")\n\nto look like this:\n\n    Enum.map_join([\"a\", \"b\", \"c\"], \", \", &String.upcase/1)\n\nThe reason for this is performance, because the two separate calls\nto `Enum.map/2` and `Enum.join/2` require two iterations whereas\n`Enum.map_join/3` performs the same work in one pass.","ref":"Credo.Check.Refactor.MapJoin.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Refactor.MapJoin","doc":"*There are no specific parameters for this check.*","ref":"Credo.Check.Refactor.MapJoin.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Refactor.MapJoin","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Refactor.MapJoin.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Refactor.MapMap","doc":"This check has a base priority of `0` and works with any version of Elixir.","ref":"Credo.Check.Refactor.MapMap.html"},{"type":"module","title":"Explanation - Credo.Check.Refactor.MapMap","doc":"One `Enum.map/2` is more efficient than `Enum.map/2 |> Enum.map/2`.\n\nThis should be refactored:\n\n    [:a, :b, :c]\n    |> Enum.map(&inspect/1)\n    |> Enum.map(&String.upcase/1)\n\nto look like this:\n\n    Enum.map([:a, :b, :c], fn letter ->\n      letter\n      |> inspect()\n      |> String.upcase()\n    end)\n\nThe reason for this is performance, because the two separate calls\nto `Enum.map/2` require two iterations whereas doing the functions\nin the single `Enum.map/2` only requires one.","ref":"Credo.Check.Refactor.MapMap.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Refactor.MapMap","doc":"*There are no specific parameters for this check.*","ref":"Credo.Check.Refactor.MapMap.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Refactor.MapMap","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Refactor.MapMap.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Refactor.MatchInCondition","doc":"This check has a base priority of `0` and works with any version of Elixir.","ref":"Credo.Check.Refactor.MatchInCondition.html"},{"type":"module","title":"Explanation - Credo.Check.Refactor.MatchInCondition","doc":"Pattern matching should only ever be used for simple assignments\ninside `if` and `unless` clauses.\n\nWhile this fine:\n\n    # okay, simple wildcard assignment:\n\n    if contents = File.read!(\"foo.txt\") do\n      do_something(contents)\n    end\n\nthe following should be avoided, since it mixes a pattern match with a\ncondition and do/else blocks.\n\n    # considered too \"complex\":\n\n    if {:ok, contents} = File.read(\"foo.txt\") do\n      do_something(contents)\n    end\n\n    # also considered \"complex\":\n\n    if allowed? && ( contents = File.read!(\"foo.txt\") ) do\n      do_something(contents)\n    end\n\nIf you want to match for something and execute another block otherwise,\nconsider using a `case` statement:\n\n    case File.read(\"foo.txt\") do\n      {:ok, contents} ->\n        do_something()\n      _ ->\n        do_something_else()\n    end","ref":"Credo.Check.Refactor.MatchInCondition.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Refactor.MatchInCondition","doc":"Use the following parameters to configure this check:\n\n### `:allow_tagged_tuples`\n\n  Allow tagged tuples in conditions, e.g. `if {:ok, contents} = File.read( \"foo.txt\") do`\n\n*This parameter defaults to* `false`.","ref":"Credo.Check.Refactor.MatchInCondition.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Refactor.MatchInCondition","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Refactor.MatchInCondition.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Refactor.ModuleDependencies","doc":"This check has a base priority of `normal` and works with any version of Elixir.","ref":"Credo.Check.Refactor.ModuleDependencies.html"},{"type":"module","title":"Explanation - Credo.Check.Refactor.ModuleDependencies","doc":"This module might be doing too much. Consider limiting the number of\nmodule dependencies.\n\nAs always: This is just a suggestion. Check the configuration options for\ntweaking or disabling this check.","ref":"Credo.Check.Refactor.ModuleDependencies.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Refactor.ModuleDependencies","doc":"Use the following parameters to configure this check:\n\n### `:max_deps`\n\n  Maximum number of module dependencies.\n\n*This parameter defaults to* `10`.\n\n### `:dependency_namespaces`\n\n  List of dependency namespaces to include in this check\n\n*This parameter defaults to* `[]`.\n\n### `:excluded_namespaces`\n\n  List of namespaces to exclude from this check\n\n*This parameter defaults to* `[]`.\n\n### `:excluded_paths`\n\n  List of paths or regex to exclude from this check\n\n*This parameter defaults to* `[~r/\\/test\\//, ~r/^test\\//]`.","ref":"Credo.Check.Refactor.ModuleDependencies.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Refactor.ModuleDependencies","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Refactor.ModuleDependencies.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Refactor.NegatedConditionsInUnless","doc":"This check has a base priority of `high` and works with any version of Elixir.","ref":"Credo.Check.Refactor.NegatedConditionsInUnless.html"},{"type":"module","title":"Explanation - Credo.Check.Refactor.NegatedConditionsInUnless","doc":"Unless blocks should avoid having a negated condition.\n\nThe code in this example ...\n\n    unless !allowed? do\n      proceed_as_planned()\n    end\n\n... should be refactored to look like this:\n\n    if allowed? do\n      proceed_as_planned()\n    end\n\nThe reason for this is not a technical but a human one. It is pretty difficult\nto wrap your head around a block of code that is executed if a negated\ncondition is NOT met. See what I mean?","ref":"Credo.Check.Refactor.NegatedConditionsInUnless.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Refactor.NegatedConditionsInUnless","doc":"*There are no specific parameters for this check.*","ref":"Credo.Check.Refactor.NegatedConditionsInUnless.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Refactor.NegatedConditionsInUnless","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Refactor.NegatedConditionsInUnless.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Refactor.NegatedConditionsWithElse","doc":"This check has a base priority of `high` and works with any version of Elixir.","ref":"Credo.Check.Refactor.NegatedConditionsWithElse.html"},{"type":"module","title":"Explanation - Credo.Check.Refactor.NegatedConditionsWithElse","doc":"An `if` block with a negated condition should not contain an else block.\n\nSo while this is fine:\n\n    if not allowed? do\n      raise \"Not allowed!\"\n    end\n\nThe code in this example ...\n\n    if not allowed? do\n      raise \"Not allowed!\"\n    else\n      proceed_as_planned()\n    end\n\n... should be refactored to look like this:\n\n    if allowed? do\n      proceed_as_planned()\n    else\n      raise \"Not allowed!\"\n    end\n\nThe same goes for negation through `!` instead of `not`.\n\nThe reason for this is not a technical but a human one. It is easier to wrap\nyour head around a positive condition and then thinking \"and else we do ...\".\n\nIn the above example raising the error in case something is not allowed\nmight seem so important to put it first. But when you revisit this code a\nwhile later or have to introduce a colleague to it, you might be surprised\nhow much clearer things get when the \"happy path\" comes first.","ref":"Credo.Check.Refactor.NegatedConditionsWithElse.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Refactor.NegatedConditionsWithElse","doc":"*There are no specific parameters for this check.*","ref":"Credo.Check.Refactor.NegatedConditionsWithElse.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Refactor.NegatedConditionsWithElse","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Refactor.NegatedConditionsWithElse.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Refactor.NegatedIsNil","doc":"This check has a base priority of `low` and works with any version of Elixir.","ref":"Credo.Check.Refactor.NegatedIsNil.html"},{"type":"module","title":"Explanation - Credo.Check.Refactor.NegatedIsNil","doc":"We should avoid negating the `is_nil` predicate function.\n\nFor example, the code here ...\n\n    def fun(%{external_id: external_id, id: id}) when not is_nil(external_id) do\n       # ...\n    end\n\n... can be refactored to look like this:\n\n    def fun(%{external_id: nil, id: id}) do\n      # ...\n    end\n\n    def fun(%{external_id: external_id, id: id}) do\n      # ...\n    end\n\n... or even better, can match on what you were expecting on the first place:\n\n    def fun(%{external_id: external_id, id: id}) when is_binary(external_id) do\n      # ...\n    end\n\n    def fun(%{external_id: nil, id: id}) do\n      # ...\n    end\n\n    def fun(%{external_id: external_id, id: id}) do\n      # ...\n    end\n\nSimilar to negating `unless` blocks, the reason for this check is not\ntechnical, but a human one. If we can use the positive, more direct and human\nfriendly case, we should.","ref":"Credo.Check.Refactor.NegatedIsNil.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Refactor.NegatedIsNil","doc":"*There are no specific parameters for this check.*","ref":"Credo.Check.Refactor.NegatedIsNil.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Refactor.NegatedIsNil","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Refactor.NegatedIsNil.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Refactor.Nesting","doc":"This check has a base priority of `0` and works with any version of Elixir.","ref":"Credo.Check.Refactor.Nesting.html"},{"type":"module","title":"Explanation - Credo.Check.Refactor.Nesting","doc":"Code should not be nested more than once inside a function.\n\n    defmodule CredoSampleModule do\n      def some_function(parameter1, parameter2) do\n        Enum.reduce(var1, list, fn({_hash, nodes}, list) ->\n          filenames = nodes |> Enum.map(&(&1.filename))\n\n          Enum.reduce(list, [], fn(item, acc) ->\n            if item.filename do\n              item               # <-- this is nested 3 levels deep\n            end\n            acc ++ [item]\n          end)\n        end)\n      end\n    end\n\nAt this point it might be a good idea to refactor the code to separate the\ndifferent loops and conditions.","ref":"Credo.Check.Refactor.Nesting.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Refactor.Nesting","doc":"Use the following parameters to configure this check:\n\n### `:max_nesting`\n\n  The maximum number of levels code should be nested.\n\n*This parameter defaults to* `2`.","ref":"Credo.Check.Refactor.Nesting.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Refactor.Nesting","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Refactor.Nesting.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Refactor.PassAsyncInTestCases","doc":"This check has a base priority of `normal` and works with any version of Elixir.","ref":"Credo.Check.Refactor.PassAsyncInTestCases.html"},{"type":"module","title":"Explanation - Credo.Check.Refactor.PassAsyncInTestCases","doc":"Test modules marked `async: true` are run concurrently, speeding up the\ntest suite and improving productivity. This should always be done when\npossible.\n\nLeaving off the `async:` option silently defaults to `false`, which may make\na test suite slower for no real reason.\n\nTest modules which cannot be run concurrently should be explicitly marked\n`async: false`, ideally with a comment explaining why.","ref":"Credo.Check.Refactor.PassAsyncInTestCases.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Refactor.PassAsyncInTestCases","doc":"*There are no specific parameters for this check.*","ref":"Credo.Check.Refactor.PassAsyncInTestCases.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Refactor.PassAsyncInTestCases","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Refactor.PassAsyncInTestCases.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Refactor.PerceivedComplexity","doc":"This check has a base priority of `0` and works with any version of Elixir.","ref":"Credo.Check.Refactor.PerceivedComplexity.html"},{"type":"module","title":"Explanation - Credo.Check.Refactor.PerceivedComplexity","doc":"Cyclomatic complexity is a software complexity metric closely correlated with\ncoding errors.\n\nIf a function feels like it's gotten too complex, it more often than not also\nhas a high CC value. So, if anything, this is useful to convince team members\nand bosses of a need to refactor parts of the code based on \"objective\"\nmetrics.","ref":"Credo.Check.Refactor.PerceivedComplexity.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Refactor.PerceivedComplexity","doc":"Use the following parameters to configure this check:\n\n### `:max_complexity`\n\n  The maximum cyclomatic complexity a function should have.\n\n*This parameter defaults to* `9`.","ref":"Credo.Check.Refactor.PerceivedComplexity.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Refactor.PerceivedComplexity","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Refactor.PerceivedComplexity.html#module-general-parameters"},{"type":"function","title":"Credo.Check.Refactor.PerceivedComplexity.complexity_for/1","doc":"Returns the Cyclomatic Complexity score for the block inside the given AST,\nwhich is expected to represent a function or macro definition.\n\n    iex> {:def, [line: 1],\n    ...>   [\n    ...>     {:first_fun, [line: 1], nil},\n    ...>     [do: {:=, [line: 2], [{:x, [line: 2], nil}, 1]}]\n    ...>   ]\n    ...> } |> Credo.Check.Refactor.CyclomaticComplexity.complexity_for\n    1.0","ref":"Credo.Check.Refactor.PerceivedComplexity.html#complexity_for/1"},{"type":"module","title":"Credo.Check.Refactor.PipeChainStart","doc":"This check has a base priority of `0` and works with any version of Elixir.","ref":"Credo.Check.Refactor.PipeChainStart.html"},{"type":"module","title":"Explanation - Credo.Check.Refactor.PipeChainStart","doc":"Pipes (`|>`) can become more readable by starting with a \"raw\" value.\n\nSo while this is easily comprehendable:\n\n    list\n    |> Enum.take(5)\n    |> Enum.shuffle\n    |> pick_winner()\n\nThis might be harder to read:\n\n    Enum.take(list, 5)\n    |> Enum.shuffle\n    |> pick_winner()\n\nAs always: This is just a suggestion. Check the configuration options for\ntweaking or disabling this check.","ref":"Credo.Check.Refactor.PipeChainStart.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Refactor.PipeChainStart","doc":"Use the following parameters to configure this check:\n\n### `:excluded_functions`\n\n  All functions listed will be ignored.\n\n*This parameter defaults to* `[]`.\n\n### `:excluded_argument_types`\n\n  All pipes with argument types listed will be ignored.\n\n*This parameter defaults to* `[]`.","ref":"Credo.Check.Refactor.PipeChainStart.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Refactor.PipeChainStart","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Refactor.PipeChainStart.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Refactor.RedundantWithClauseResult","doc":"This check has a base priority of `high` and works with any version of Elixir.","ref":"Credo.Check.Refactor.RedundantWithClauseResult.html"},{"type":"module","title":"Explanation - Credo.Check.Refactor.RedundantWithClauseResult","doc":"`with` statements are useful when you need to chain a sequence\nof pattern matches, stopping at the first one that fails.\n\nIf the match of the last clause in a `with` statement is identical to the expression in the\nin its body, the code should be refactored to remove the redundant expression.\n\nThis should be refactored:\n\n    with {:ok, map} <- check(input),\n         {:ok, result} <- something(map) do\n      {:ok, result}\n    end\n\nto look like this:\n\n    with {:ok, map} <- check(input) do\n      something(map)\n    end","ref":"Credo.Check.Refactor.RedundantWithClauseResult.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Refactor.RedundantWithClauseResult","doc":"*There are no specific parameters for this check.*","ref":"Credo.Check.Refactor.RedundantWithClauseResult.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Refactor.RedundantWithClauseResult","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Refactor.RedundantWithClauseResult.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Refactor.RejectFilter","doc":"This check has a base priority of `0` and works with any version of Elixir.","ref":"Credo.Check.Refactor.RejectFilter.html"},{"type":"module","title":"Explanation - Credo.Check.Refactor.RejectFilter","doc":"One `Enum.filter/2` is more efficient than `Enum.reject/2 |> Enum.filter/2`.\n\nThis should be refactored:\n\n    [\"a\", \"b\", \"c\"]\n    |> Enum.reject(&String.contains?(&1, \"x\"))\n    |> Enum.filter(&String.contains?(&1, \"a\"))\n\nto look like this:\n\n    Enum.filter([\"a\", \"b\", \"c\"], fn letter ->\n      !String.contains?(letter, \"x\") && String.contains?(letter, \"a\")\n    end)\n\nThe reason for this is performance, because the two calls to\n`Enum.reject/2` and `Enum.filter/2` require two iterations whereas\ndoing the functions in the single `Enum.filter/2` only requires one.","ref":"Credo.Check.Refactor.RejectFilter.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Refactor.RejectFilter","doc":"*There are no specific parameters for this check.*","ref":"Credo.Check.Refactor.RejectFilter.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Refactor.RejectFilter","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Refactor.RejectFilter.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Refactor.RejectReject","doc":"This check has a base priority of `0` and works with any version of Elixir.","ref":"Credo.Check.Refactor.RejectReject.html"},{"type":"module","title":"Explanation - Credo.Check.Refactor.RejectReject","doc":"One `Enum.reject/2` is more efficient than `Enum.reject/2 |> Enum.reject/2`.\n\nThis should be refactored:\n\n    [\"a\", \"b\", \"c\"]\n    |> Enum.reject(&String.contains?(&1, \"x\"))\n    |> Enum.reject(&String.contains?(&1, \"a\"))\n\nto look like this:\n\n    Enum.reject([\"a\", \"b\", \"c\"], fn letter ->\n      String.contains?(letter, \"x\") || String.contains?(letter, \"a\")\n    end)\n\nThe reason for this is performance, because the two separate calls\nto `Enum.reject/2` require two iterations whereas doing the\nfunctions in the single `Enum.reject/2` only requires one.","ref":"Credo.Check.Refactor.RejectReject.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Refactor.RejectReject","doc":"*There are no specific parameters for this check.*","ref":"Credo.Check.Refactor.RejectReject.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Refactor.RejectReject","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Refactor.RejectReject.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Refactor.UnlessWithElse","doc":"This check has a base priority of `high` and works with any version of Elixir.","ref":"Credo.Check.Refactor.UnlessWithElse.html"},{"type":"module","title":"Explanation - Credo.Check.Refactor.UnlessWithElse","doc":"An `unless` block should not contain an else block.\n\nSo while this is fine:\n\n    unless allowed? do\n      raise \"Not allowed!\"\n    end\n\nThis should be refactored:\n\n    unless allowed? do\n      raise \"Not allowed!\"\n    else\n      proceed_as_planned()\n    end\n\nto look like this:\n\n    if allowed? do\n      proceed_as_planned()\n    else\n      raise \"Not allowed!\"\n    end\n\nThe reason for this is not a technical but a human one. The `else` in this\ncase will be executed when the condition is met, which is the opposite of\nwhat the wording seems to imply.","ref":"Credo.Check.Refactor.UnlessWithElse.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Refactor.UnlessWithElse","doc":"*There are no specific parameters for this check.*","ref":"Credo.Check.Refactor.UnlessWithElse.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Refactor.UnlessWithElse","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Refactor.UnlessWithElse.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Refactor.VariableRebinding","doc":"This check has a base priority of `0` and works with any version of Elixir.","ref":"Credo.Check.Refactor.VariableRebinding.html"},{"type":"module","title":"Explanation - Credo.Check.Refactor.VariableRebinding","doc":"You might want to refrain from rebinding variables.\n\nAlthough technically fine, rebinding to the same name can lead to less\nprecise naming.\n\nConsider this example:\n\n    def find_a_good_time do\n      time = MyApp.DateTime.now\n      time = MyApp.DateTime.later(time, 5, :days)\n      {:ok, time} = verify_available_time(time)\n\n      time\n    end\n\nWhile there is nothing wrong with this, many would consider the following\nimplementation to be easier to comprehend:\n\n    def find_a_good_time do\n      today = DateTime.now\n      proposed_time = DateTime.later(today, 5, :days)\n      {:ok, verified_time} = verify_available_time(proposed_time)\n\n      verified_time\n    end\n\nIn some rare cases you might really want to rebind a variable.  This can be\nenabled \"opt-in\" on a per-variable basis by setting the :allow_bang option\nto true and adding a bang suffix sigil to your variable.\n\n    def uses_mutating_parameters(params!) do\n      params! = do_a_thing(params!)\n      params! = do_another_thing(params!)\n      params! = do_yet_another_thing(params!)\n    end","ref":"Credo.Check.Refactor.VariableRebinding.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Refactor.VariableRebinding","doc":"Use the following parameters to configure this check:\n\n### `:allow_bang`\n\n  Variables with a bang suffix will be ignored.\n\n*This parameter defaults to* `false`.","ref":"Credo.Check.Refactor.VariableRebinding.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Refactor.VariableRebinding","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Refactor.VariableRebinding.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Refactor.WithClauses","doc":"This check has a base priority of `high` and works with any version of Elixir.","ref":"Credo.Check.Refactor.WithClauses.html"},{"type":"module","title":"Explanation - Credo.Check.Refactor.WithClauses","doc":"`with` statements are useful when you need to chain a sequence\nof pattern matches, stopping at the first one that fails.\n\nBut sometimes, we go a little overboard with them (pun intended).\n\nIf the first or last clause in a `with` statement is not a `<-` clause,\nit still compiles and works, but is not really utilizing what the `with`\nmacro provides and can be misleading.\n\n    with ref = make_ref(),\n         {:ok, user} <- User.create(ref),\n         :ok <- send_email(user),\n         Logger.debug(\"Created user: #{inspect(user)}\") do\n      user\n    end\n\nHere, both the first and last clause are actually not matching anything.\n\nIf we move them outside of the `with` (the first ones) or inside the body\nof the `with` (the last ones), the code becomes more focused and .\n\nThis `with` should be refactored like this:\n\n    ref = make_ref()\n\n    with {:ok, user} <- User.create(ref),\n         :ok <- send_email(user) do\n      Logger.debug(\"Created user: #{inspect(user)}\")\n      user\n    end","ref":"Credo.Check.Refactor.WithClauses.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Refactor.WithClauses","doc":"*There are no specific parameters for this check.*","ref":"Credo.Check.Refactor.WithClauses.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Refactor.WithClauses","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Refactor.WithClauses.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Warning.ApplicationConfigInModuleAttribute","doc":"This check has a base priority of `high` and works with any version of Elixir.","ref":"Credo.Check.Warning.ApplicationConfigInModuleAttribute.html"},{"type":"module","title":"Explanation - Credo.Check.Warning.ApplicationConfigInModuleAttribute","doc":"Module attributes are evaluated at compile time and not at run time. As\na result, certain configuration read calls made in your module attributes\nmay work as expected during local development, but may break once in a\ndeployed context.\n\nThis check analyzes all of the module attributes present within a module,\nand validates that there are no unsafe calls.\n\nThese unsafe calls include:\n\n- `Application.fetch_env/2`\n- `Application.fetch_env!/2`\n- `Application.get_all_env/1`\n- `Application.get_env/3`\n- `Application.get_env/2`\n\nAs of Elixir 1.10 you can leverage `Application.compile_env/3` and\n`Application.compile_env!/2` if you wish to set configuration at\ncompile time using module attributes.","ref":"Credo.Check.Warning.ApplicationConfigInModuleAttribute.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Warning.ApplicationConfigInModuleAttribute","doc":"*There are no specific parameters for this check.*","ref":"Credo.Check.Warning.ApplicationConfigInModuleAttribute.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Warning.ApplicationConfigInModuleAttribute","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Warning.ApplicationConfigInModuleAttribute.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Warning.BoolOperationOnSameValues","doc":"This check has a base priority of `high` and works with any version of Elixir.","ref":"Credo.Check.Warning.BoolOperationOnSameValues.html"},{"type":"module","title":"Explanation - Credo.Check.Warning.BoolOperationOnSameValues","doc":"Boolean operations with identical values on the left and right side are\nmost probably a logical fallacy or a copy-and-paste error.\n\nExamples:\n\n    x && x\n    x || x\n    x and x\n    x or x\n\nEach of these cases behaves the same as if you were just writing `x`.","ref":"Credo.Check.Warning.BoolOperationOnSameValues.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Warning.BoolOperationOnSameValues","doc":"*There are no specific parameters for this check.*","ref":"Credo.Check.Warning.BoolOperationOnSameValues.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Warning.BoolOperationOnSameValues","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Warning.BoolOperationOnSameValues.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Warning.Dbg","doc":"This check has a base priority of `high` and requires Elixir `>= 1.14.0-dev`.","ref":"Credo.Check.Warning.Dbg.html"},{"type":"module","title":"Explanation - Credo.Check.Warning.Dbg","doc":"Calls to dbg/0 and dbg/2 should mostly be used during debugging sessions.\n\nThis check warns about those calls, because they probably have been committed\nin error.","ref":"Credo.Check.Warning.Dbg.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Warning.Dbg","doc":"*There are no specific parameters for this check.*","ref":"Credo.Check.Warning.Dbg.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Warning.Dbg","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Warning.Dbg.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Warning.ExpensiveEmptyEnumCheck","doc":"This check has a base priority of `high` and works with any version of Elixir.","ref":"Credo.Check.Warning.ExpensiveEmptyEnumCheck.html"},{"type":"module","title":"Explanation - Credo.Check.Warning.ExpensiveEmptyEnumCheck","doc":"Checking if the size of the enum is `0` can be very expensive, since you are\ndetermining the exact count of elements.\n\nChecking if an enum is empty should be done by using\n\n    Enum.empty?(enum)\n\nor\n\n    list == []\n\n\nFor Enum.count/2: Checking if an enum doesn't contain specific elements should\nbe done by using\n\n    not Enum.any?(enum, condition)","ref":"Credo.Check.Warning.ExpensiveEmptyEnumCheck.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Warning.ExpensiveEmptyEnumCheck","doc":"*There are no specific parameters for this check.*","ref":"Credo.Check.Warning.ExpensiveEmptyEnumCheck.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Warning.ExpensiveEmptyEnumCheck","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Warning.ExpensiveEmptyEnumCheck.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Warning.ForbiddenModule","doc":"This check has a base priority of `high` and works with any version of Elixir.","ref":"Credo.Check.Warning.ForbiddenModule.html"},{"type":"module","title":"Explanation - Credo.Check.Warning.ForbiddenModule","doc":"Some modules that are included by a package may be hazardous\nif used by your application. Use this check to allow these modules in\nyour dependencies but forbid them to be used in your application.\n\nExamples:\n\nThe `:ecto_sql` package includes the `Ecto.Adapters.SQL` module,\nbut direct usage of the `Ecto.Adapters.SQL.query/4` function, and related functions, may\ncause issues when using Ecto's dynamic repositories.","ref":"Credo.Check.Warning.ForbiddenModule.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Warning.ForbiddenModule","doc":"Use the following parameters to configure this check:\n\n### `:modules`\n\n  List of modules or `{Module, \"Error message\"}` tuples that must not be used.\n\n*This parameter defaults to* `[]`.","ref":"Credo.Check.Warning.ForbiddenModule.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Warning.ForbiddenModule","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Warning.ForbiddenModule.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Warning.IExPry","doc":"This check has a base priority of `high` and works with any version of Elixir.","ref":"Credo.Check.Warning.IExPry.html"},{"type":"module","title":"Explanation - Credo.Check.Warning.IExPry","doc":"While calls to IEx.pry might appear in some parts of production code,\nmost calls to this function are added during debugging sessions.\n\nThis check warns about those calls, because they might have been committed\nin error.","ref":"Credo.Check.Warning.IExPry.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Warning.IExPry","doc":"*There are no specific parameters for this check.*","ref":"Credo.Check.Warning.IExPry.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Warning.IExPry","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Warning.IExPry.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Warning.IoInspect","doc":"This check has a base priority of `high` and works with any version of Elixir.","ref":"Credo.Check.Warning.IoInspect.html"},{"type":"module","title":"Explanation - Credo.Check.Warning.IoInspect","doc":"While calls to IO.inspect might appear in some parts of production code,\nmost calls to this function are added during debugging sessions.\n\nThis check warns about those calls, because they might have been committed\nin error.","ref":"Credo.Check.Warning.IoInspect.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Warning.IoInspect","doc":"*There are no specific parameters for this check.*","ref":"Credo.Check.Warning.IoInspect.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Warning.IoInspect","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Warning.IoInspect.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Warning.LazyLogging","doc":"This check has a base priority of `high` and requires Elixir `< 1.7.0`.","ref":"Credo.Check.Warning.LazyLogging.html"},{"type":"module","title":"Explanation - Credo.Check.Warning.LazyLogging","doc":"Ensures laziness of Logger calls.\n\nYou will want to wrap expensive logger calls into a zero argument\nfunction (`fn -> \"string that gets logged\" end`).\n\nExample:\n\n    # preferred\n\n    Logger.debug fn ->\n      \"This happened: #{expensive_calculation(arg1, arg2)}\"\n    end\n\n    # NOT preferred\n    # the interpolation is executed whether or not the info is logged\n\n    Logger.debug \"This happened: #{expensive_calculation(arg1, arg2)}\"","ref":"Credo.Check.Warning.LazyLogging.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Warning.LazyLogging","doc":"Use the following parameters to configure this check:\n\n### `:ignore`\n\n  Do not raise an issue for these Logger calls.\n\n*This parameter defaults to* `[:error, :warn, :info]`.","ref":"Credo.Check.Warning.LazyLogging.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Warning.LazyLogging","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Warning.LazyLogging.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Warning.LeakyEnvironment","doc":"This check has a base priority of `high` and works with any version of Elixir.","ref":"Credo.Check.Warning.LeakyEnvironment.html"},{"type":"module","title":"Explanation - Credo.Check.Warning.LeakyEnvironment","doc":"OS child processes inherit the environment of their parent process. This\nincludes sensitive configuration parameters, such as credentials. To\nminimize the risk of such values leaking, clear or overwrite them when\nspawning executables.\n\nThe functions `System.cmd/2` and `System.cmd/3` allow environment variables be cleared by\nsetting their value to `nil`:\n\n    System.cmd(\"env\", [], env: %{\"DB_PASSWORD\" => nil})","ref":"Credo.Check.Warning.LeakyEnvironment.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Warning.LeakyEnvironment","doc":"*There are no specific parameters for this check.*","ref":"Credo.Check.Warning.LeakyEnvironment.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Warning.LeakyEnvironment","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Warning.LeakyEnvironment.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Warning.MapGetUnsafePass","doc":"This check has a base priority of `normal` and works with any version of Elixir.","ref":"Credo.Check.Warning.MapGetUnsafePass.html"},{"type":"module","title":"Explanation - Credo.Check.Warning.MapGetUnsafePass","doc":"`Map.get/2` can lead into runtime errors if the result is passed into a pipe\nwithout a proper default value. This happens when the next function in the\npipe cannot handle `nil` values correctly.\n\nExample:\n\n    %{foo: [1, 2 ,3], bar: [4, 5, 6]}\n    |> Map.get(:missing_key)\n    |> Enum.each(&IO.puts/1)\n\nThis will cause a `Protocol.UndefinedError`, since `nil` isn't `Enumerable`.\nOften times while iterating over enumerables zero iterations is preferable\nto being forced to deal with an exception. Had there been a `[]` default\nparameter this could have been averted.\n\nIf you are sure the value exists and can't be nil, please use `Map.fetch!/2`.\nIf you are not sure, `Map.get/3` can help you provide a safe default value.","ref":"Credo.Check.Warning.MapGetUnsafePass.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Warning.MapGetUnsafePass","doc":"*There are no specific parameters for this check.*","ref":"Credo.Check.Warning.MapGetUnsafePass.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Warning.MapGetUnsafePass","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Warning.MapGetUnsafePass.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Warning.MissedMetadataKeyInLoggerConfig","doc":"This check has a base priority of `high` and works with any version of Elixir.","ref":"Credo.Check.Warning.MissedMetadataKeyInLoggerConfig.html"},{"type":"module","title":"Explanation - Credo.Check.Warning.MissedMetadataKeyInLoggerConfig","doc":"Ensures custom metadata keys are included in logger config.\n\nNote that all metadata is optional and may not always be available.\n\nFor example, you might wish to include a custom `:error_code` metadata in your logs:\n\n    Logger.error(\"We have a problem\", [error_code: :pc_load_letter])\n\nIn your app's logger configuration, you would need to include the `:error_code` key:\n\n    config :logger, :default_formatter,\n      format: \"[$level] $message $metadata\\n\",\n      metadata: [:error_code, :file]\n\nThat way your logs might then receive lines like this:\n\n    [error] We have a problem error_code=pc_load_letter file=lib/app.ex\n\nIf you want to allow any metadata to be printed, you can use `:all` in the logger's\nmetadata config.","ref":"Credo.Check.Warning.MissedMetadataKeyInLoggerConfig.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Warning.MissedMetadataKeyInLoggerConfig","doc":"Use the following parameters to configure this check:\n\n### `:ignore_logger_functions`\n\n  Do not raise an issue for these Logger functions.\n\n*This parameter defaults to* `nil`.\n\n### `:metadata_keys`\n\n  Do not raise an issue for these Logger metadata keys.\n  \n  By default, we read the metadata keys configured as the current environment's\n  `:default_formatter` (or `:console` for older versions of Elixir).\n  \n  You can use this parameter to dynamically load the environment/backend you care about,\n  via `.credo.exs` (e.g. reading the `:file_log` config from `config/prod.exs`):\n  \n      {Credo.Check.Warning.MissedMetadataKeyInLoggerConfig,\n        [\n          metadata_keys:\n            \"config/prod.exs\"\n            |> Config.Reader.read!()\n            |> get_in([:logger, :file_log, :metadata])\n        ]}\n  \n\n*This parameter defaults to* `[]`.","ref":"Credo.Check.Warning.MissedMetadataKeyInLoggerConfig.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Warning.MissedMetadataKeyInLoggerConfig","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Warning.MissedMetadataKeyInLoggerConfig.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Warning.MixEnv","doc":"This check has a base priority of `high` and works with any version of Elixir.","ref":"Credo.Check.Warning.MixEnv.html"},{"type":"module","title":"Explanation - Credo.Check.Warning.MixEnv","doc":"Mix is a build tool and, as such, it is not expected to be available in production.\nTherefore, it is recommended to access Mix.env only in configuration files and inside\nmix.exs, never in your application code (lib).\n\n(from the Elixir docs)","ref":"Credo.Check.Warning.MixEnv.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Warning.MixEnv","doc":"Use the following parameters to configure this check:\n\n### `:excluded_paths`\n\n  List of paths or regex to exclude from this check\n\n*This parameter defaults to* `[]`.","ref":"Credo.Check.Warning.MixEnv.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Warning.MixEnv","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Warning.MixEnv.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Warning.OperationOnSameValues","doc":"This check has a base priority of `high` and works with any version of Elixir.","ref":"Credo.Check.Warning.OperationOnSameValues.html"},{"type":"module","title":"Explanation - Credo.Check.Warning.OperationOnSameValues","doc":"Operations on the same values always yield the same result and therefore make\nlittle sense in production code.\n\nExamples:\n\n    x == x  # always returns true\n    x  = x  # always returns true\n    x != x  # always returns false\n    x > x   # always returns false\n    y / y   # always returns 1\n    y - y   # always returns 0\n\nIn practice they are likely the result of a debugging session or were made by\nmistake.","ref":"Credo.Check.Warning.OperationOnSameValues.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Warning.OperationOnSameValues","doc":"*There are no specific parameters for this check.*","ref":"Credo.Check.Warning.OperationOnSameValues.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Warning.OperationOnSameValues","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Warning.OperationOnSameValues.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Warning.OperationWithConstantResult","doc":"This check has a base priority of `high` and works with any version of Elixir.","ref":"Credo.Check.Warning.OperationWithConstantResult.html"},{"type":"module","title":"Explanation - Credo.Check.Warning.OperationWithConstantResult","doc":"Some numerical operations always yield the same result and therefore make\nlittle sense in production code.\n\nExamples:\n\n    x * 1   # always returns x\n    x * 0   # always returns 0\n\nIn practice they are likely the result of a debugging session or were made by\nmistake.","ref":"Credo.Check.Warning.OperationWithConstantResult.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Warning.OperationWithConstantResult","doc":"*There are no specific parameters for this check.*","ref":"Credo.Check.Warning.OperationWithConstantResult.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Warning.OperationWithConstantResult","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Warning.OperationWithConstantResult.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Warning.RaiseInsideRescue","doc":"This check has a base priority of `0` and works with any version of Elixir.","ref":"Credo.Check.Warning.RaiseInsideRescue.html"},{"type":"module","title":"Explanation - Credo.Check.Warning.RaiseInsideRescue","doc":"Using `Kernel.raise` inside of a `rescue` block creates a new stacktrace.\n\nMost of the time, this is not what you want to do since it obscures the cause of the original error.\n\nExample:\n\n    # preferred\n\n    try do\n      raise \"oops\"\n    rescue\n      error ->\n        Logger.warn(\"An exception has occurred\")\n\n        reraise error, System.stacktrace\n    end\n\n    # NOT preferred\n\n    try do\n      raise \"oops\"\n    rescue\n      error ->\n        Logger.warn(\"An exception has occurred\")\n\n        raise error\n    end","ref":"Credo.Check.Warning.RaiseInsideRescue.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Warning.RaiseInsideRescue","doc":"*There are no specific parameters for this check.*","ref":"Credo.Check.Warning.RaiseInsideRescue.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Warning.RaiseInsideRescue","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Warning.RaiseInsideRescue.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Warning.SpecWithStruct","doc":"This check has a base priority of `normal` and works with any version of Elixir.","ref":"Credo.Check.Warning.SpecWithStruct.html"},{"type":"module","title":"Explanation - Credo.Check.Warning.SpecWithStruct","doc":"Structs create compile-time dependencies between modules.  Using a struct in a spec\nwill cause the module to be recompiled whenever the struct's module changes.\n\nIt is preferable to define and use `MyModule.t()` instead of `%MyModule{}` in specs.\n\nExample:\n\n    # preferred\n    @spec a_function(MyModule.t()) :: any\n\n    # NOT preferred\n    @spec a_function(%MyModule{}) :: any","ref":"Credo.Check.Warning.SpecWithStruct.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Warning.SpecWithStruct","doc":"*There are no specific parameters for this check.*","ref":"Credo.Check.Warning.SpecWithStruct.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Warning.SpecWithStruct","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Warning.SpecWithStruct.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Warning.UnsafeExec","doc":"This check has a base priority of `high` and works with any version of Elixir.","ref":"Credo.Check.Warning.UnsafeExec.html"},{"type":"module","title":"Explanation - Credo.Check.Warning.UnsafeExec","doc":"Spawning external commands can lead to command injection vulnerabilities.\n\nUse a safe API where arguments are passed as an explicit list, rather\nthan unsafe APIs that run a shell to parse the arguments from a single\nstring.\n\nSafe APIs include:\n\n  * `System.cmd/2,3`\n  * `:erlang.open_port/2`, passing `{:spawn_executable, file_name}` as the\n    first parameter, and any arguments using the `:args` option\n\nUnsafe APIs include:\n\n  * `:os.cmd/1,2`\n  * `:erlang.open_port/2`, passing `{:spawn, command}` as the first\n    parameter","ref":"Credo.Check.Warning.UnsafeExec.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Warning.UnsafeExec","doc":"*There are no specific parameters for this check.*","ref":"Credo.Check.Warning.UnsafeExec.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Warning.UnsafeExec","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Warning.UnsafeExec.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Warning.UnsafeToAtom","doc":"This check has a base priority of `high` and works with any version of Elixir.","ref":"Credo.Check.Warning.UnsafeToAtom.html"},{"type":"module","title":"Explanation - Credo.Check.Warning.UnsafeToAtom","doc":"Creating atoms from unknown or external sources dynamically is a potentially\nunsafe operation because atoms are not garbage-collected by the runtime.\n\nCreating an atom from a string or charlist should be done by using\n\n    String.to_existing_atom(string)\n\nor\n\n    List.to_existing_atom(charlist)\n\nModule aliases should be constructed using\n\n    Module.safe_concat(prefix, suffix)\n\nor\n\n    Module.safe_concat([prefix, infix, suffix])\n\nJason.decode/Jason.decode! should be called using `keys: :atoms!` (*not* `keys: :atoms`):\n\n    Jason.decode(str, keys: :atoms!)\n\nor `:keys` should be omitted (which defaults to `:strings`):\n\n    Jason.decode(str)","ref":"Credo.Check.Warning.UnsafeToAtom.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Warning.UnsafeToAtom","doc":"*There are no specific parameters for this check.*","ref":"Credo.Check.Warning.UnsafeToAtom.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Warning.UnsafeToAtom","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Warning.UnsafeToAtom.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Warning.UnusedEnumOperation","doc":"This check has a base priority of `high` and works with any version of Elixir.","ref":"Credo.Check.Warning.UnusedEnumOperation.html"},{"type":"module","title":"Explanation - Credo.Check.Warning.UnusedEnumOperation","doc":"With the exception of `Enum.each/2`, the result of a call to the\nEnum module's functions has to be used.\n\nWhile this is correct ...\n\n    def prepend_my_username(my_username, usernames) do\n      valid_usernames = Enum.reject(usernames, &is_nil/1)\n\n      [my_username] ++ valid_usernames\n    end\n\n... we forgot to save the downcased username in this example:\n\n    # This is bad because it does not modify the usernames variable!\n\n    def prepend_my_username(my_username, usernames) do\n      Enum.reject(usernames, &is_nil/1)\n\n      [my_username] ++ valid_usernames\n    end\n\nSince Elixir variables are immutable, Enum operations never work on the\nvariable you pass in, but return a new variable which has to be used somehow\n(the exception being `Enum.each/2` which iterates a list and returns `:ok`).","ref":"Credo.Check.Warning.UnusedEnumOperation.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Warning.UnusedEnumOperation","doc":"*There are no specific parameters for this check.*","ref":"Credo.Check.Warning.UnusedEnumOperation.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Warning.UnusedEnumOperation","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Warning.UnusedEnumOperation.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Warning.UnusedFileOperation","doc":"This check has a base priority of `high` and works with any version of Elixir.","ref":"Credo.Check.Warning.UnusedFileOperation.html"},{"type":"module","title":"Explanation - Credo.Check.Warning.UnusedFileOperation","doc":"The result of a call to the File module's functions has to be used.\n\nWhile this is correct ...\n\n    def read_from_cwd(filename) do\n      # TODO: use Path.join/2\n      filename = File.cwd!()   \"/\"   filename\n\n      File.read(filename)\n    end\n\n... we forgot to save the result in this example:\n\n    def read_from_cwd(filename) do\n      File.cwd!()   \"/\"   filename\n\n      File.read(filename)\n    end\n\nSince Elixir variables are immutable, many File operations don't work on the\nvariable you pass in, but return a new variable which has to be used somehow.","ref":"Credo.Check.Warning.UnusedFileOperation.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Warning.UnusedFileOperation","doc":"*There are no specific parameters for this check.*","ref":"Credo.Check.Warning.UnusedFileOperation.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Warning.UnusedFileOperation","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Warning.UnusedFileOperation.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Warning.UnusedKeywordOperation","doc":"This check has a base priority of `high` and works with any version of Elixir.","ref":"Credo.Check.Warning.UnusedKeywordOperation.html"},{"type":"module","title":"Explanation - Credo.Check.Warning.UnusedKeywordOperation","doc":"The result of a call to the Keyword module's functions has to be used.\n\nWhile this is correct ...\n\n    def clean_and_verify_options!(keywords) do\n      keywords = Keyword.delete(keywords, :debug)\n\n      if Enum.length(keywords) == 0, do: raise \"OMG!!!1\"\n\n      keywords\n    end\n\n... we forgot to save the result in this example:\n\n    def clean_and_verify_options!(keywords) do\n      Keyword.delete(keywords, :debug)\n\n      if Enum.length(keywords) == 0, do: raise \"OMG!!!1\"\n\n      keywords\n    end\n\nKeyword operations never work on the variable you pass in, but return a new\nvariable which has to be used somehow.","ref":"Credo.Check.Warning.UnusedKeywordOperation.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Warning.UnusedKeywordOperation","doc":"*There are no specific parameters for this check.*","ref":"Credo.Check.Warning.UnusedKeywordOperation.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Warning.UnusedKeywordOperation","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Warning.UnusedKeywordOperation.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Warning.UnusedListOperation","doc":"This check has a base priority of `high` and works with any version of Elixir.","ref":"Credo.Check.Warning.UnusedListOperation.html"},{"type":"module","title":"Explanation - Credo.Check.Warning.UnusedListOperation","doc":"The result of a call to the List module's functions has to be used.\n\nWhile this is correct ...\n\n    def sort_usernames(usernames) do\n      usernames = List.flatten(usernames)\n\n      List.sort(usernames)\n    end\n\n... we forgot to save the result in this example:\n\n    def sort_usernames(usernames) do\n      List.flatten(usernames)\n\n      List.sort(usernames)\n    end\n\nList operations never work on the variable you pass in, but return a new\nvariable which has to be used somehow.","ref":"Credo.Check.Warning.UnusedListOperation.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Warning.UnusedListOperation","doc":"*There are no specific parameters for this check.*","ref":"Credo.Check.Warning.UnusedListOperation.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Warning.UnusedListOperation","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Warning.UnusedListOperation.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Warning.UnusedOperation","doc":"","ref":"Credo.Check.Warning.UnusedOperation.html"},{"type":"module","title":"Credo.Check.Warning.UnusedPathOperation","doc":"This check has a base priority of `high` and works with any version of Elixir.","ref":"Credo.Check.Warning.UnusedPathOperation.html"},{"type":"module","title":"Explanation - Credo.Check.Warning.UnusedPathOperation","doc":"The result of a call to the Path module's functions has to be used.\n\nWhile this is correct ...\n\n    def read_from_cwd(filename) do\n      filename = Path.join(cwd, filename)\n\n      File.read(filename)\n    end\n\n... we forgot to save the result in this example:\n\n    def read_from_cwd(filename) do\n      Path.join(cwd, filename)\n\n      File.read(filename)\n    end\n\nPath operations never work on the variable you pass in, but return a new\nvariable which has to be used somehow.","ref":"Credo.Check.Warning.UnusedPathOperation.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Warning.UnusedPathOperation","doc":"*There are no specific parameters for this check.*","ref":"Credo.Check.Warning.UnusedPathOperation.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Warning.UnusedPathOperation","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Warning.UnusedPathOperation.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Warning.UnusedRegexOperation","doc":"This check has a base priority of `high` and works with any version of Elixir.","ref":"Credo.Check.Warning.UnusedRegexOperation.html"},{"type":"module","title":"Explanation - Credo.Check.Warning.UnusedRegexOperation","doc":"The result of a call to the Regex module's functions has to be used.\n\nWhile this is correct ...\n\n    def extract_username_and_salute(regex, string) do\n      [string] = Regex.run(regex, string)\n\n      \"Hi #{string}\"\n    end\n\n... we forgot to save the downcased username in this example:\n\n    def extract_username_and_salute(regex, string) do\n      Regex.run(regex, string)\n\n      \"Hi #{string}\"\n    end\n\nRegex operations never work on the variable you pass in, but return a new\nvariable which has to be used somehow.","ref":"Credo.Check.Warning.UnusedRegexOperation.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Warning.UnusedRegexOperation","doc":"*There are no specific parameters for this check.*","ref":"Credo.Check.Warning.UnusedRegexOperation.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Warning.UnusedRegexOperation","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Warning.UnusedRegexOperation.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Warning.UnusedStringOperation","doc":"This check has a base priority of `high` and works with any version of Elixir.","ref":"Credo.Check.Warning.UnusedStringOperation.html"},{"type":"module","title":"Explanation - Credo.Check.Warning.UnusedStringOperation","doc":"The result of a call to the String module's functions has to be used.\n\nWhile this is correct ...\n\n    def salutation(username) do\n      username = String.downcase(username)\n\n      \"Hi #{username}\"\n    end\n\n... we forgot to save the downcased username in this example:\n\n    # This is bad because it does not modify the username variable!\n\n    def salutation(username) do\n      String.downcase(username)\n\n      \"Hi #{username}\"\n    end\n\nSince Elixir variables are immutable, String operations never work on the\nvariable you pass in, but return a new variable which has to be used somehow.","ref":"Credo.Check.Warning.UnusedStringOperation.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Warning.UnusedStringOperation","doc":"*There are no specific parameters for this check.*","ref":"Credo.Check.Warning.UnusedStringOperation.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Warning.UnusedStringOperation","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Warning.UnusedStringOperation.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Warning.UnusedTupleOperation","doc":"This check has a base priority of `high` and works with any version of Elixir.","ref":"Credo.Check.Warning.UnusedTupleOperation.html"},{"type":"module","title":"Explanation - Credo.Check.Warning.UnusedTupleOperation","doc":"The result of a call to the Tuple module's functions has to be used.\n\nWhile this is correct ...\n\n    def remove_magic_item!(tuple) do\n      tuple = Tuple.delete_at(tuple, 0)\n\n      if Enum.length(tuple) == 0, do: raise \"OMG!!!1\"\n\n      tuple\n    end\n\n... we forgot to save the result in this example:\n\n    def remove_magic_item!(tuple) do\n      Tuple.delete_at(tuple, 0)\n\n      if Enum.length(tuple) == 0, do: raise \"OMG!!!1\"\n\n      tuple\n    end\n\nTuple operations never work on the variable you pass in, but return a new\nvariable which has to be used somehow.","ref":"Credo.Check.Warning.UnusedTupleOperation.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Warning.UnusedTupleOperation","doc":"*There are no specific parameters for this check.*","ref":"Credo.Check.Warning.UnusedTupleOperation.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Warning.UnusedTupleOperation","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Warning.UnusedTupleOperation.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Warning.WrongTestFileExtension","doc":"This check has a base priority of `high` and works with any version of Elixir.","ref":"Credo.Check.Warning.WrongTestFileExtension.html"},{"type":"module","title":"Explanation - Credo.Check.Warning.WrongTestFileExtension","doc":"Invoking mix test from the command line will run the tests in each file\nmatching the pattern `*_test.exs` found in the test directory of your project.\n\n(from the `ex_unit` docs)\n\nThis check ensures that test files are not ending with `.ex` (which would cause them to be skipped).","ref":"Credo.Check.Warning.WrongTestFileExtension.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Warning.WrongTestFileExtension","doc":"*There are no specific parameters for this check.*","ref":"Credo.Check.Warning.WrongTestFileExtension.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Warning.WrongTestFileExtension","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Warning.WrongTestFileExtension.html#module-general-parameters"},{"type":"behaviour","title":"Credo.Check.Consistency.Collector","doc":"A behavior for modules that walk through source files and\nidentify consistency issues.\n\nWhen defining a consistency check, you would typically use\nthis structure for the main module, responsible\nfor formatting issue messages:\n\n    defmodule Credo.Check.Consistency.SomeCheck do\n      use Credo.Check, run_on_all: true\n\n      @collector Credo.Check.Consistency.SomeCheck.Collector\n\n      def run(source_files, exec, params) when is_list(source_files) do\n        issue_formatter = &issues_for/3\n\n        @collector.find_and_append_issues(source_files, exec, params, issue_formatter)\n      end\n\n      defp issues_for(expected, source_file, params) do\n        issue_meta = IssueMeta.for(source_file, params)\n        issue_locations =\n          @collector.find_locations_not_matching(expected, source_file)\n\n        Enum.map(issue_locations, fn(location) ->\n          format_issue issue_meta, message: ... # write an issue message\n        end)\n      end\n\nThe actual analysis would be performed by another module\nimplementing the `Credo.Check.Consistency.Collector` behavior:\n\n    defmodule Credo.Check.Consistency.SomeCheck.Collector do\n      use Credo.Check.Consistency.Collector\n\n      def collect_matches(source_file, params) do\n        # ...\n      end\n\n      def find_locations_not_matching(expected, source_file) do\n        # ...\n      end\n    end\n\nRead further for more information on `collect_matches/2`,\n`find_locations_not_matching/2`, and `issue_formatter`.","ref":"Credo.Check.Consistency.Collector.html"},{"type":"function","title":"Credo.Check.Consistency.Collector.append_issue_via_issue_service/2","doc":"","ref":"Credo.Check.Consistency.Collector.html#append_issue_via_issue_service/2"},{"type":"callback","title":"Credo.Check.Consistency.Collector.collect_matches/2","doc":"When you call `@collector.find_and_append_issues/4` inside the check module,\nthe collector first counts the occurrences of different matches\n(e.g. :with_space and :without_space for a space around operators check)\nper each source file.\n\n`collect_matches/2` produces a map of matches as keys and their frequencies\nas values (e.g. %{with_space: 50, without_space: 40}).\n\nThe maps for individual source files are then merged, producing a map\nthat reflects frequency trends for the whole codebase.","ref":"Credo.Check.Consistency.Collector.html#c:collect_matches/2"},{"type":"function","title":"Credo.Check.Consistency.Collector.find_issues/5","doc":"","ref":"Credo.Check.Consistency.Collector.html#find_issues/5"},{"type":"callback","title":"Credo.Check.Consistency.Collector.find_locations_not_matching/2","doc":"`issue_formatter` may call the `@collector.find_locations_not_matching/2`\nfunction to obtain additional metadata for each occurrence of\nan unexpected match in a given file.\n\nAn example implementation that returns a list of line numbers on\nwhich unexpected occurrences were found:\n\n    def find_locations_not_matching(expected, source_file) do\n      traverse(source_file, fn(match, line_no, acc) ->\n        if match != expected do\n          acc ++ [line_no]\n        else\n          acc\n        end\n      end)\n    end\n\n    defp traverse(source_file, fun), do: ...","ref":"Credo.Check.Consistency.Collector.html#c:find_locations_not_matching/2"},{"type":"type","title":"Credo.Check.Consistency.Collector.issue_formatter/0","doc":"","ref":"Credo.Check.Consistency.Collector.html#t:issue_formatter/0"},{"type":"module","title":"Credo.Check.Consistency.ExceptionNames","doc":"This check has a base priority of `high` and works with any version of Elixir.","ref":"Credo.Check.Consistency.ExceptionNames.html"},{"type":"module","title":"Explanation - Credo.Check.Consistency.ExceptionNames","doc":"Exception names should end with a common suffix like \"Error\".\n\nTry to name your exception modules consistently:\n\n    defmodule BadCodeError do\n      defexception [:message]\n    end\n\n    defmodule ParserError do\n      defexception [:message]\n    end\n\nInconsistent use should be avoided:\n\n    defmodule BadHTTPResponse do\n      defexception [:message]\n    end\n\n    defmodule HTTPHeaderException do\n      defexception [:message]\n    end\n\nWhile this is not necessarily a concern for the correctness of your code,\nyou should use a consistent style throughout your codebase.","ref":"Credo.Check.Consistency.ExceptionNames.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Consistency.ExceptionNames","doc":"*There are no specific parameters for this check.*","ref":"Credo.Check.Consistency.ExceptionNames.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Consistency.ExceptionNames","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Consistency.ExceptionNames.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Consistency.LineEndings","doc":"This check has a base priority of `high` and works with any version of Elixir.","ref":"Credo.Check.Consistency.LineEndings.html"},{"type":"module","title":"Explanation - Credo.Check.Consistency.LineEndings","doc":"Windows and Linux/macOS systems use different line-endings in files.\n\nIt seems like a good idea not to mix these in the same codebase.\n\nWhile this is not necessarily a concern for the correctness of your code,\nyou should use a consistent style throughout your codebase.","ref":"Credo.Check.Consistency.LineEndings.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Consistency.LineEndings","doc":"Use the following parameters to configure this check:\n\n### `:force`\n\n  Force a choice, values can be `:unix` or `:windows`.\n\n*This parameter defaults to* `nil`.","ref":"Credo.Check.Consistency.LineEndings.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Consistency.LineEndings","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Consistency.LineEndings.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Consistency.MultiAliasImportRequireUse","doc":"This check has a base priority of `high` and works with any version of Elixir.","ref":"Credo.Check.Consistency.MultiAliasImportRequireUse.html"},{"type":"module","title":"Explanation - Credo.Check.Consistency.MultiAliasImportRequireUse","doc":"When using alias, import, require or use for multiple names from the same\nnamespace, you have two options:\n\nUse single instructions per name:\n\n    alias Ecto.Query\n    alias Ecto.Schema\n    alias Ecto.Multi\n\nor use one multi instruction per namespace:\n\n    alias Ecto.{Query, Schema, Multi}\n\nWhile this is not necessarily a concern for the correctness of your code,\nyou should use a consistent style throughout your codebase.","ref":"Credo.Check.Consistency.MultiAliasImportRequireUse.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Consistency.MultiAliasImportRequireUse","doc":"*There are no specific parameters for this check.*","ref":"Credo.Check.Consistency.MultiAliasImportRequireUse.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Consistency.MultiAliasImportRequireUse","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Consistency.MultiAliasImportRequireUse.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Consistency.ParameterPatternMatching","doc":"This check has a base priority of `high` and works with any version of Elixir.","ref":"Credo.Check.Consistency.ParameterPatternMatching.html"},{"type":"module","title":"Explanation - Credo.Check.Consistency.ParameterPatternMatching","doc":"When capturing a parameter using pattern matching you can either put the parameter name before or after the value\ni.e.\n\n    def parse({:ok, values} = pair)\n\nor\n\n    def parse(pair = {:ok, values})\n\nNeither of these is better than the other, but it seems a good idea not to mix the two patterns in the same codebase.\n\nWhile this is not necessarily a concern for the correctness of your code,\nyou should use a consistent style throughout your codebase.","ref":"Credo.Check.Consistency.ParameterPatternMatching.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Consistency.ParameterPatternMatching","doc":"Use the following parameters to configure this check:\n\n### `:force`\n\n  Force a choice, values can be `:after` or `:before`.\n\n*This parameter defaults to* `nil`.","ref":"Credo.Check.Consistency.ParameterPatternMatching.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Consistency.ParameterPatternMatching","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Consistency.ParameterPatternMatching.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Consistency.SpaceAroundOperators","doc":"This check has a base priority of `high` and works with any version of Elixir.","ref":"Credo.Check.Consistency.SpaceAroundOperators.html"},{"type":"module","title":"Explanation - Credo.Check.Consistency.SpaceAroundOperators","doc":"Use spaces around operators like `+`, `-`, `*` and `/`. This is the\n**preferred** way, although other styles are possible, as long as it is\napplied consistently.\n\n    # preferred\n\n    1 + 2 * 4\n\n    # also okay\n\n    1+2*4\n\nWhile this is not necessarily a concern for the correctness of your code,\nyou should use a consistent style throughout your codebase.","ref":"Credo.Check.Consistency.SpaceAroundOperators.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Consistency.SpaceAroundOperators","doc":"Use the following parameters to configure this check:\n\n### `:ignore`\n\n  List of operators to be ignored for this check.\n\n*This parameter defaults to* `[:|]`.","ref":"Credo.Check.Consistency.SpaceAroundOperators.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Consistency.SpaceAroundOperators","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Consistency.SpaceAroundOperators.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Consistency.SpaceInParentheses","doc":"This check has a base priority of `high` and works with any version of Elixir.","ref":"Credo.Check.Consistency.SpaceInParentheses.html"},{"type":"module","title":"Explanation - Credo.Check.Consistency.SpaceInParentheses","doc":"Don't use spaces after `(`, `[`, and `{` or before `}`, `]`, and `)`. This is\nthe **preferred** way, although other styles are possible, as long as it is\napplied consistently.\n\n    # preferred\n\n    Helper.format({1, true, 2}, :my_atom)\n\n    # also okay\n\n    Helper.format( { 1, true, 2 }, :my_atom )\n\nWhile this is not necessarily a concern for the correctness of your code,\nyou should use a consistent style throughout your codebase.","ref":"Credo.Check.Consistency.SpaceInParentheses.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Consistency.SpaceInParentheses","doc":"Use the following parameters to configure this check:\n\n### `:allow_empty_enums`\n\n  Allows [], %{} and similar empty enum values to be used regardless of spacing throughout the codebase.\n\n*This parameter defaults to* `false`.","ref":"Credo.Check.Consistency.SpaceInParentheses.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Consistency.SpaceInParentheses","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Consistency.SpaceInParentheses.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Consistency.TabsOrSpaces","doc":"This check has a base priority of `high` and works with any version of Elixir.","ref":"Credo.Check.Consistency.TabsOrSpaces.html"},{"type":"module","title":"Explanation - Credo.Check.Consistency.TabsOrSpaces","doc":"Tabs should be used consistently.\n\nNOTE: This check does not verify the indentation depth, but checks whether\nor not soft/hard tabs are used consistently across all source files.\n\nIt is very common to use 2 spaces wide soft-tabs, but that is not a strict\nrequirement and you can use hard-tabs if you like that better.\n\nWhile this is not necessarily a concern for the correctness of your code,\nyou should use a consistent style throughout your codebase.","ref":"Credo.Check.Consistency.TabsOrSpaces.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Consistency.TabsOrSpaces","doc":"Use the following parameters to configure this check:\n\n### `:force`\n\n  Force a choice, values can be `:spaces` or `:tabs`.\n\n*This parameter defaults to* `nil`.","ref":"Credo.Check.Consistency.TabsOrSpaces.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Consistency.TabsOrSpaces","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Consistency.TabsOrSpaces.html#module-general-parameters"},{"type":"module","title":"Credo.Check.Consistency.UnusedVariableNames","doc":"This check has a base priority of `high` and works with any version of Elixir.","ref":"Credo.Check.Consistency.UnusedVariableNames.html"},{"type":"module","title":"Explanation - Credo.Check.Consistency.UnusedVariableNames","doc":"Elixir allows us to use `_` as a name for variables that are not meant to be\nused. But it’s a common practice to give these variables meaningful names\nanyway (`_user` instead of `_`), but some people prefer to name them all anonymously (`_`).\n\nA single style should be present in the same codebase.","ref":"Credo.Check.Consistency.UnusedVariableNames.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - Credo.Check.Consistency.UnusedVariableNames","doc":"Use the following parameters to configure this check:\n\n### `:force`\n\n  Force a choice, values can be `:meaningful` or `:anonymous`.\n\n*This parameter defaults to* `nil`.","ref":"Credo.Check.Consistency.UnusedVariableNames.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - Credo.Check.Consistency.UnusedVariableNames","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"Credo.Check.Consistency.UnusedVariableNames.html#module-general-parameters"},{"type":"module","title":"Credo.CLI","doc":"`Credo.CLI` is the entrypoint for both the Mix task and the escript.","ref":"Credo.CLI.html"},{"type":"function","title":"Credo.CLI.main/1","doc":"Runs Credo with the given `argv` and exits the process.\n\nSee `Credo.run/1` if you want to run Credo programmatically.","ref":"Credo.CLI.html#main/1"},{"type":"behaviour","title":"Credo.CLI.Command","doc":"`Command` is used to describe commands which can be executed from the command line.\n\nThe default command is `Credo.CLI.Command.Suggest.SuggestCommand`.\n\nA basic command that writes \"Hello World\" can be implemented like this:\n\n    defmodule HelloWorldCommand do\n      use Credo.CLI.Command\n\n      alias Credo.CLI.Output.UI\n\n      def call(_exec, _opts) do\n        UI.puts([:yellow, \"Hello \", :orange, \"World\"])\n      end\n    end","ref":"Credo.CLI.Command.html"},{"type":"callback","title":"Credo.CLI.Command.call/1","doc":"Is called when a Command is invoked.\n\n    defmodule FooTask do\n      use Credo.Execution.Task\n\n      def call(exec) do\n        IO.inspect(exec)\n      end\n    end\n\nThe `call/1` functions receives an `exec` struct and must return a (modified) `Credo.Execution`.","ref":"Credo.CLI.Command.html#c:call/1"},{"type":"callback","title":"Credo.CLI.Command.call/2","doc":"Runs the Command","ref":"Credo.CLI.Command.html#c:call/2"},{"type":"callback","title":"Credo.CLI.Command.cli_switches/0","doc":"","ref":"Credo.CLI.Command.html#c:cli_switches/0"},{"type":"callback","title":"Credo.CLI.Command.init/1","doc":"Is called when a Command is initialized.\n\nThe `init/1` functions receives an `exec` struct and must return a (modified) `Credo.Execution`.\n\nThis can be used to initialize Execution pipelines for the current Command:\n\n    defmodule FooTask do\n      use Credo.Execution.Task\n\n      def init(exec) do\n        Execution.put_pipeline(exec, __MODULE__,\n          run_my_thing: [\n            {RunMySpecialThing, []}\n          ],\n          filter_results: [\n            {FilterResults, []}\n          ],\n          print_results: [\n            {PrintResultsAndSummary, []}\n          ]\n        )\n      end\n    end","ref":"Credo.CLI.Command.html#c:init/1"},{"type":"callback","title":"Credo.CLI.Command.short_description/0","doc":"Returns a short, one-line description of what the command does","ref":"Credo.CLI.Command.html#c:short_description/0"},{"type":"callback","title":"Credo.CLI.Command.treat_unknown_args_as_files?/0","doc":"","ref":"Credo.CLI.Command.html#c:treat_unknown_args_as_files?/0"},{"type":"module","title":"Credo.CLI.Filename","doc":"This module can be used to handle filenames given at the command line.","ref":"Credo.CLI.Filename.html"},{"type":"function","title":"Credo.CLI.Filename.contains_line_no?/1","doc":"Returns `true` if a given `filename` contains a pos_suffix.\n\n    iex> Credo.CLI.Filename.contains_line_no?(\"lib/credo/sources.ex:39:8\")\n    true\n\n    iex> Credo.CLI.Filename.contains_line_no?(\"lib/credo/sources.ex:39\")\n    true\n\n    iex> Credo.CLI.Filename.contains_line_no?(\"lib/credo/sources.ex\")\n    false","ref":"Credo.CLI.Filename.html#contains_line_no?/1"},{"type":"function","title":"Credo.CLI.Filename.pos_suffix/2","doc":"Returns a suffix for a filename, which contains a line and column value.\n\n    iex> Credo.CLI.Filename.pos_suffix(39, 8)\n    \":39:8\"\n\n    iex> Credo.CLI.Filename.pos_suffix(39, nil)\n    \":39\"\n\nThese are used in this way: lib/credo/sources.ex:39:8","ref":"Credo.CLI.Filename.html#pos_suffix/2"},{"type":"function","title":"Credo.CLI.Filename.remove_line_no_and_column/1","doc":"Removes the pos_suffix for a filename.\n\n    iex> Credo.CLI.Filename.remove_line_no_and_column(\"lib/credo/sources.ex:39:8\")\n    \"lib/credo/sources.ex\"","ref":"Credo.CLI.Filename.html#remove_line_no_and_column/1"},{"type":"function","title":"Credo.CLI.Filename.with/2","doc":"Adds a pos_suffix to a filename.\n\n    iex> Credo.CLI.Filename.with(\"test/file.exs\", %{:line_no => 1, :column => 2})\n    \"test/file.exs:1:2\"","ref":"Credo.CLI.Filename.html#with/2"},{"type":"module","title":"Credo.CLI.Options","doc":"The `Options` struct represents the options given on the command line.\n\nThe `Options` struct is stored as part of the `Execution` struct.","ref":"Credo.CLI.Options.html"},{"type":"function","title":"Credo.CLI.Options.parse/7","doc":"","ref":"Credo.CLI.Options.html#parse/7"},{"type":"function","title":"Credo.CLI.Options.parse/9","doc":"Returns a `Options` struct for the given parameters.","ref":"Credo.CLI.Options.html#parse/9"},{"type":"module","title":"Credo.CLI.Output","doc":"This module provides helper functions regarding command line output.","ref":"Credo.CLI.Output.html"},{"type":"function","title":"Credo.CLI.Output.check_color/1","doc":"","ref":"Credo.CLI.Output.html#check_color/1"},{"type":"function","title":"Credo.CLI.Output.check_tag/2","doc":"","ref":"Credo.CLI.Output.html#check_tag/2"},{"type":"function","title":"Credo.CLI.Output.complain_about_invalid_source_files/1","doc":"","ref":"Credo.CLI.Output.html#complain_about_invalid_source_files/1"},{"type":"function","title":"Credo.CLI.Output.complain_about_timed_out_source_files/1","doc":"","ref":"Credo.CLI.Output.html#complain_about_timed_out_source_files/1"},{"type":"function","title":"Credo.CLI.Output.foreground_color/1","doc":"Returns a suitable foreground color for a given `background_color`.\n\n    iex> Credo.CLI.Output.foreground_color(:yellow)\n    :black\n\n    iex> Credo.CLI.Output.foreground_color(:blue)\n    :white","ref":"Credo.CLI.Output.html#foreground_color/1"},{"type":"function","title":"Credo.CLI.Output.issue_color/1","doc":"Returns a suitable color for a given priority.\n\n    iex> Credo.CLI.Output.issue_color(%Credo.Issue{priority: :higher})\n    :red\n\n    iex> Credo.CLI.Output.issue_color(%Credo.Issue{priority: 20})\n    :red","ref":"Credo.CLI.Output.html#issue_color/1"},{"type":"function","title":"Credo.CLI.Output.print_skipped_checks/1","doc":"","ref":"Credo.CLI.Output.html#print_skipped_checks/1"},{"type":"function","title":"Credo.CLI.Output.priority_arrow/1","doc":"Returns a suitable arrow for a given priority.\n\n    iex> Credo.CLI.Output.priority_arrow(:high)\n    \"↗\"\n\n    iex> Credo.CLI.Output.priority_arrow(10)\n    \"↗\"\n\n    iex> Credo.CLI.Output.priority_arrow(%Credo.Issue{priority: 10})\n    \"↗\"","ref":"Credo.CLI.Output.html#priority_arrow/1"},{"type":"function","title":"Credo.CLI.Output.priority_name/1","doc":"Returns a suitable name for a given priority.\n\n    iex> Credo.CLI.Output.priority_name(:normal)\n    \"normal\"\n\n    iex> Credo.CLI.Output.priority_name(1)\n    \"normal\"\n\n    iex> Credo.CLI.Output.priority_name(%Credo.Issue{priority: 1})\n    \"normal\"","ref":"Credo.CLI.Output.html#priority_name/1"},{"type":"function","title":"Credo.CLI.Output.term_columns/1","doc":"","ref":"Credo.CLI.Output.html#term_columns/1"},{"type":"module","title":"Credo.CLI.Switch","doc":"","ref":"Credo.CLI.Switch.html"},{"type":"function","title":"Credo.CLI.Switch.boolean/2","doc":"","ref":"Credo.CLI.Switch.html#boolean/2"},{"type":"function","title":"Credo.CLI.Switch.ensure/1","doc":"","ref":"Credo.CLI.Switch.html#ensure/1"},{"type":"function","title":"Credo.CLI.Switch.keep/2","doc":"","ref":"Credo.CLI.Switch.html#keep/2"},{"type":"function","title":"Credo.CLI.Switch.string/2","doc":"","ref":"Credo.CLI.Switch.html#string/2"},{"type":"module","title":"Credo.Backports","doc":"","ref":"Credo.Backports.html"},{"type":"module","title":"Credo.Backports.Enum","doc":"","ref":"Credo.Backports.Enum.html"},{"type":"function","title":"Credo.Backports.Enum.slice/2","doc":"","ref":"Credo.Backports.Enum.html#slice/2"},{"type":"module","title":"Credo.Backports.String","doc":"","ref":"Credo.Backports.String.html"},{"type":"function","title":"Credo.Backports.String.slice/2","doc":"","ref":"Credo.Backports.String.html#slice/2"},{"type":"module","title":"Credo.CLI.Command.Diff.Task.FilterIssues","doc":"","ref":"Credo.CLI.Command.Diff.Task.FilterIssues.html"},{"type":"module","title":"Credo.CLI.Command.Diff.Task.GetGitDiff","doc":"","ref":"Credo.CLI.Command.Diff.Task.GetGitDiff.html"},{"type":"function","title":"Credo.CLI.Command.Diff.Task.GetGitDiff.store_resulting_execution/5","doc":"","ref":"Credo.CLI.Command.Diff.Task.GetGitDiff.html#store_resulting_execution/5"},{"type":"module","title":"Credo.CLI.Command.Explain.ExplainCommand.ExplainCheck","doc":"","ref":"Credo.CLI.Command.Explain.ExplainCommand.ExplainCheck.html"},{"type":"module","title":"Credo.CLI.Command.Explain.ExplainCommand.ExplainIssue","doc":"","ref":"Credo.CLI.Command.Explain.ExplainCommand.ExplainIssue.html"},{"type":"function","title":"Credo.CLI.Command.Explain.ExplainCommand.ExplainIssue.print_result/3","doc":"","ref":"Credo.CLI.Command.Explain.ExplainCommand.ExplainIssue.html#print_result/3"},{"type":"module","title":"Credo.CLI.Command.Explain.ExplainCommand.ExplainIssuePreCheck","doc":"","ref":"Credo.CLI.Command.Explain.ExplainCommand.ExplainIssuePreCheck.html"},{"type":"module","title":"Credo.CLI.Command.Info.InfoCommand.PrintInfo","doc":"","ref":"Credo.CLI.Command.Info.InfoCommand.PrintInfo.html"},{"type":"function","title":"Credo.CLI.Command.Info.InfoCommand.PrintInfo.call/2","doc":"","ref":"Credo.CLI.Command.Info.InfoCommand.PrintInfo.html#call/2"},{"type":"module","title":"Credo.CLI.Output.FirstRunHint","doc":"","ref":"Credo.CLI.Output.FirstRunHint.html"},{"type":"function","title":"Credo.CLI.Output.FirstRunHint.call/1","doc":"","ref":"Credo.CLI.Output.FirstRunHint.html#call/1"},{"type":"module","title":"Credo.CLI.Output.FormatDelegator","doc":"This module can be used to easily delegate print-statements for different\nformats to different modules.\n\nExample:\n\n    use Credo.CLI.Output.FormatDelegator,\n      default: Credo.CLI.Command.Suggest.Output.Default,\n      flycheck: Credo.CLI.Command.Suggest.Output.FlyCheck,\n      oneline: Credo.CLI.Command.Suggest.Output.Oneline,\n      json: Credo.CLI.Command.Suggest.Output.Json","ref":"Credo.CLI.Output.FormatDelegator.html"},{"type":"module","title":"Credo.CLI.Output.UI","doc":"This module provides functions used to create the UI.\n\n    >>> alias Credo.CLI.Output.UI\n    >>> UI.puts \"This is a test.\"\n    This is a test.\n    nil\n\n    >>> alias Credo.CLI.Output.UI\n    >>> UI.warn \"This is a warning.\"\n    This is a warning.\n    nil","ref":"Credo.CLI.Output.UI.html"},{"type":"function","title":"Credo.CLI.Output.UI.edge/0","doc":"Returns the edge (`┃`) which is used in much of Credo's output as a binary.","ref":"Credo.CLI.Output.UI.html#edge/0"},{"type":"function","title":"Credo.CLI.Output.UI.edge/2","doc":"","ref":"Credo.CLI.Output.UI.html#edge/2"},{"type":"function","title":"Credo.CLI.Output.UI.puts/0","doc":"","ref":"Credo.CLI.Output.UI.html#puts/0"},{"type":"function","title":"Credo.CLI.Output.UI.puts/1","doc":"","ref":"Credo.CLI.Output.UI.html#puts/1"},{"type":"function","title":"Credo.CLI.Output.UI.puts/2","doc":"","ref":"Credo.CLI.Output.UI.html#puts/2"},{"type":"function","title":"Credo.CLI.Output.UI.puts_edge/2","doc":"","ref":"Credo.CLI.Output.UI.html#puts_edge/2"},{"type":"function","title":"Credo.CLI.Output.UI.truncate/2","doc":"Truncate a line to fit within a specified maximum length.\nTruncation is indicated by a trailing ellipsis (…), and you can override this\nusing an optional third argument.\n\n    iex> Credo.CLI.Output.UI.truncate(nil, 7)\n    \"\"\n    iex> Credo.CLI.Output.UI.truncate(\"  7 chars\\n\", 7)\n    \"  7 ch…\"\n    iex> Credo.CLI.Output.UI.truncate(\"  more than 7\\n\", 7)\n    \"  more…\"\n    iex> Credo.CLI.Output.UI.truncate(\"  more than 7\\n\", 7, \" ...\")\n    \"  m ...\"","ref":"Credo.CLI.Output.UI.html#truncate/2"},{"type":"function","title":"Credo.CLI.Output.UI.truncate/3","doc":"","ref":"Credo.CLI.Output.UI.html#truncate/3"},{"type":"function","title":"Credo.CLI.Output.UI.use_colors/1","doc":"","ref":"Credo.CLI.Output.UI.html#use_colors/1"},{"type":"function","title":"Credo.CLI.Output.UI.warn/1","doc":"","ref":"Credo.CLI.Output.UI.html#warn/1"},{"type":"function","title":"Credo.CLI.Output.UI.wrap_at/2","doc":"","ref":"Credo.CLI.Output.UI.html#wrap_at/2"},{"type":"module","title":"Credo.ConfigBuilder","doc":"","ref":"Credo.ConfigBuilder.html"},{"type":"function","title":"Credo.ConfigBuilder.parse/1","doc":"","ref":"Credo.ConfigBuilder.html#parse/1"},{"type":"module","title":"Credo.ConfigFile","doc":"`ConfigFile` structs represent all loaded and merged config files in a run.","ref":"Credo.ConfigFile.html"},{"type":"function","title":"Credo.ConfigFile.merge/1","doc":"Merges the given structs from left to right, meaning that later entries\noverwrites earlier ones.\n\n    merge(base, other)\n\nAny options in `other` will overwrite those in `base`.\n\nThe `files:` field is merged, meaning that you can define `included` and/or\n`excluded` and only override the given one.\n\nThe `checks:` field is merged.","ref":"Credo.ConfigFile.html#merge/1"},{"type":"function","title":"Credo.ConfigFile.merge/2","doc":"","ref":"Credo.ConfigFile.html#merge/2"},{"type":"function","title":"Credo.ConfigFile.merge_checks/2","doc":"","ref":"Credo.ConfigFile.html#merge_checks/2"},{"type":"function","title":"Credo.ConfigFile.merge_files/2","doc":"","ref":"Credo.ConfigFile.html#merge_files/2"},{"type":"function","title":"Credo.ConfigFile.read_from_file_path/5","doc":"Returns the provided config_file merged into the default configuration.\n\n- `config_file`: full path to the custom configuration file\n- `config_name`: name of the configuration to load\n- `safe`: if +true+, the config files are loaded using static analysis rather\n          than `Code.eval_string/1`","ref":"Credo.ConfigFile.html#read_from_file_path/5"},{"type":"function","title":"Credo.ConfigFile.read_or_default/4","doc":"Returns Execution struct representing a consolidated Execution for all `.credo.exs`\nfiles in `relevant_directories/1` merged into the default configuration.\n\n- `config_name`: name of the configuration to load\n- `safe`: if +true+, the config files are loaded using static analysis rather\n          than `Code.eval_string/1`","ref":"Credo.ConfigFile.html#read_or_default/4"},{"type":"function","title":"Credo.ConfigFile.relevant_directories/1","doc":"Returns all parent directories of the given `dir` as well as each `./config`\nsub-directory.","ref":"Credo.ConfigFile.html#relevant_directories/1"},{"type":"module","title":"Credo.Execution.ExecutionTiming","doc":"The `ExecutionTiming` module can help in timing the execution of code parts and\nstoring those timing inside the `Credo.Execution` struct.","ref":"Credo.Execution.ExecutionTiming.html"},{"type":"function","title":"Credo.Execution.ExecutionTiming.all/1","doc":"Returns all timings for the given `exec`.","ref":"Credo.Execution.ExecutionTiming.html#all/1"},{"type":"function","title":"Credo.Execution.ExecutionTiming.append/3","doc":"Adds a timing piped from `run/2` to the given `exec` (using the given values of `tags`, `started_at` and `duration`).","ref":"Credo.Execution.ExecutionTiming.html#append/3"},{"type":"function","title":"Credo.Execution.ExecutionTiming.append/4","doc":"Adds a timing to the given `exec` using the given values of `tags`, `started_at` and `duration`.","ref":"Credo.Execution.ExecutionTiming.html#append/4"},{"type":"function","title":"Credo.Execution.ExecutionTiming.by_tag/2","doc":"Returns all timings for the given `exec` and `tag_name`.","ref":"Credo.Execution.ExecutionTiming.html#by_tag/2"},{"type":"function","title":"Credo.Execution.ExecutionTiming.by_tag/3","doc":"Returns all timings for the given `exec` and `tag_name` where the tag's value also matches the given `regex`.","ref":"Credo.Execution.ExecutionTiming.html#by_tag/3"},{"type":"function","title":"Credo.Execution.ExecutionTiming.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Credo.Execution.ExecutionTiming.html#child_spec/1"},{"type":"function","title":"Credo.Execution.ExecutionTiming.ended_at/1","doc":"Returns the latest timestamp plus its duration for the given `exec`.","ref":"Credo.Execution.ExecutionTiming.html#ended_at/1"},{"type":"function","title":"Credo.Execution.ExecutionTiming.grouped_by_tag/2","doc":"Groups all timings for the given `exec` and `tag_name`.","ref":"Credo.Execution.ExecutionTiming.html#grouped_by_tag/2"},{"type":"function","title":"Credo.Execution.ExecutionTiming.inspect/2","doc":"Runs the given `fun` and prints the time it took with the given `label`.\n\n    iex> Credo.Execution.ExecutionTiming.inspect(\"foo\", fn -> some_complicated_stuff() end)\n    foo: 51284","ref":"Credo.Execution.ExecutionTiming.html#inspect/2"},{"type":"function","title":"Credo.Execution.ExecutionTiming.now/0","doc":"Returns the current timestamp in the same format (microseconds) as the returned starting times of `run/1`.","ref":"Credo.Execution.ExecutionTiming.html#now/0"},{"type":"function","title":"Credo.Execution.ExecutionTiming.run/1","doc":"Runs the given `fun` and returns a tuple of `{started_at, duration, result}`.\n\n    iex> Credo.Execution.ExecutionTiming.run(fn -> some_complicated_stuff() end)\n    {1540540119448181, 51284, [:whatever, :fun, :returned]}","ref":"Credo.Execution.ExecutionTiming.html#run/1"},{"type":"function","title":"Credo.Execution.ExecutionTiming.run/2","doc":"Same as `run/1` but takes `fun` and `args` separately.","ref":"Credo.Execution.ExecutionTiming.html#run/2"},{"type":"function","title":"Credo.Execution.ExecutionTiming.started_at/1","doc":"Returns the earliest timestamp for the given `exec`.","ref":"Credo.Execution.ExecutionTiming.html#started_at/1"},{"type":"behaviour","title":"Credo.Execution.Task","doc":"A Task is a step in a pipeline, which is given an `Credo.Execution` struct and must return one as well.\n\nTasks in a pipeline are only called if they are not \"halted\" (see `Credo.Execution.halt/2`).\n\nIt implements a `call/1` or `call/2` callback, which is called with the `Credo.Execution` struct\nas first parameter (and the Task's options as the second in case of `call/2`).","ref":"Credo.Execution.Task.html"},{"type":"callback","title":"Credo.Execution.Task.call/1","doc":"Is called by the pipeline and contains the Task's actual code.\n\n    defmodule FooTask do\n      use Credo.Execution.Task\n\n      def call(exec) do\n        IO.inspect(exec)\n      end\n    end\n\nThe `call/1` functions receives an `exec` struct and must return a (modified) `Credo.Execution`.","ref":"Credo.Execution.Task.html#c:call/1"},{"type":"callback","title":"Credo.Execution.Task.call/2","doc":"Works like `call/1`, but receives the options, which are optional when registering the Task, as second argument.\n\n    defmodule FooTask do\n      use Credo.Execution.Task\n\n      def call(exec, opts) do\n        IO.inspect(opts)\n\n        exec\n      end\n    end","ref":"Credo.Execution.Task.html#c:call/2"},{"type":"callback","title":"Credo.Execution.Task.error/1","doc":"Gets called if `call` holds the execution via `Credo.Execution.halt/1` or `Credo.Execution.halt/2`.","ref":"Credo.Execution.Task.html#c:error/1"},{"type":"callback","title":"Credo.Execution.Task.error/2","doc":"Works like `error/1`, but receives the options, which were given during pipeline registration, as second argument.","ref":"Credo.Execution.Task.html#c:error/2"},{"type":"module","title":"Credo.Sources","doc":"This module is used to find and read all source files for analysis.","ref":"Credo.Sources.html"},{"type":"function","title":"Credo.Sources.find/1","doc":"Finds sources for a given `Credo.Execution`.\n\nThrough the `files` key, configs may contain a list of `included` and `excluded`\npatterns. For `included`, patterns can be file paths, directory paths and globs.\nFor `excluded`, patterns can also be specified as regular expressions.\n\n    iex> Sources.find(%Credo.Execution{files: %{excluded: [\"not_me.ex\"], included: [\"*.ex\"]}})\n\n    iex> Sources.find(%Credo.Execution{files: %{excluded: [~r/messy/], included: [\"lib/mix\", \"root.ex\"]}})","ref":"Credo.Sources.html#find/1"},{"type":"function","title":"Credo.Sources.find_in_dir/3","doc":"Finds sources in a given `directory` using a list of `included` and `excluded`\npatterns. For `included`, patterns can be file paths, directory paths and globs.\nFor `excluded`, patterns can also be specified as regular expressions.\n\n    iex> Sources.find_in_dir(\"/home/rrrene/elixir\", [\"*.ex\"], [\"not_me.ex\"])\n\n    iex> Sources.find_in_dir(\"/home/rrrene/elixir\", [\"*.ex\"], [~r/messy/])","ref":"Credo.Sources.html#find_in_dir/3"},{"type":"module","title":"Credo.Watcher","doc":"","ref":"Credo.Watcher.html"},{"type":"function","title":"Credo.Watcher.run/1","doc":"","ref":"Credo.Watcher.html#run/1"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"1.7.2 - Changelog","doc":"- Fix compatibility & compiler warnings with Elixir 1.16\n- Fix a bottleneck in multicore performance\n- Cache scope priorities\n- `CHANGELOG.md` is now included in the package\n- `Credo.Check.Readability.NestedFunctionCalls` no longer warns on function calls in interpolated strings\n- `Credo.Check.Readability.PredicateFunctionNames` fixed false positives\n- `Credo.Check.Readability.RedundantBlankLines` fixed a bug with double quotes multiline strings\n- `Credo.Check.Refactor.ModuleDependencies` now takes modules for `:excluded_namespaces` parameter\n- `Credo.Check.Refactor.NegatedIsNil` no longer leaks memory\n- `Credo.Check.Warning.Dbg` now warns when part of a pipeline and called without parens\n- `Credo.Check.Warning.MissedMetadataKeyInLoggerConfig` received various fixes and improvements","ref":"changelog.html#1-7-2"},{"type":"extras","title":"1.7.1 - Changelog","doc":"- Fix compatibility & compiler warnings with Elixir 1.15\n- Improve docs\n- `Credo.Check.Readability.Specs` works for parensless definitions","ref":"changelog.html#1-7-1"},{"type":"extras","title":"1.7.0 - Changelog","doc":"- `Credo.Check.Readability.ModuleDoc` works for Phoenix 1.7+ views\n- `Credo.Check.Readability.FunctionNames` now ignores custom operators\n- `Credo.Check.Refactor.Apply` now works in pipes\n- `Credo.Check.Consistency.ExceptionNames` does no longer yield an issue if there is only one match\n- `Credo.Check.Readability.ModuleNames` now supports an `:ignore` parameter\n- `Credo.Check.Design.AliasUsage` now supports an `:if_referenced` parameter\n- `Credo.Check.Readability.FunctionNames` now works for acronyms in predicate functions\n- `Credo.Check.Readability.NestedFunctionCalls` now works for calls already in a pipeline and local function calls\n\n#","ref":"changelog.html#1-7-0"},{"type":"extras","title":"Add SARIF support - Changelog","doc":"Credo 1.7 provides a formatter that will output data in [SARIF](http://sarifweb.azurewebsites.net) format, allowing [direct GitHub support](https://help.github.com/en/github/finding-security-vulnerabilities-and-errors-in-your-code/uploading-a-sarif-file-to-github) via the `Security` tab.\n\nYou can now use `mix credo --format=sarif` to output results in SARIF format.\n\n#","ref":"changelog.html#add-sarif-support"},{"type":"extras","title":"Add IDs to checks - Changelog","doc":"This was requested for SARIF support and has been added to provide a unique identifier for checks that is more technical than the check name.\n\nCheck authors can add IDs to their custom checks by using the `:id` option:\n\n    defmodule MyCheck do\n      use Credo.Check,\n        id: \"EX5042\",\n        category: :warning,\n        # ...\n    end\n\nCredo's naming scheme for these IDs is simple:\n\n```\nEX5042\n^^\n```\n\n`EX` stands for Elixir.\n\n```\nEX5042\n  ^\n```\n\nThe first digit represents the category.\n\n```\nEX5042\n   ^\n```\n\nThe second digit is always `0` for Credo's standard checks (see below).\n\n```\nEX5042\n    ^^\n```\n\nThe last two digits are the incremental number of the check.\n\nThis means that you can extend Credo with\n\n* up to 99 categories,\n* up to 999 checks per category or\n* use the second digit for something completely different\n\nall while adhering to Credo's own scheme (and of course, you can simply invent a completely different naming scheme for your checks).\n\n#","ref":"changelog.html#add-ids-to-checks"},{"type":"extras","title":"Allow passing of multiple files to Mix task - Changelog","doc":"It is now possible to pass a list of files to `mix credo`.\n\n```\nmix credo lib/foo.ex lib/foo/bar.ex lib/foo/baz.ex\n```\n\nThis means that you can now use the output of commands to determine which files to analyse, e.g.:\n\n```\nmix credo $(git diff --name-only)\n```\n\n#","ref":"changelog.html#allow-passing-of-multiple-files-to-mix-task"},{"type":"extras","title":"Ensure stable ordering of analysis results - Changelog","doc":"Continuing our quest to remove ambiguity and reduce undocumented behaviour, Credo now orders its results by default.\n\nCredo's results always had one caveat: Their order was determined by the runtime behaviour of the checks and workers running the checks (it was up to the output mechanism to provide its own stable order).\n\nNow, results are always sorted by check ID, filename and line number.\n\n#","ref":"changelog.html#ensure-stable-ordering-of-analysis-results"},{"type":"extras","title":"New checks - Changelog","doc":"- `Credo.Check.Readability.OneArityFunctionInPipe`\n- `Credo.Check.Readability.OnePipePerLine`\n- `Credo.Check.Refactor.FilterCount`\n- `Credo.Check.Refactor.PassAsyncInTestCases`\n- `Credo.Check.Warning.MissedMetadataKeyInLoggerConfig`","ref":"changelog.html#new-checks"},{"type":"extras","title":"1.6.7 - Changelog","doc":"- Improve compatibility with Elixir 1.14 (based on v1.14.0-rc.1)","ref":"changelog.html#1-6-7"},{"type":"extras","title":"1.6.6 - Changelog","doc":"- Fix error when analysing single-line modules\n- Fix false positive for `Credo.Check.Readability.SpaceAroundOperators`\n- Fix false positive for `Credo.Check.Warning.UnusedStringOperation`\n- Fix bug in `Credo.Code.Scope.mod_name/1`","ref":"changelog.html#1-6-6"},{"type":"extras","title":"1.6.5 - Changelog","doc":"- Include the 'only_greater_than' value in the `Credo.Check.Readability.LargeNumbers` message\n- Ignore Phoenix.LiveView modules by default\n- Fix for false positive in `Credo.Check.Refactor.Apply`\n- Fix for false positive in `Credo.Check.Refactor.NegatedIsNil`\n- Fix for false positive in `Credo.Check.Readability.WithSingleClause`","ref":"changelog.html#1-6-5"},{"type":"extras","title":"1.6.4 - Changelog","doc":"- Fix for false positive in `Credo.Check.Readability.MaxLineLength`\n- Fix a bug in `Credo.Check.Refactor.PipeChainStart`\n- Fix error message in `gen.check` command","ref":"changelog.html#1-6-4"},{"type":"extras","title":"1.6.3 - Changelog","doc":"- The HTML report generated by `--debug` now includes slowest files, checks and file/check combinations\n- Fix for false positive in `Credo.Check.Consistency.UnusedVariableNames`\n- Fix for false positive in `Credo.Check.Readability.SpaceAfterCommas`\n- Fix a bug in `Credo.Check.Warning.ForbiddenModule`\n- Fix a bug in `Credo.Check.Warning.MixEnv`\n- `Credo.Check.Readability.LargeNumbers` now supports `:trailing_digits`","ref":"changelog.html#1-6-3"},{"type":"extras","title":"1.6.2 - Changelog","doc":"- Bug fixes\n- Add `-i` as shorthand for `--ignore`","ref":"changelog.html#1-6-2"},{"type":"extras","title":"1.6.1 - Changelog","doc":"- Improve compatibility with Elixir 1.13 (based on v1.13.0-rc.1)","ref":"changelog.html#1-6-1"},{"type":"extras","title":"1.6.0 - Changelog","doc":"- Credo changes from supporting the last 5 minor Elixir versions to the last 6, to be compatible with [Elixir's own support policy regarding bug fixes and security patches](https://hexdocs.pm/elixir/1.12/compatibility-and-deprecations.html)\n- Credo's docs now provide the ability to link specific check params, e.g. [check out :max_length for MaxLineLength](Credo.Check.Readability.MaxLineLength.html#module-max_length)\n- `Credo.Check.Readability.SinglePipe` now supports `:allow_0_arity_functions`\n- `Credo.Check.Design.AliasUsage` now supports `:only`\n- Credo now fails with an error message if a plugin module can not be initialized\n- Prepare compatibility with Elixir 1.13\n\n#","ref":"changelog.html#1-6-0"},{"type":"extras","title":"First Run Mode - Changelog","doc":"Credo 1.6 features a new mode, designed to be run every time you introduce Credo to an existing codebase.\n\n```bash\nmix credo --first-run\n```\n\nThis offers a couple of suggestions on how to introduce Credo to your workflow/CI.\n\nAll of these suggestions are contextualized and project-specific, here's an example when running it on Credo's codebase:\n\n```bash\n-------------------------------------- 8< --------------------------------------\n\n\n# Where to start?\n\nThat's a lot of issues to deal with at once.\n\nWhile not recommended, you could simply start ignoring issues for the time being:\n\n    mix credo --ignore readability      # exclude checks matching a given phrase\n\nYou can use `diff` to only show the issues that were introduced on this branch:\n\n    mix credo diff master","ref":"changelog.html#first-run-mode"},{"type":"extras","title":"Compare to a point in history - Changelog","doc":"Alternatively, you can use `diff` to only show the issues that were introduced after\na certain tag or commit:\n\n    mix credo diff v1.5.6               # use the latest tag\n\n    mix credo diff e0d84ba9             # use the current HEAD of master\n\nLastly, you can compare your working dir against this point in time:\n\n    mix credo diff --since 2021-08-28   # use the current date","ref":"changelog.html#compare-to-a-point-in-history"},{"type":"extras","title":"Every project is different - Changelog","doc":"Introducing code analysis to an existing codebase should not be about following any\n\"best practice\" in particular, it should be about helping you to get to know the ropes\nand make the changes you want.\n\nTry the options outlined above to see which one is working for this project!\n```\n\n#","ref":"changelog.html#every-project-is-different"},{"type":"extras","title":"New Diff Options - Changelog","doc":"`mix credo diff` is often used when developing on a branch and comparing that branch with a base branch.\n\nLet's illustrate this with the following example:\n\n```text\n(feature-a)                        J---K---L\n                                  /         \\\n(master)          base---A---B---C---D---E---F\n                              \\\n(current branch)               X---Y---Z\n```\n\n- `mix credo diff --from-git-ref master` (same as `mix credo diff master`) - this will compare the current branch to the current state of the given ref (commit `F` in the example above)\n- `mix credo diff --from-git-merge-base master` - this will compare the current branch to the point where the current branch was branched off of the given ref (commit `B` in the example above)\n\nThere is also `--from-dir` which you can use to compare the current dir to another dir, thus decoupling the `diff` command from Git, e.g. `mix credo diff --from-dir ../credo_v1_5_6`\n\n#","ref":"changelog.html#new-diff-options"},{"type":"extras","title":"Pinning Checks in a Project's Config - Changelog","doc":"Credo's config always had one caveat: Your configuration settings are merged with the default config, without you having any chance of knowing what the default config is (except by generating a fresh one via `mix credo.gen.config`).\n\n    %{\n      configs: [\n        %{\n          name: \"default\",\n          checks: [\n            # this configures `LargeNumbers` and all default checks are still enabled\n            {Credo.Check.Readability.LargeNumbers, only_greater_than: 99_999}\n          ]\n        }\n      ]\n    }\n\nThis adds an additional problem: When checks are added to the default config, they are also added for you, because there is no way to explicitly say, which checks should run.\n\nCredo 1.6 adds this option to explicitly say which checks are enabled on your project by changing the `:checks` key in the config from a `List` to a `Map` with an `:enabled` key:\n\n    %{\n      configs: [\n        %{\n          name: \"default\",\n          checks: %{\n            enabled: [\n              # this means that only `LargeNumbers` will run for this project\n              {Credo.Check.Readability.LargeNumbers, only_greater_than: 99_999}\n            ]\n          }\n        }\n      ]\n    }\n\nYou can now go the other way as well and disable checks explicitly while keeping their params instead of replacing them with `false`:\n\n    %{\n      configs: [\n        %{\n          name: \"default\",\n          checks: %{\n            disabled: [\n              # this means that `LargeNumbers` is disabled for this project\n              {Credo.Check.Readability.LargeNumbers, only_greater_than: 99_999}\n            ]\n          }\n        }\n      ]\n    }\n\nThis has the added benefit that, when re-enabled via [`--enable-disabled-checks`](suggest_command.html#enable-disabled-checks), the check is enabled with its customized params.\n\n[Credo configs are transitive](config_file.html#transitive-configuration-files) in nature, so what about a situation where you want to pin checks for an umbrella, but overwrite individual checks in a child app? You can use the `:extra` option:\n\n    # my_umbrella/.credo.exs\n    %{\n      configs: [\n        %{\n          name: \"default\",\n          checks: %{\n            enabled: [\n              {Credo.Check.Readability.LargeNumbers, []}\n            ]\n          }\n        }\n      ]\n    }\n\n    # my_umbrella/apps/my_app2/.credo.exs\n    %{\n      configs: [\n        %{\n          name: \"default\",\n          checks: %{\n            extra: [\n              # this means that the checks config from the parent applies,\n              # only `LargeNumbers` being configured differently for this project\n              {Credo.Check.Readability.LargeNumbers, only_greater_than: 99_999}\n            ]\n          }\n        }\n      ]\n    }\n\nOf course, the \"old\" way of specifying a list of checks still works.\n\n#","ref":"changelog.html#pinning-checks-in-a-project-s-config"},{"type":"extras","title":"Exit Status - Changelog","doc":"- Credo succeeds with an exit status of 0 (like any other program).\n- Credo fails with an [exit status between 1 and 127](exit_statuses.html#issue-statuses) if it shows any issues.\n- [Exit statuses above or equal to 128](exit_statuses.html#actual-custom-errors) indicate an Elixir runtime error during analysis itself.\n\nBefore Credo 1.6 it was unclear which exit status \"ranges\" where intended for which kind of error.\nFor example, we are not enforcing the ranges for issue errors or plugin errors, but this change gives an official guideline to these considerations.\n\n#","ref":"changelog.html#exit-status"},{"type":"extras","title":"Working Directory - Changelog","doc":"Up until now, Credo provided the ability to analyse files and directories anywhere on disk by simply typing\n\n```bash\n# BEFORE (now deprecated):\nmix credo ../other-elixir-project/\n```\n\nThis was not really documented and not really supported very well, resulting in most people not really utilizing it and instead running Credo from the root of their project using `mix credo`.\n\nNow there is `--working-dir`, which allows users to define their working directory explicitly:\n\n```bash\nmix credo --working-dir ../other-elixir-project/\n```\n\n#","ref":"changelog.html#working-directory"},{"type":"extras","title":"New checks - Changelog","doc":"- `Credo.Check.Design.SkipTestWithoutComment`\n- `Credo.Check.Readability.PipeIntoAnonymousFunctions`\n- `Credo.Check.Readability.SingleFunctionToBlockPipe`\n- `Credo.Check.Refactor.Apply`\n- `Credo.Check.Refactor.IoPuts`\n- `Credo.Check.Refactor.MapJoin`\n- `Credo.Check.Refactor.RedundantWithClauseResult`","ref":"changelog.html#new-checks"},{"type":"extras","title":"1.5.6 - Changelog","doc":"- Ensure compatibility with Elixir 1.12","ref":"changelog.html#1-5-6"},{"type":"extras","title":"1.5.5 - Changelog","doc":"- Fix bug where compilation warnings are shown if compilation directory is not part of a Git work tree\n- Fix bug in `mix credo diff` where too many issues are reported because","ref":"changelog.html#1-5-5"},{"type":"extras","title":"1.5.4 - Changelog","doc":"- Fix bug where `:exit_status` was not a valid option for `use Credo.Check`\n- Fix bug where `mix credo diff` exited with a non-zero exit status even if no issues were shown\n- `mix credo diff` now fails with an error if the given `ref` was not found","ref":"changelog.html#1-5-4"},{"type":"extras","title":"1.5.3 - Changelog","doc":"- Fix bug in `diff` and `version` command when `git` is not installed","ref":"changelog.html#1-5-3"},{"type":"extras","title":"1.5.2 - Changelog","doc":"- Fix output bug in `diff` command\n- Fix bug in `diff` command when run on a Git ref that does not contain a given `--config-name`\n- Fix bug in all commands when a given `--config-name` is not found\n- Fix false positive in `Credo.Check.Warning.ExpensiveEmptyEnumCheck`\n- Fix false positive in `Credo.Check.Refactor.PipeChainStart` regarding custom operators\n- Fix `--version` to include pre version and build info for unpublished versions (e.g. when the dep is included via `path:` or `github:`)","ref":"changelog.html#1-5-2"},{"type":"extras","title":"1.5.1 - Changelog","doc":"- Fix bug in `diff` command when run on a subdirectory of a Git repository\n- Fix bug in new general param `:files` when excluding files","ref":"changelog.html#1-5-1"},{"type":"extras","title":"1.5.0 - Changelog","doc":"- Credo now requires Elixir 1.7 or newer\n- Refactor check runner (much faster now for common scenarios)\n- Add param `allow_acronyms` to check `Credo.Check.Readability.FunctionNames`\n- Add name of check to message when printing issues with `--verbose`\n- Add support for \"dynamic\" tagging for checks via `.credo.exs`\n\n      # Overwrite all tags for `FooCheck`\n      {FooCheck, [tags: [:my_tag]]}\n\n      # Add tags for `FooCheck`\n      {SomeCredoCheck, [tags: [:__initial__, :my_tag]]}\n\n  Tags can then be used as usual, via the CLI switch `--checks-with[out]-tag`:\n\n      # Only run checks tagged `:my_tag` during analysis\n      $ mix credo --checks-with-tag my_tag\n\n      # Exclude all checks tagged `:my_tag` from analysis\n      $ mix credo --checks-without-tag my_tag\n\n#","ref":"changelog.html#1-5-0"},{"type":"extras","title":"New switch to enable file watcher - Changelog","doc":"You can now ask Credo to re-run on file changes:\n\n      $ mix credo --watch\n\n#","ref":"changelog.html#new-switch-to-enable-file-watcher"},{"type":"extras","title":"New `diff` command - Changelog","doc":"You can now ask Credo to only report changes in files that were changed since a given Git ref:\n\n      $ mix credo diff HEAD^\n      $ mix credo diff master\n\nYou can, of course, combine this with the new `--watch` switch to iteratively fix issues that have come up since the last release:\n\n      $ mix credo diff v1.4.0 --watch\n\n#","ref":"changelog.html#new-diff-command"},{"type":"extras","title":"New general check param `:files` - Changelog","doc":"You can now include/exclude specific files or patterns for specific checks.\n\nThe syntax is the same as for the top-level `:files` key:\n\n```elixir\n# check included for Elixir files in lib/ only\n{Credo.Check.Consistency.ExceptionNames, files: %{included: [\"lib/**/*.ex\"]}},\n\n# check excluded for a specific file\n{Credo.Check.Warning.IExPry, files: %{excluded: [\"lib/debug_server.ex\"]}},\n\n# check included for all Elixir script files, but excluded for test scripts\n{Credo.Check.Warning.IoInspect, files: %{included: [\"**/*.exs\"], excluded: [\"**/*_test.exs\"]}},\n```\n\nThis means that you can now also include/exclude specific files or patterns for your custom checks **by default**.\n\nThe syntax is the same as with all other check params:\n\n```elixir\ndefmodule MyApp.Check.SomethingAboutTests do\n  use Credo.Check,\n    base_priority: :normal,\n    explanations: [\n      check: \"\"\"\n      ...\n      \"\"\"\n    ],\n    param_defaults: [\n      files: %{included: [\"**/*_test.exs\"]}\n    ]\n```\n\nPlease note that these params do not \"override\" the top-level config, but are applied to the result of the top-level config's resolution.\n\n#","ref":"changelog.html#new-general-check-param-files"},{"type":"extras","title":"New checks - Changelog","doc":"These new checks can now be enabled:\n\n- `Credo.Check.Readability.BlockPipe`\n- `Credo.Check.Readability.ImplTrue`\n- `Credo.Check.Readability.SeparateAliasRequire`\n\nAdditionally, `Credo.Check.Warning.ApplicationConfigInModuleAttribute` is a new check which warns about reading environment variables into module attributes at compile-time and is enabled by default.","ref":"changelog.html#new-checks"},{"type":"extras","title":"1.4.1 - Changelog","doc":"- Ensure compatibility with Elixir 1.11.0","ref":"changelog.html#1-4-1"},{"type":"extras","title":"1.4.0 - Changelog","doc":"- Credo's schema for pre-release names changes: There is now a `.` after the `rc` like in many other Elixir projects.\n\n- Add support for explaining checks (in addition to issues), i.e.\n\n      $ mix credo explain Credo.Check.Design.AliasUsage\n\n- Add support for tags on checks\n\n  Checks can now declare tags via the `__using__` macro, i.e.\n\n      defmodule MyCheck do\n        use Credo.Check, tags: [:foo]\n\n        def run(%SourceFile{} = source_file, params) do\n          #\n        end\n      end\n\n  Tags can be used via the CLI switch `--checks-with[out]-tag`:\n\n      # Only run checks tagged `:foo` during analysis\n      $ mix credo --checks-with-tag foo\n\n      # Exclude all checks tagged `:foo` from analysis\n      $ mix credo --checks-without-tag foo\n\n- Add validation of check params in config\n\n  If a param is not found, Credo checks for mispellings and suggests corrections:\n\n      $ mix credo\n      ** (config) Credo.Check.Design.AliasUsage: unknown param `fi_called_more_often_than`. Did you mean `if_called_more_often_than`?\n\n- Add auto-generated check docs\n- Add new documentation on Hex with [extra guides](https://hexdocs.pm/credo/1.4.0/overview.html) and [CHANGELOG](https://hexdocs.pm/credo/1.4.0/changelog.html)","ref":"changelog.html#1-4-0"},{"type":"extras","title":"1.3.2 - Changelog","doc":"- Support non-ascii characters in variable names\n- Fix false positive in `Credo.Check.Readability.ParenthesesOnZeroArityDefs`","ref":"changelog.html#1-3-2"},{"type":"extras","title":"1.3.1 - Changelog","doc":"- Fix new check (`Credo.Check.Readability.StrictModuleLayout`)\n- Ignore module attributes in UnsafeToAtom","ref":"changelog.html#1-3-1"},{"type":"extras","title":"1.3.0 - Changelog","doc":"- Enable `Credo.Check.Readability.UnnecessaryAliasExpansion` check by default\n- Fix bugs when removing heredocs and charlists from sources\n- Fix false positive on TrailingWhiteSpace\n- Add `ignore: [:fun1, :fun2]` param to all `UnusedOperation*` checks; to ignore unused `Enum.reduce/3` operations, use\n\n      {Credo.Check.Warning.UnusedEnumOperation, [ignore: [:reduce]]},\n\n#","ref":"changelog.html#1-3-0"},{"type":"extras","title":"New switch to re-enable disabled checks - Changelog","doc":"Use `--enable-disabled-checks [pattern]` to re-enable checks that were disabled in the config using `{CheckModule, false}`. This comes in handy when using checks on a case-by-case basis\n\nAs with other check-related switches, `pattern` is a comma-delimted list of patterns:\n\n    $ mix credo info --enable-disabled-checks Credo.Check.Readability.Specs,Credo.Check.Refactor.DoubleBooleanNegation\n\nOf course, we can have the same effect by choosing the pattern less explicitly:\n\n    $ mix credo info --enable-disabled-checks specs,double\n\n#","ref":"changelog.html#new-switch-to-re-enable-disabled-checks"},{"type":"extras","title":"New API for custom checks - Changelog","doc":"> This deprecates the mandatory use of `@explanation` and `@default_params` module attributes for checks.\n\nBefore `v1.3` you had to define module attributes named `@explanation` and `@default_params` before calling\n`use Credo.Check`.\n\nNow you can pass `:explanations` (plural) and `:param_defaults` options directly to `use Credo.Check`.\n\n```elixir\ndefmodule MyCheck do\n  use Credo.Check,\n    category: :warning,\n    base_priority: :high,\n    param_defaults: [param1: 42, param2: \"offline\"],\n    explanations: [\n      check: \"...\",\n      params: [\n        param1: \"Your favorite number\",\n        param2: \"Online/Offline mode\"\n      ]\n    ]\n\n  def run(%SourceFile{} = source_file, params) do\n    #\n  end\nend\n```\n\nPlease note that these options\nare also **just a convenience** to implement the functions specified by the  `Credo.Check` behaviour.\nYou can alternatively implement the respective functions yourself:\n\n```elixir\ndefmodule MyCheck do\n  use Credo.Check\n\n  def category, do: :warning\n\n  def base_priority, do: :high\n\n  def explanations do\n    [\n      check: \"...\",\n      params: [\n        param1: \"Your favorite number\",\n        param2: \"Online/Offline mode\"\n      ]\n    ]\n  end\n\n  def param_defaults, do: [param1: 42, param2: \"offline\"]\n\n  def run(%SourceFile{} = source_file, params) do\n    #\n  end\nend\n```\n\n#","ref":"changelog.html#new-api-for-custom-checks"},{"type":"extras","title":"New checks - Changelog","doc":"- `Credo.Check.Readability.StrictModuleLayout`\n- `Credo.Check.Readability.WithCustomTaggedTuple`\n- `Credo.Check.Warning.LeakyEnvironment`\n- `Credo.Check.Warning.UnsafeExec`","ref":"changelog.html#new-checks"},{"type":"extras","title":"1.2.3 - Changelog","doc":"- Fix performance bottleneck in `Credo.Service.ETSTableHelper`","ref":"changelog.html#1-2-3"},{"type":"extras","title":"1.2.2 - Changelog","doc":"- Fix token interpretation of Floats","ref":"changelog.html#1-2-2"},{"type":"extras","title":"1.2.1 - Changelog","doc":"- Actually enable the new check (`Credo.Check.Warning.MixEnv`)","ref":"changelog.html#1-2-1"},{"type":"extras","title":"1.2.0 - Changelog","doc":"- Commands can now have their own pipelines, so that plugins can extend them.\n- Add pipelines to `SuggestCommand` and `ListCommand`\n- `Credo.Plugin.append_task/4` and `Credo.Plugin.prepend_task/4` let you append/prepend tasks to Command's pipelines.\n- Validate options given to `use Credo.Check`\n- Fix `TrailingWhiteSpace` check on Windows\n- Fix `MultiAlias` to work with submodule expansion\n- Fix `UnusedVariableNames` bug\n- Fix `Heredocs.replace_with_spaces/5` bug\n- Fix `InterpolationHelper.replace_interpolations/2` bug\n- Fix crash in `AliasAs` check when __MODULE__ is aliased\n- Fix speed pitfall in `Scope.name/2`\n- New config option: `parse_timeout`\n- Improved default settings for umbrella apps\n\n#","ref":"changelog.html#1-2-0"},{"type":"extras","title":"New checks - Changelog","doc":"- `Credo.Check.Warning.MixEnv`","ref":"changelog.html#new-checks"},{"type":"extras","title":"1.1.5 - Changelog","doc":"- Add JSON output to `categories` and `explain` commands\n- Include number of executed checks in summary\n- Fix wrong trigger in `SinglePipe`","ref":"changelog.html#1-1-5"},{"type":"extras","title":"1.1.4 - Changelog","doc":"- Fix name parsing bug in `UnusedVariableNames`\n- Fix bug when redefining operators in `FunctionNames`\n- Fix false positive in `Specs`","ref":"changelog.html#1-1-4"},{"type":"extras","title":"1.1.3 - Changelog","doc":"- Improve warning message about skipped checks\n- Fix false positive in `FunctionNames`\n- Fix typespec in `IssueMeta`\n- Add `AliasAs` check to list of optional checks","ref":"changelog.html#1-1-3"},{"type":"extras","title":"1.1.2 - Changelog","doc":"- Fix bug in `Heredocs` regarding indentation\n- Fix bug in `FunctionNames` when using `unquote/1` in guards\n- Fix bug in `FunctionNames` when defining `sigil_` functions for uppercase sigils","ref":"changelog.html#1-1-2"},{"type":"extras","title":"1.1.1 - Changelog","doc":"- Fix incompatibilities between Elixir 1.9, `Credo.Code.Token` and  `Credo.Code.InterpolationHelper`\n- Fix error in `Heredocs` with certain UTF-8 chars\n- New param for `ParenthesesOnZeroArityDefs`: use `[parens: true]` to force presence of parentheses on zero arity defs","ref":"changelog.html#1-1-1"},{"type":"extras","title":"1.1.0 - Changelog","doc":"- Credo now requires Elixir 1.5 and Erlang/OTP 19\n- Fix false negative in `DuplicatedCode`\n- `PipeChainStart` has been made opt-in\n\n#","ref":"changelog.html#1-1-0"},{"type":"extras","title":"Plugin Support - Changelog","doc":"Credo now supports plugins that run alongside Credo's own analysis. While Credo provided the ability to write custom checks since `v0.4.0`, users can now access the complete toolset of Credo to create their own commands, require compilation, run external tools and still include the results in Credo's standard report.\n\nPlease refer to Credo's [README](https://github.com/rrrene/credo#plugins) as well as the [Credo Demo Plugin](https://github.com/rrrene/credo_demo_plugin) for further information on how to get started.\n\n#","ref":"changelog.html#plugin-support"},{"type":"extras","title":"New checks - Changelog","doc":"- Credo.Check.Refactor.WithClauses","ref":"changelog.html#new-checks"},{"type":"extras","title":"1.0.5 - Changelog","doc":"- Fix bug due to commented-out heredocs\n- Fix variable explanation for `VariableRebinding` check\n- Add `MultiAlias` check to experimental checks","ref":"changelog.html#1-0-5"},{"type":"extras","title":"1.0.4 - Changelog","doc":"- Ignore heredocs in `RedundantBlankLines`\n- Fix bug in `StringSigils`\n- Minor refactorings","ref":"changelog.html#1-0-4"},{"type":"extras","title":"1.0.3 - Changelog","doc":"- Fix bug in `Name.full/1`\n- Fix bug in `UI.truncate/2`\n- Disable `LazyLogging` for Elixir >= 1.7\n- Add UnnecessaryAliasExpansion check to experimental checks","ref":"changelog.html#1-0-3"},{"type":"extras","title":"1.0.2 - Changelog","doc":"- Fix false positive in `MapInto`\n- Disable `MapInto` for Elixir 1.8 and higher\n- Ensure issues are sorted by filename, line number and column number\n- Warn about ineffective check filter patterns\n- Add `ModuleDependencies` check to experimental checks","ref":"changelog.html#1-0-2"},{"type":"extras","title":"1.0.1 - Changelog","doc":"- Compilation warnings for Elixir 1.8\n- Fix `StringSigils` to not crash with strings containing non-UTF8 characters","ref":"changelog.html#1-0-1"},{"type":"extras","title":"1.0.0 - Changelog","doc":"- Improve documentation\n- Add error handling for malformed config files\n- Write all warnings to `:stderr`\n- Fix false positive for charlists in PipeChainStart\n- Remove deprecated --one-line switch\n- Deactivate checks `DuplicatedCode` and `DoubleBooleanNegation` by default\n\n#","ref":"changelog.html#1-0-0"},{"type":"extras","title":"BREAKING CHANGES - Changelog","doc":"These changes concern people writing their own checks for Credo.\n\n- `Credo.Check.CodeHelper` was removed. Please use the corresponding functions inside the `Credo.Code` namespace.","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"0.10.2 - Changelog","doc":"- Fix bug in AliasOrder","ref":"changelog.html#0-10-2"},{"type":"extras","title":"0.10.1 - Changelog","doc":"- Fixed \"unnecessary atom quotes\" compiler warning during analysis\n- Handle timeouts when reading source files\n- Ignore function calls for OperationOnSameValues\n- Do not treat `|>` as an operator in SpaceAroundOperators\n- Fix AliasOrder bug for multi alias statements\n- Fix multiple false positives for SpaceAroundOperators\n- ... and lots of important little fixes to issue messages, docs and the like!","ref":"changelog.html#0-10-1"},{"type":"extras","title":"0.10.0 - Changelog","doc":"- Switch `poison` for `jason`\n- Add command-line switch to load a custom configuration file (`--config-file`)\n- Add a debug report in HTML format when running Credo using `--debug`\n- Add `node_modules/` to default file excludes\n- Add `:ignore_urls` param for MaxLineLength\n- Report violation for `not` as well as `!` in Refactor.NegatedConditionWithElse\n- Fix false positive on LargeNumbers\n- Fix NegatedConditionWithElse for `not/2` as well\n- Disable PreferUnquotedAtoms for Elixir >= 1.7.0\n\n#","ref":"changelog.html#0-10-0"},{"type":"extras","title":"New checks - Changelog","doc":"- Credo.Check.Refactor.MapInto","ref":"changelog.html#new-checks"},{"type":"extras","title":"0.9.3 - Changelog","doc":"- Fix bug in Scope\n- Fix false positive in MatchInConditionTest\n- Fix false positive in UnusedEnumOperation\n- Fix custom tasks by resolving config before validating it\n- Add text support to `--min-priority` CLI switch (you can now set it to low/normal/high)\n\n#","ref":"changelog.html#0-9-3"},{"type":"extras","title":"New checks - Changelog","doc":"- Credo.Check.Readability.AliasOrder","ref":"changelog.html#new-checks"},{"type":"extras","title":"0.9.2 - Changelog","doc":"- Add `:ignore_comments` param to LongQuoteBlocks\n- Fix false positive in UnusedPathOperation","ref":"changelog.html#0-9-2"},{"type":"extras","title":"0.9.1 - Changelog","doc":"- Fix false positive in SpaceAroundOperators\n- Fix false positive in UnusedEnumOperation","ref":"changelog.html#0-9-1"},{"type":"extras","title":"0.9.0 - Changelog","doc":"- Add JSON support\n- Ensure compatibility with Elixir 1.6\n- Format codebase using Elixir 1.6 Formatter\n- Rework internals in preparation of 1.0 release\n- Credo now requires Elixir 1.4\n- Include `test` directory in default config\n- Add `excluded_argument_types` to PipeChainStart\n- Emit warnings for non-existing checks, which are referenced in config\n- Improve VariableNames\n- ModuleDoc now raises an issue for empty strings in @moduledoc tags\n- Fix bug on ModuleNames\n- Fix false positive in VariableRebinding\n- Fix false positive in SpaceAroundOperators\n- Fix false positive on BoolOperationOnSameValues\n- Fix false positive on SpaceAfterCommas\n- Fix false positive on MaxLineLength\n- Fix false positive in ParenthesesInCondition\n- Remove `NameRedeclarationBy*` checks\n- Remove support for @lint attributes","ref":"changelog.html#0-9-0"},{"type":"extras","title":"0.8.10 - Changelog","doc":"- Maintenance release","ref":"changelog.html#0-8-10"},{"type":"extras","title":"0.8.9 - Changelog","doc":"- Fix false positive in ParenthesesInCondition\n- Fix Code.to_tokens/1 for Elixir 1.6\n- Fix documentation for several checks\n\n#","ref":"changelog.html#0-8-9"},{"type":"extras","title":"New checks - Changelog","doc":"- Credo.Check.Warning.ExpensiveEmptyEnumCheck","ref":"changelog.html#new-checks"},{"type":"extras","title":"0.8.8 - Changelog","doc":"- Fix false positive for `LargeNumbers`\n- Fix `SpaceAroundOperators` for @type module attributes\n- Ignore def arguments and specs for `OperationOnSameValues`\n- Fix crash in `ParenthesesOnZeroArityDefs` for variables named `defp`\n- Fix false positives for `TagHelper`","ref":"changelog.html#0-8-8"},{"type":"extras","title":"0.8.7 - Changelog","doc":"- Fix false positive in `ModuleAttributeNames`\n- Fix false positives in unused return checks\n- Fix underlining in \"list\" action\n- Fix CLI argument parsing for `mix credo.gen.check`\n- Fix loading of custom checks\n- Prevent error when run against empty umbrella projects\n- Prevent output for tests","ref":"changelog.html#0-8-7"},{"type":"extras","title":"0.8.6 - Changelog","doc":"- Fix false positive in SpaceAfterCommas\n- Fix false positive in SpaceAroundOperators\n- Fix bug with extracting explain command args\n- Allow anonymous functions to be piped as raw values","ref":"changelog.html#0-8-6"},{"type":"extras","title":"0.8.5 - Changelog","doc":"- Speed up scope counting in CLI summary","ref":"changelog.html#0-8-5"},{"type":"extras","title":"0.8.4 - Changelog","doc":"- Remove `CheckForUpdates` for good\n- Fix `RaiseInsideRescue` for implicit try","ref":"changelog.html#0-8-4"},{"type":"extras","title":"0.8.3 - Changelog","doc":"- Do not run `CheckForUpdates` on CI systems and in editor integrations","ref":"changelog.html#0-8-3"},{"type":"extras","title":"0.8.2 - Changelog","doc":"- Refactor all consistency checks, providing a nice speed improvement (thx @little-bobby-tables)\n- Improve Elixir 1.5 compatibility","ref":"changelog.html#0-8-2"},{"type":"extras","title":"0.8.1 - Changelog","doc":"- Fix misleading issue message for `LongQuoteBlocks`","ref":"changelog.html#0-8-1"},{"type":"extras","title":"0.8.0 - Changelog","doc":"- Load source files in parallel\n- Improve high memory consumption\n- Fix comment handling of Charlists, Sigils and Strings\n- `LazyLogging` now only checks for `debug` calls by default\n- Add `--mute-exit-status` CLI switch, which mutes Credo's exit status (this will be used for integration tests as it means that any non-zero exit status results from a runtime error of Credo)\n- Add default param values to `mix explain` output\n- `TagTODO` and `TagFIXME` now also report tags from doc-related module attributes (`@doc`, `@moduledoc`, `@shortdoc`)\n- Fix false positives for `TrailingWhiteSpace`\n- Fix compiler warnings for `Sigils`\n\n#","ref":"changelog.html#0-8-0"},{"type":"extras","title":"BREAKING CHANGES - Changelog","doc":"These changes concern people writing their own checks for Credo.\n\n- `Credo.SourceFile` struct was refactored: `source`, `lines` and `ast` are now stored in ETS tables.\n- `Credo.Config` struct was replaced by `Credo.Execution`.\n- `run/3` callbacks for `Credo.Check` are now `run/4` callbacks as they have to receive the execution's `Credo.Execution` struct.\n\n#","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"Config Comments replace `@lint` attributes - Changelog","doc":"`@lint` attributes are deprecated and will be removed in Credo `0.9.0` because\nthey are causing a compiler warning in Elixir `>= 1.4`.\n\nUsers of Credo can now disable individual lines or files for all or just\nspecific checks.\n\nFor now, config comments let you exclude individual files completely\n\n    # credo:disable-for-this-file\n    defmodule SomeApp.ThirdPartyCode do\n    end\n\nor deactivate specific lines:\n\n    def my_fun do\n      # credo:disable-for-next-line\n      IO.inspect :this_is_actually_okay\n    end\n\nor add the check module to exclude just that one check:\n\n    def my_fun do\n      # credo:disable-for-next-line Credo.Check.Warning.IoInspect\n      IO.inspect :this_is_actually_okay\n    end\n\nor use a Regex to be more flexible which checks to exclude:\n\n    def my_fun do\n      # credo:disable-for-next-line /IoInspect/\n      IO.inspect :this_is_actually_okay\n    end\n\nHere's a list with the syntax options:\n\n* `# credo:disable-for-this-file` - to disable for the entire file\n* `# credo:disable-for-next-line` - to disable for the next line\n* `# credo:disable-for-previous-line` - to disable for the previous line\n* `# credo:disable-for-lines: ` - to disable for the given number of lines (negative for previous lines)\n\n#","ref":"changelog.html#config-comments-replace-lint-attributes"},{"type":"extras","title":"New checks - Changelog","doc":"- Credo.Check.Refactor.LongQuoteBlocks","ref":"changelog.html#new-checks"},{"type":"extras","title":"0.7.4 - Changelog","doc":"-\tFix false positives in SpacesAroundOperators\n- Fix `--all` CLI switch\n- Always enforce `strict` mode for ` : `\n- Improve docs on checks\n- Disable `MultiAliasImportRequireUse` by default\n\n#","ref":"changelog.html#0-7-4"},{"type":"extras","title":"Disabled checks - Changelog","doc":"- Credo.Check.Consistency.MultiAliasImportRequireUse","ref":"changelog.html#disabled-checks"},{"type":"extras","title":"0.7.3 - Changelog","doc":"- Fix filename annotation when using `--read-from-stdin`\n- Fix filename handling on Windows\n- Fix consistency checks triggered by contents of sigils\n- Fix consistency checks triggered by contents of charlists\n\n#","ref":"changelog.html#0-7-3"},{"type":"extras","title":"New check - Changelog","doc":"- Credo.Check.Warning.LazyLogging","ref":"changelog.html#new-check"},{"type":"extras","title":"0.7.2 - Changelog","doc":"- Fix `@lint` attribute deprecation hint\n- Fix filename handling bug for Windows\n- Fix flycheck formatting\n- Add param to ignore strings/heredocs in `TrailingWhiteSpace`\n\n#","ref":"changelog.html#0-7-2"},{"type":"extras","title":"New check - Changelog","doc":"- Credo.Check.Readability.SpaceAfterCommas","ref":"changelog.html#new-check"},{"type":"extras","title":"0.7.1 - Changelog","doc":"- Fix `--config_name`CLI switch\n- Fix `UI.wrap_at/2` for Unicode strings\n- Fix false positive for `ModuleNames`","ref":"changelog.html#0-7-1"},{"type":"extras","title":"0.7.0 - Changelog","doc":"- Added deprecation hint about `@lint` attributes\n- Fixed file inclusion/exclusion bug\n- Fixed false positives in `SpaceAroundOperators`\n- Deprecated `NameRedeclarationBy*` checks\n- Fixed false positives in `PipeChainStart`\n- Changed `AppendSingleItem`'s priority and make it opt-in\n- Renamed `NoParenthesesWhenZeroArity` to `ParenthesesOnZeroArityDefs`\n- Fixed a bug in `ParenthesesOnZeroArityDefs`\n\n#","ref":"changelog.html#0-7-0"},{"type":"extras","title":"Added/deprecated checks - Changelog","doc":"- Credo.Check.Warning.MapGetUnsafePass\n- Credo.Check.Refactor.AppendSingleItem\n- Credo.Check.Readability.Semicolons\n\nSwitched some checks to opt-in by default\n\n- Credo.Check.Readability.Specs\n- Credo.Check.Refactor.ABCSize\n- Credo.Check.Refactor.VariableRebinding\n- Credo.Check.Warning.MapGetUnsafePass\n- Credo.Check.Warning.NameRedeclarationByAssignment\n- Credo.Check.Warning.NameRedeclarationByCase\n- Credo.Check.Warning.NameRedeclarationByDef\n- Credo.Check.Warning.NameRedeclarationByFn","ref":"changelog.html#added-deprecated-checks"},{"type":"extras","title":"0.6.1 - Changelog","doc":"- Fixed false positives for `StringSigils` in heredocs\n- Fixed a bug in `SourceFile.column`","ref":"changelog.html#0-6-1"},{"type":"extras","title":"0.6.0 - Changelog","doc":"- Do not warn about ParenthesesInCondition in one-line `if` call\n- Add `--no-strict` CLI switch\n- Fixed exit status for `mix credo list`\n- Fixed exclusion of checks set to `low` priority\n\n#","ref":"changelog.html#0-6-0"},{"type":"extras","title":"New Checks - Changelog","doc":"- consistency/multi_alias_import_require_use\n- readability/no_parentheses_when_zero_arity\n- readability/redundant_blank_lines\n- readability/single_pipe\n- readability/specs\n- readability/string_sigils\n- refactor/double_boolean_negation\n- refactor/variable_rebinding","ref":"changelog.html#new-checks"},{"type":"extras","title":"0.5.3 - Changelog","doc":"- Fix crash in CheckForUpdates","ref":"changelog.html#0-5-3"},{"type":"extras","title":"0.5.2 - Changelog","doc":"- Add ability to specify `strict` in `.credo.exs` config file\n- Add `no_case?` to variable name checks\n- Add `Module.name` utility method\n- Fix bug in NameRedeclarationByDef\n- Fix bug in LargeNumbers","ref":"changelog.html#0-5-2"},{"type":"extras","title":"0.5.1 - Changelog","doc":"- Use Hex.pm API to retrieve update information for Credo","ref":"changelog.html#0-5-1"},{"type":"extras","title":"0.5.0 - Changelog","doc":"- See below","ref":"changelog.html#0-5-0"},{"type":"extras","title":"0.4.14 - Changelog","doc":"- Fix compiler error due to usage of undocumented Hex APIs (sorry)","ref":"changelog.html#0-4-14"},{"type":"extras","title":"0.4.13 - Changelog","doc":"- Prevent crashing when parsing non-UTF-8 source files\n- Fix yet another issue with finding sources\n- Fix false positives for PipeChainStart\n- Fix false positives for AbcSize\n- Remove dead code\n- Update deps","ref":"changelog.html#0-4-13"},{"type":"extras","title":"0.4.12 - Changelog","doc":"- Prevent ModuleDoc from checking nested modules for ignored modules\n- Fix issue with ParenthesesInCondition\n- Fix issue with PipChainStart\n- Fix issue with reading from stdin\n- Log errors to stderr","ref":"changelog.html#0-4-12"},{"type":"extras","title":"0.4.11 - Changelog","doc":"- Fix compatibility issues with Elixir < 1.2\n- Fix yet another issue with finding sources","ref":"changelog.html#0-4-11"},{"type":"extras","title":"0.4.10 - Changelog","doc":"- Fix another issue with finding sources","ref":"changelog.html#0-4-10"},{"type":"extras","title":"0.4.9 - Changelog","doc":"- Fix issues with finding sources\n- Don't enforce @moduledoc requirement for Mixfile or Test modules","ref":"changelog.html#0-4-9"},{"type":"extras","title":"0.4.8 - Changelog","doc":"- Add `exclude_functions` option to `PipeChainStart`\n- No longer report issues in case of ambiguous aliases in `AliasUsage`\n- Fix false positives for `LargeNumbers` in Elixir `>= 1.3.2` (again)\n- Lots of refactorings","ref":"changelog.html#0-4-8"},{"type":"extras","title":"0.4.7 - Changelog","doc":"- Ignore module attributes like `@spec` in `AliasUsage`\n- Improve source file loading","ref":"changelog.html#0-4-7"},{"type":"extras","title":"0.4.6 - Changelog","doc":"- Add `ignore_specs` option to `MaxLineLength`\n- Fix false positives for `LargeNumbers` in Elixir `>= 1.3.2`\n- Fix compiler warnings in preparation for Elixir v1.4","ref":"changelog.html#0-4-6"},{"type":"extras","title":"0.4.5 - Changelog","doc":"- Warnings about redeclaring names of Kernel or local functions now only consider function of arity zero.\n- Warnings for operations with constant result now ignore division by 1\n- Better explanation how to configure checks in `explain` output","ref":"changelog.html#0-4-5"},{"type":"extras","title":"0.4.4 - Changelog","doc":"- New check: readability/parentheses_in_condition","ref":"changelog.html#0-4-4"},{"type":"extras","title":"0.4.3 - Changelog","doc":"- Fix compatibility issues in `Credo.CLI.Command.GenCheck` for Elixir < 1.2","ref":"changelog.html#0-4-3"},{"type":"extras","title":"0.4.2 - Changelog","doc":"- Fix outdated comments regarding .credo.exs in README\n- Fix name generator including \"Lib.\" prefix for custom checks","ref":"changelog.html#0-4-2"},{"type":"extras","title":"0.4.1 - Changelog","doc":"- Maintenance release because I apparently don't understand how merging works :sweat:","ref":"changelog.html#0-4-1"},{"type":"extras","title":"0.4.0 - Changelog","doc":"#","ref":"changelog.html#0-4-0"},{"type":"extras","title":"Custom check support - Changelog","doc":"- Adds support for custom checks in your projects.\n\n  Using two new mix commands `mix credo.gen.config` and `mix credo.gen.check`\n  you can generate the boilerplate to include custom checks in your projects.\n\n#","ref":"changelog.html#custom-check-support"},{"type":"extras","title":"BREAKING CHANGE: Checks listed in `.credo.exs` - Changelog","doc":"- Prior to `v0.4.0`, `.credo.exs` contained the full list of checks specific to your project\n- Starting with `v0.4.0` the check list in `credo.exs` will be merged with the standard check list, with your definitions overwriting the defaults\n- PRO: you can customize individual tasks to your liking and still benefit from additional standard checks with each new release\n- CON: this means checks have to be disabled explicitly in `.credo.exs`\n\n#","ref":"changelog.html#breaking-change-checks-listed-in-credo-exs"},{"type":"extras","title":"New Checks - Changelog","doc":"- readability/large_numbers\n- warning/bool_operation_on_same_values\n- warning/unused_file_operation\n- warning/unused_path_operation\n- warning/unused_regex_operation\n\n#","ref":"changelog.html#new-checks"},{"type":"extras","title":"Minor Improvements - Changelog","doc":"- Ready for Elixir 1.3\n- Checks for new Credo versions automatically, like Hex does (can be disabled)\n- Umbrella apps work out of the box now\n- DuplicatedCode can now ignore macro calls\n- ModuleDoc now ignores modules declaring exceptions\n- ModuleDoc now allows modules to be ignored based on their name\n- MatchInCondition now allows \"simple\" wildcard assignments in conditionals\n- Checks analysing all files in the codebase sequentially (consistency checks)\n  are now run in parallel\n- If `--only` is given, all issues are shown (`mix credo --only MaxLineLength`\n  previously yielded no results, since all issues needed `--strict` to actually\n  be displayed)","ref":"changelog.html#minor-improvements"},{"type":"extras","title":"0.3.13 - Changelog","doc":"- Fix false positives for `NameRedeclarationByDef`.\n- Fix false positives for `UnusedEnumOperation`.","ref":"changelog.html#0-3-13"},{"type":"extras","title":"0.3.12 - Changelog","doc":"- Fix false positives for `SpaceInParentheses`.\n- Fix false positive for `SpaceAroundOperators`.","ref":"changelog.html#0-3-12"},{"type":"extras","title":"0.3.11 - Changelog","doc":"- Fix a bug with checks on function names when declaring a variable with the name `def`, `defp` or `defmacro`.","ref":"changelog.html#0-3-11"},{"type":"extras","title":"0.3.10 - Changelog","doc":"- Fix a bug resulting in `UnicodeConversionError` for code containing UTF-8 comments.","ref":"changelog.html#0-3-10"},{"type":"extras","title":"0.3.9 - Changelog","doc":"- Fix a bug in `AliasUsage`.","ref":"changelog.html#0-3-9"},{"type":"extras","title":"0.3.8 - Changelog","doc":"- Fix false positives for `AliasUsage`.","ref":"changelog.html#0-3-8"},{"type":"extras","title":"0.3.7 - Changelog","doc":"- Fix false positive for `SpaceAroundOperators`.","ref":"changelog.html#0-3-7"},{"type":"extras","title":"0.3.6 - Changelog","doc":"- Fix false positives for `SpaceAroundOperators` and `PipeChainStart`.\n- Add option to read from STDIN for better editor integration","ref":"changelog.html#0-3-6"},{"type":"extras","title":"0.3.5 - Changelog","doc":"- Remove superfluous call to `IO.inspect`.\n- Update deps requirements to make HexFaktor happy.","ref":"changelog.html#0-3-5"},{"type":"extras","title":"0.3.4 - Changelog","doc":"- Fix false positives for `SpaceAroundOperators` in binary pattern matches.\n- Fix a bug when supplying a single file via the CLI.","ref":"changelog.html#0-3-4"},{"type":"extras","title":"0.3.3 - Changelog","doc":"- Fix false positives for `SpaceAroundOperators` and `SpaceInParentheses`.","ref":"changelog.html#0-3-3"},{"type":"extras","title":"0.3.2 - Changelog","doc":"- `mix do credo,  ` was broken and never ran ` `, even if `credo` succeeded (exited with exit status 0). Now it runs ` ` as long as `credo` succeeds.","ref":"changelog.html#0-3-2"},{"type":"extras","title":"0.3.1 - Changelog","doc":"- Fix compiler warnings\n- Improve copywriting for consistency checks (thx @vdaniuk)","ref":"changelog.html#0-3-1"},{"type":"extras","title":"0.3.0 - Changelog","doc":"#","ref":"changelog.html#0-3-0"},{"type":"extras","title":"Per-function lint support - Changelog","doc":"- Adds support for `@lint` attributes used to configure linting for specific\n  functions.\n\n  For now, this lets you exclude functions completely\n\n      @lint false\n      def my_fun do\n      end\n\n  or deactivate specific checks *with the same syntax used in the config file*:\n\n      @lint {Credo.Check.Design.TagTODO, false}\n      def my_fun do\n      end\n\n  or use a Regex instead of the check module to exclude multiple checks at once:\n\n      @lint {~r/Refactor/, false}\n      def my_fun do\n      end\n\n  Finally, you can supply multiple tuples as a list and combine the above:\n\n      @lint [{Credo.Check.Design.TagTODO, false}, {~r/Refactor/, false}]\n      def my_fun do\n      end\n\n#","ref":"changelog.html#per-function-lint-support"},{"type":"extras","title":"New Checks - Changelog","doc":"- consistency/space_around_operators\n- consistency/space_in_parentheses\n\n#","ref":"changelog.html#new-checks"},{"type":"extras","title":"Minor Improvements - Changelog","doc":"- Add `--format` CLI switch\n- Include experimental Flycheck support via `--format=flycheck`\n- **Deprecate** `--one-line` CLI switch, use `--format=oneline` instead\n- Add convenience alias `--ignore` for `--ignore-checks`\n- Fix colors for terminals with light backgrounds (thx @lucasmazza)","ref":"changelog.html#minor-improvements"},{"type":"extras","title":"0.2.6 - Changelog","doc":"- Fix false positives for UnusedEnumOperation checks (thx @kbaird)","ref":"changelog.html#0-2-6"},{"type":"extras","title":"0.2.5 - Changelog","doc":"- Fix error occurring when a project has exactly one `defexception` module\n- Change the tag for Refactoring Opportunities from \"[R]\" to \"[F]\" (thx @rranelli)","ref":"changelog.html#0-2-5"},{"type":"extras","title":"0.2.4 - Changelog","doc":"- Remove unused alias to avoid warning during compilation","ref":"changelog.html#0-2-4"},{"type":"extras","title":"0.2.3 - Changelog","doc":"- Improves docs and UI wording (thx @crack and @jessejanderson)","ref":"changelog.html#0-2-3"},{"type":"extras","title":"0.2.2 - Changelog","doc":"- Adds a missing word to the output of the `categories` command (thx @bakkdoor)","ref":"changelog.html#0-2-2"},{"type":"extras","title":"0.2.1 - Changelog","doc":"- Fixes a problem with CaseTrivialMatches crashing","ref":"changelog.html#0-2-1"},{"type":"extras","title":"0.2.0 - Changelog","doc":"#","ref":"changelog.html#0-2-0"},{"type":"extras","title":"Error Status - Changelog","doc":"Credo now fails with an exit status != 0 if it shows any issues. This will enable usage of Credo inside CI systems/build chains.\n\nThe exit status of each check is customizable and exit statuses of all\nencountered checks are collected, uniqued and summed:\n\n    issues\n    |> Enum.map(&(&1.exit_status))\n    |> Enum.uniq\n    |> Enum.reduce(0, &(&1+&2))\n\nThis way you can reason\nabout the encountered issues right from the exit status.\n\nDefault values for the checks are based on their category:\n\n    consistency:  1\n    design:       2\n    readability:  4\n    refactor:     8\n    warning:     16\n\nSo an exit status of 12 tells you that you have only Readability Issues and Refactoring Opportunities, but e.g. no Warnings.\n\n#","ref":"changelog.html#error-status"},{"type":"extras","title":"New Checks - Changelog","doc":"- readability/module_doc\n- refactor/case_trivial_matches\n- refactor/cond_statements\n- refactor/function_arity\n- refactor/match_in_condition\n- refactor/pipe_chain_start\n- warning/operation_with_constant_result\n- warning/unused_enum_operation\n- warning/unused_keyword_operation\n- warning/unused_list_operation\n- warning/unused_tuple_operation\n\n#","ref":"changelog.html#new-checks"},{"type":"extras","title":"Minor Improvements - Changelog","doc":"- There are two new aliases for command line switches:\n  - you can use `--only` as alias for `--checks`\n  - you can use `--strict` as alias for `--all-priorities`\n- `mix credo --only  ` will always display a full list of results\n  (you no longer need to specify `--all` separately)\n- `mix credo explain  ` now also explains the available configuration parameters for the issue/check\n- The ExceptionNames check no longer fails if only a single exception module is found (#22).","ref":"changelog.html#minor-improvements"},{"type":"extras","title":"0.1.10 - Changelog","doc":"- Apply many fixes in anticipation of Elixir v1.2 (thx @c-rack)\n- Improve docs\n- Wrap long issue descriptions in `suggest` command","ref":"changelog.html#0-1-10"},{"type":"extras","title":"0.1.9 - Changelog","doc":"- Add missing `-A` alias for `--all-priorities`\n- Improve wording in the CLI a bit","ref":"changelog.html#0-1-9"},{"type":"extras","title":"0.1.8 - Changelog","doc":"- Add `apps/` to default directories","ref":"changelog.html#0-1-8"},{"type":"extras","title":"0.1.7 - Changelog","doc":"- Bugfix to `NameRedeclarationBy\\*` checks\n- `Sources.exclude` had a bug when excluding directories","ref":"changelog.html#0-1-7"},{"type":"extras","title":"0.1.6 - Changelog","doc":"- Rename CLI switch `--pedantic` to `--all-priorities` (alias is `-A`)\n- Fix a bug in SourceFile.column (#7)\n- Improve README section about basic usage, commands and issues","ref":"changelog.html#0-1-6"},{"type":"extras","title":"0.1.1 - 0.1.5 - Changelog","doc":"Multiple Hex releases due to the fact that I apparently don't understand how deps compilation works :sweat:","ref":"changelog.html#0-1-1-0-1-5"},{"type":"extras","title":"0.1.0 - Changelog","doc":"Initial release","ref":"changelog.html#0-1-0"},{"type":"extras","title":"Overview","doc":"# Overview\n\nCredo is a static code analysis tool for the Elixir language with a focus on teaching and code consistency.\n\n![Credo](https://raw.github.com/rrrene/credo/master/assets/screenshot.png)\n\n`credo` can show you refactoring opportunities in your code, complex code fragments, warn you about common mistakes, show inconsistencies in your naming scheme and - if needed - help you enforce a desired coding style.\n\nThe basic functionality will be familiar to you if you used a linter like JavaScript's [ESLint](https://eslint.org/), Ruby's [RuboCop](https://github.com/rubocop-hq/rubocop) or C#'s [Stylecop](https://github.com/StyleCop/StyleCop) before.\n\nContrary to the aforementioned tools, Credo puts a strong emphasis on teaching and code consistency.\n\nTo get started, you might want to check out these guides:\n\n* [Installation](../introduction/installation.md)\n* [Basic usage](../introduction/basic_usage.md)\n* [Configuration via `.credo.exs`](../configuration/config_file.md)\n\nOnce you are familiar with the basics, look into these topics:\n\n* [Adding custom checks](../custom_checks/adding_checks.md)\n* [Testing custom checks](../custom_checks/testing_checks.md)\n* [Plugins](../plugins/creating_plugins.md)\n\nAny incorrect or unclear information in these docs should be considered a bug.\nIn case you find something, you can help maintaining these docs by [filing an issue on GitHub](https://github.com/rrrene/credo/issues) and/or opening a pull request.","ref":"overview.html"},{"type":"extras","title":"Installation","doc":"# Installation\n\nThe easiest way to add Credo to your project is by [using Mix](http://elixir-lang.org/getting-started/mix-otp/introduction-to-mix.html).\n\nAdd `:credo` as a dependency to your project's `mix.exs`:\n\n```elixir\ndefp deps do\n  [\n    {:credo, \"~> 1.6\", only: [:dev, :test], runtime: false}\n  ]\nend\n```\n\nAnd run:\n\n```bash\n$ mix deps.get\n```","ref":"installation.html"},{"type":"extras","title":"Basic Usage","doc":"# Basic Usage\n\nSince Credo is all about teaching people, you can find out more about that particular entry.","ref":"basic_usage.html"},{"type":"extras","title":"Run Analysis - Basic Usage","doc":"To run credo in the current project, just type:\n\n```bash\n$ mix credo\n```\n\nThis will run the code analysis and suggest places to edit your code.","ref":"basic_usage.html#run-analysis"},{"type":"extras","title":"Explain Issues - Basic Usage","doc":"Running Credo will yield issues like this:\n\n```bash\n┃  Refactoring opportunities\n┃\n┃ [F] ↗ Avoid negated conditions in if-else blocks.\n┃       lib/foo/bar.ex:306 #(Foo.Bar.deprecated_def_explanations)\n```\n\nTo find out more about the issue, just append its location to the `credo` command:\n\n```bash\nmix credo lib/foo/bar.ex:306\n```\n\nThe result is an explanation of the issue, including the check that raised the issue, its configuration options and how to disable it.","ref":"basic_usage.html#explain-issues"},{"type":"extras","title":"Advanced Usage - Basic Usage","doc":"#","ref":"basic_usage.html#advanced-usage"},{"type":"extras","title":"Strict Analysis - Basic Usage","doc":"Like any code linter, Credo reports issues. Contrary to many other linters these issues are not created equal. Each issue is assigned a priority, based on a base priority set by the config and a dynamic component based on violation severity and location in the source code.\n\nThese priorities hint at the importance of each issue and are displayed in the command-line interface using arrows: ↑ ↗ → ↘ ↓\n\nBy default, only issues with a positive priority are part of the report (↑ ↗ →).\n\nTo include all issues, just type:\n\n```bash\nmix credo --strict\n```\n\n#","ref":"basic_usage.html#strict-analysis"},{"type":"extras","title":"Format output as JSON - Basic Usage","doc":"Credo can provide the output of every command as JSON:\n\n```bash\n$ mix credo lib/foo/bar.ex:306 --format json\n```\n\n```json\n{\n  \"explanations\": [\n    {\n      \"category\": \"refactor\",\n      \"check\": \"Elixir.Foo.Bar.Refactor.NegatedConditionsWithElse\",\n      \"column\": null,\n      \"explanation_for_issue\": \"An `if` block with a negated condition should not contain an else block.\\n\\nSo while this is fine:\\n\\n    if not allowed? do\\n      raise \\\"Not allowed!\\\"\\n    end\\n\\nThe code in this example ...\\n\\n    if not allowed? do\\n      raise \\\"Not allowed!\\\"\\n    else\\n      proceed_as_planned()\\n    end\\n\\n... should be refactored to look like this:\\n\\n    if allowed? do\\n      proceed_as_planned()\\n    else\\n      raise \\\"Not allowed!\\\"\\n    end\\n\\nThe same goes for negation through `!` instead of `not`.\\n\\nThe reason for this is not a technical but a human one. It is easier to wrap\\nyour head around a positive condition and then thinking \\\"and else we do ...\\\".\\n\\nIn the above example raising the error in case something is not allowed\\nmight seem so important to put it first. But when you revisit this code a\\nwhile later or have to introduce a colleague to it, you might be surprised\\nhow much clearer things get when the \\\"happy path\\\" comes first.\\n\",\n      \"filename\": \"lib/foo/bar.ex\",\n      \"line_no\": 306,\n      \"message\": \"Avoid negated conditions in if-else blocks.\",\n      \"priority\": 12,\n      \"related_code\": [\n        [304, \"    explanation = Module.get_attribute(env.module, explanation)\"],\n        [305, \"\"],\n        [306, \"    if not is_nil(explanation) do\"],\n        [307, \"      # deprecated - remove once we ditch @explanation\"],\n        [308, \"      quote do\"]\n      ],\n      \"scope\": \"Foo.Bar.deprecated_def_explanations\",\n      \"trigger\": \"!\"\n    }\n  ]\n}\n```","ref":"basic_usage.html#format-output-as-json"},{"type":"extras","title":"Further reading - Basic Usage","doc":"If you are interested in more ways to configure Credo, continue reading in the following sections:\n\n* [CLI switches](../configuration/cli_switches.md)\n* [Configuration file](../configuration/config_file.md)","ref":"basic_usage.html#further-reading"},{"type":"extras","title":"Exit Status","doc":"# Exit Status\n\nCredo succeeds with an exit status of 0 (like any other program).\n\nCredo fails with an exit status between 1 and 127 if it shows any issues.\n\nExit statuses above or equal to 128 indicate an actual runtime error during analysis itself.\n\nThis enables shell based pipeline workflows (e.g. on CI systems) which test Credo compliance.","ref":"exit_statuses.html"},{"type":"extras","title":"Issue Statuses - Exit Status","doc":"The exit status of each check is used to construct a bit map of the types of issues which were encountered by or-ing them together to produce the final result:\n\n```elixir\nimport Bitwise\n\nissues\n|> Enum.map(&(&1.exit_status))\n|> Enum.reduce(0, &(&1 ||| &2))\n```\n\nThis way you can reason about the encountered issues right from the exit status.\n\nDefault values for the checks are based on their category:\n\n    consistency:          1\n    design:               2\n    readability:          4\n    refactor:             8\n    warning:              16\n\nLet's see what this means using an example:\n\n```bash\n$ mix credo\n\n[...snip...]\n\n$ echo $?\n12\n```\n\nSo an exit status of `12` tells you that you have only Readability Issues (`4`) and Refactoring Opportunities (`8`), but e.g. no Warnings.\n\nNaturally, custom checks and plugins can provide their own exit statuses.\n\n```bash\n :    32\n :    64\n```","ref":"exit_statuses.html#issue-statuses"},{"type":"extras","title":"Actual & Custom Errors - Exit Status","doc":"To also allow for actual errors, an exit status of `>= 128` signals something went wrong during analysis itself.\n\nSince one cannot combine these, they do not follow the bitwise notation described above:\n\n```bash\nGeneric Credo error:  128\nCredo Config errors:  129-131\nReserved errors:      132-191\n```\n\nNaturally, plugins can provide their own exit statuses.\n\n```bash\n :      192-255\n```","ref":"exit_statuses.html#actual-custom-errors"},{"type":"extras","title":"Mix Tasks","doc":"# Mix Tasks\n\nAfter including Credo in a project's dependencies (see [Installation](../introduction/installation.md)), there are a number of built-in mix tasks available:\n\n```bash\n$ mix help | grep -i credo\nmix credo                 # Run code analysis (use `--help` for options)\nmix credo.gen.check       # Generate a new custom check for Credo\nmix credo.gen.config      # Generate a new config for Credo\n```\n\nIf you want to know more about `mix`, check out [Introduction to Mix](https://elixir-lang.org/getting-started/mix-otp/introduction-to-mix.html).\n\n`mix credo`\n\nRuns Credo's analysis.\n\nCheck out [Configuration](../configuration/cli_switches.md) on how to customize inputs and outputs.\n\n`mix credo.gen.check`\n\nGenerates a custom Credo check.\n\n`mix credo.gen.config`\n\nGenerates a Credo config file.\n\nCheck out [Configuration](../configuration/config_file.md) on how to customize it.","ref":"mix_tasks.html"},{"type":"extras","title":".credo.exs","doc":"# .credo.exs\n\nCredo is configured via a file called `.credo.exs`.\n\nYou can use `mix credo gen.config` to generate a complete example configuration.\n\n```bash\n$ mix credo gen.config\n* creating .credo.exs\n```\n\nThis file can live in your project's `config/` or root folder, both is fine.","ref":"config_file.html"},{"type":"extras","title":"Config Keys - .credo.exs","doc":"Credo's config is a plain `.exs` file, no magic here. It contains a map with a single key (`:configs`), which contains a list of maps that represent the individual configs (most of the time, it's just one, named \"default\").\n\n```elixir\n# .credo.exs or config/.credo.exs\n%{\n  configs: [\n    %{\n      name: \"default\",\n      files: %{\n        included: [\"lib/\", \"src/\", \"web/\", \"apps/\"],\n        excluded: []\n      },\n      plugins: [],\n      requires: [],\n      strict: false,\n      parse_timeout: 5000,\n      color: true,\n      checks: %{\n        enabled: [\n          {Credo.Check.Design.AliasUsage, priority: :low},\n          # ... other checks omitted for readability ...\n        ]\n      }\n    }\n  ]\n}\n```\n\nThe config keys available are:\n\n- [`:name`](#name)\n- [`:checks`](#checks)\n- [`:color`](#color)\n- [`:files`](#files)\n- [`:parse_timeout`](#parse_timeout)\n- [`:plugins`](#plugins)\n- [`:requires`](#requires)\n- [`:strict`](#strict)\n\n### `:name`\n\nCredo configs are given names. The default configuration is named `default`.\n\nYou can specify which config to use on the command line (again, `default` is run by ... default):\n\n```bash\nmix credo --config-name  \n```\n\nFor example, say we have a directory `lib/that_big_namespace` with tons of issues and we do want to split our regular linting and the necessary clean up in that directory.\n\nWe can exclude the directory from our `default` config and add another config for just that directory.\n\n```elixir\n# .credo.exs\n%{\n  configs: [\n    %{\n      name: \"default\",\n      files: %{\n        included: [\"lib/\", \"src/\", \"web/\", \"apps/\"],\n        excluded: [\"lib/that_big_namespace\"]\n      }\n    },\n    %{\n      name: \"spring-cleaning\",\n      files: %{\n        included: [\"lib/that_big_namespace\"],\n        excluded: []\n      }\n    }\n  ]\n}\n```\n\nNow you can use\n\n```bash\nmix credo --config-name spring-cleaning\n```\n\nto run the custom configuration we added.\n\n\n### `:checks`\n\nConfigures check modules that Credo should load at start up.\n\nRead more about [check configuration](./check_params.md) and [adding custom checks](../custom_checks/adding_checks.md).\n\n#### `:enabled`\n\nEnables *only* the given checks. This is an easy way to pin a project's checks.\n\n```elixir\n# .credo.exs\n%{\n  configs: [\n    %{\n      name: \"default\",\n      checks: %{\n        enabled: [\n          # this means that only `TabsOrSpaces` will run\n          {Credo.Check.Consistency.TabsOrSpaces, []},\n        ]\n      }\n      # files etc.\n    }\n  ]\n}\n```\n\n#### `:disabled`\n\nDisables the given checks. This is an easy way to use the default checks, but explicitly disable some you don't need or want.\n\n```elixir\n# .credo.exs\n%{\n  configs: [\n    %{\n      name: \"default\",\n      checks: %{\n        disabled: [\n          # this means that `TabsOrSpaces` will not run\n          {Credo.Check.Consistency.TabsOrSpaces, []},\n        ]\n      }\n      # files etc.\n    }\n  ]\n}\n```\n\nThis has the added benefit that, when re-enabled via [`--enable-disabled-checks`](suggest_command.html#enable-disabled-checks), check are enabled with their [customized params](./check_params.md).\n\n\n#### `:extra`\n\nEnables and configures the given checks with the given parameters.\n\nThis can be used in a [Credo Plugin](../plugins/creating_plugins.md) to [add a check to the current Credo config](Credo.Plugin.html#register_default_config/2) ([configs are transitive](config_file.html#transitive-configuration-files)) without interfering with what other configs are doing.\n\n```elixir\n# a manually registered Config in a Credo Plugin (see links above)\n%{\n  configs: [\n    %{\n      name: \"default\",\n      checks: %{\n        extra: [\n          {MyCredoPlugin.Check.BetterTabsOrSpaces, []},\n        ]\n      }\n      # files etc.\n    }\n  ]\n}\n```\n\nThis can also be useful in a situation where you want to enable checks for an umbrella, but want to overwrite individual checks in a child app.\n\n\n```elixir\n# my_umbrella/.credo.exs\n%{\n  configs: [\n    %{\n      name: \"default\",\n      checks: %{\n        enabled: [\n          {Credo.Check.Readability.LargeNumbers, []}\n        ]\n      }\n    }\n  ]\n}\n\n# my_umbrella/apps/my_app2/.credo.exs\n%{\n  configs: [\n    %{\n      name: \"default\",\n      checks: %{\n        extra: [\n          # this means that the checks config from the parent applies,\n          # only `LargeNumbers` being configured differently for this project\n          {Credo.Check.Readability.LargeNumbers, only_greater_than: 99_999}\n        ]\n      }\n    }\n  ]\n}\n```\n\n\n### `:color`\n\nSet to `false` to disable colored output (*defaults to `true`*).\n\nThis is equivalent to using the `--no-color` CLI switch.\n\n```elixir\n# .credo.exs\n%{\n  configs: [\n    %{\n      name: \"default\",\n      color: false,\n      # files, checks etc.\n    }\n  ]\n}\n```\n\n\n### `:files`\n\n```elixir\n# .credo.exs\n%{\n  configs: [\n    %{\n      name: \"default\",\n      files: %{\n        included: [\"mix.exs\", \"lib/\", \"src/\", \"web/\", \"apps/\"],\n        excluded: [\"test/\"]\n      },\n      # checks etc.\n    }\n  ]\n}\n```\n\nThe `:files` map can have two fields:\n\n- `:included` contains a list of files, directories and globs (as in `\"**/*_test.exs\"`)\n- `:excluded` contains a list of files, directories, globs (as in `\"**/*_test.exs\"`) and regular expressions (as in `~r\"/_build/\"`)\n\n\n### `:parse_timeout`\n\nConfigures a timeout for parsing source files in milliseconds (*defaults to 5000 milliseconds*).\n\n```elixir\n# .credo.exs\n%{\n  configs: [\n    %{\n      name: \"default\",\n      parse_timeout: 60_000,\n      # files, checks etc.\n    }\n  ]\n}\n```\n\n\n### `:plugins`\n\nConfigures plugin modules that Credo should load at start up (*defaults to `[]`*).\n\nThis is needed to [enable and configure plugins](../plugins/using_plugins.md) in the analysis.\n\n```elixir\n# .credo.exs\n%{\n  configs: [\n    %{\n      name: \"default\",\n      plugins: [\n        {CredoDemoPlugin, []}\n      ],\n      # files, checks etc.\n    }\n  ]\n}\n```\n\nAll plugins are configured using a two-element tuple:\n\n```elixir\n{MyApp.PluginModule, params}\n```\n\n- `MyApp.PluginModule` - the module representing the plugin to be configured\n- `params` - can be either `false` (to disable the plugin) or a keyword list of parameters (to configure the plugin)\n\n\n### `:requires`\n\nConfigures Elixir source files that Credo should require at start up (*defaults to `[]`*).\n\nThis is needed to [add local custom checks](../custom_checks/adding_checks.md) in the analysis.\n\n```elixir\n# .credo.exs\n%{\n  configs: [\n    %{\n      name: \"default\",\n      requires: [\"lib/check/my_check.ex\"],\n      # files, checks etc.\n    }\n  ]\n}\n```\n\n\n### `:strict`\n\nSet to `true` to enable low priority checks (*defaults to `false`*).\n\nThis is equivalent to using the `--strict` CLI switch.\n\n```elixir\n# .credo.exs\n%{\n  configs: [\n    %{\n      name: \"default\",\n      strict: true,\n      # files, checks etc.\n    }\n  ]\n}\n```","ref":"config_file.html#config-keys"},{"type":"extras","title":"Using a custom configuration - .credo.exs","doc":"You can tell Credo to use a custom config key instead of `default`:\n\n```bash\nmix credo --config-name  \n```\n\nFor example, given the following config file:\n\n```elixir\n# .credo.exs\n%{\n  configs: [\n    %{\n      name: \"default\",\n      # files, checks etc.\n    },\n    %{\n      name: \"picky\",\n      strict: true,\n      # files, checks etc.\n    },\n  ]\n}\n```\n\nYou can run the `picky` config with this command:\n\n```bash\nmix credo --config-name picky\n```","ref":"config_file.html#using-a-custom-configuration"},{"type":"extras","title":"Using a specific configuration file - .credo.exs","doc":"You can tell Credo to use a specific config file anywhere in the file system:\n\n```bash\nmix credo --config-file  \n```\n\n> ##","ref":"config_file.html#using-a-specific-configuration-file"},{"type":"extras","title":"Note {: .warning} - .credo.exs","doc":">\n> Specifying a config file this way, only that config file's contents are loaded.\n> The \"Transitive configuration files\" mechanism described in the next section does not apply in this case.","ref":"config_file.html#note-warning"},{"type":"extras","title":"Transitive configuration files - .credo.exs","doc":"Credo traverses the filesystem's folder structure upwards to find additional config files which it applies to the current project.\n\nConsider the following directory structure:\n\n```bash\n/\n  home/\n    rrrene/\n      projects/\n        foo/\n          .credo.exs\n        bar/\n      .credo.exs\n```\n\nIn this example, there is a Credo config file in my home folder and one in my project.\n\nFor project `foo/`, the contents of `/home/rrrene/projects/foo/.credo.exs` are merged with the settings in `/home/rrrene/.credo.exs` and Credo's default config.\n\nFor project `bar/`, the contents of `/home/rrrene/.credo.exs` and Credo's default config are relevant.\n\nPlease note that, as mentioned above, Credo's config can also reside in a `config/` subdirectory at every step of the way.\n\nGiven this directory structure:\n\n```bash\n/\n  home/\n    rrrene/\n      config/\n        .credo.exs\n      projects/\n        bar/\n        foo/\n          config/\n            .credo.exs\n```\n\nFor project `foo/`, the contents of `/home/rrrene/projects/foo/config/.credo.exs` are merged with the settings in `/home/rrrene/config/.credo.exs` and Credo's default config.\n\nThis works great for umbrella projects, where you can have individual `.credo.exs` files for each app and/or a global one in the umbrella's `config/` or root folder.\nThis way, you can enable/disable settings on a per-app basis.","ref":"config_file.html#transitive-configuration-files"},{"type":"extras","title":"Default checks - .credo.exs","doc":"By default, these checks are enabled:\n\n- `Credo.Check.Consistency.ExceptionNames`\n- `Credo.Check.Consistency.LineEndings`\n- `Credo.Check.Consistency.ParameterPatternMatching`\n- `Credo.Check.Consistency.SpaceAroundOperators`\n- `Credo.Check.Consistency.SpaceInParentheses`\n- `Credo.Check.Consistency.TabsOrSpaces`\n- `Credo.Check.Design.AliasUsage`\n- `Credo.Check.Design.TagFIXME`\n- `Credo.Check.Design.TagTODO`\n- `Credo.Check.Readability.AliasOrder`\n- `Credo.Check.Readability.FunctionNames`\n- `Credo.Check.Readability.LargeNumbers`\n- `Credo.Check.Readability.MaxLineLength`\n- `Credo.Check.Readability.ModuleAttributeNames`\n- `Credo.Check.Readability.ModuleDoc`\n- `Credo.Check.Readability.ModuleNames`\n- `Credo.Check.Readability.ParenthesesInCondition`\n- `Credo.Check.Readability.ParenthesesOnZeroArityDefs`\n- `Credo.Check.Readability.PipeIntoAnonymousFunctions`\n- `Credo.Check.Readability.PredicateFunctionNames`\n- `Credo.Check.Readability.PreferImplicitTry`\n- `Credo.Check.Readability.RedundantBlankLines`\n- `Credo.Check.Readability.Semicolons`\n- `Credo.Check.Readability.SpaceAfterCommas`\n- `Credo.Check.Readability.StringSigils`\n- `Credo.Check.Readability.TrailingBlankLine`\n- `Credo.Check.Readability.TrailingWhiteSpace`\n- `Credo.Check.Readability.UnnecessaryAliasExpansion`\n- `Credo.Check.Readability.VariableNames`\n- `Credo.Check.Readability.WithSingleClause`\n- `Credo.Check.Refactor.Apply`\n- `Credo.Check.Refactor.CondStatements`\n- `Credo.Check.Refactor.CyclomaticComplexity`\n- `Credo.Check.Refactor.FilterCount`\n- `Credo.Check.Refactor.FilterFilter`\n- `Credo.Check.Refactor.FunctionArity`\n- `Credo.Check.Refactor.LongQuoteBlocks`\n- `Credo.Check.Refactor.MapJoin`\n- `Credo.Check.Refactor.MatchInCondition`\n- `Credo.Check.Refactor.NegatedConditionsInUnless`\n- `Credo.Check.Refactor.NegatedConditionsWithElse`\n- `Credo.Check.Refactor.Nesting`\n- `Credo.Check.Refactor.RedundantWithClauseResult`\n- `Credo.Check.Refactor.RejectReject`\n- `Credo.Check.Refactor.UnlessWithElse`\n- `Credo.Check.Refactor.WithClauses`\n- `Credo.Check.Warning.ApplicationConfigInModuleAttribute`\n- `Credo.Check.Warning.BoolOperationOnSameValues`\n- `Credo.Check.Warning.Dbg`\n- `Credo.Check.Warning.ExpensiveEmptyEnumCheck`\n- `Credo.Check.Warning.IExPry`\n- `Credo.Check.Warning.IoInspect`\n- `Credo.Check.Warning.MissedMetadataKeyInLoggerConfig`\n- `Credo.Check.Warning.OperationOnSameValues`\n- `Credo.Check.Warning.OperationWithConstantResult`\n- `Credo.Check.Warning.RaiseInsideRescue`\n- `Credo.Check.Warning.SpecWithStruct`\n- `Credo.Check.Warning.UnsafeExec`\n- `Credo.Check.Warning.UnusedEnumOperation`\n- `Credo.Check.Warning.UnusedFileOperation`\n- `Credo.Check.Warning.UnusedKeywordOperation`\n- `Credo.Check.Warning.UnusedListOperation`\n- `Credo.Check.Warning.UnusedPathOperation`\n- `Credo.Check.Warning.UnusedRegexOperation`\n- `Credo.Check.Warning.UnusedStringOperation`\n- `Credo.Check.Warning.UnusedTupleOperation`\n- `Credo.Check.Warning.WrongTestFileExtension`\n\nTo run them all, just use `mix credo --strict`.\n\nTo see which checks are enabled with your current config and/or command line switches, run\n\n```\nmix credo info --verbose\n```\n\nIf you want to see which checks are enabled with a set of command line switches, just append them to the command:\n\n```\nmix credo info --verbose --strict\n```","ref":"config_file.html#default-checks"},{"type":"extras","title":"Command line switches","doc":"# Command line switches\n\nMost configuration options are also available as command line switches, so that you can e.g. only run certain checks at a time to focus attention on those issues.","ref":"cli_switches.html"},{"type":"extras","title":"Common use cases - Command line switches","doc":"Here are a couple of common use case and their respective command line switches:\n\n\n#","ref":"cli_switches.html#common-use-cases"},{"type":"extras","title":"Output Formats - Command line switches","doc":"Use `--format` to format the output in one of the following formats:\n\n- `--format flycheck` for [Flycheck](http://www.flycheck.org/) output\n- `--format json` for [JSON](https://www.json.org/) output\n- `--format sarif` for [SARIF](https://www.oasis-open.org/committees/tc_home.php?wg_abbrev=sarif) output\n\nAdditionally, you can deactivate the output's coloring by using `--no-color`.\n\n\n#","ref":"cli_switches.html#output-formats"},{"type":"extras","title":"Watch & re-run analysis on file changes - Command line switches","doc":"Use `--watch` to watch for file changes and re-run the analysis.\n\n```bash\nmix credo --watch\nmix credo list --only readability --watch\nmix credo diff v1.4.0 --watch\n```\n\n\n#","ref":"cli_switches.html#watch-re-run-analysis-on-file-changes"},{"type":"extras","title":"Only run some checks - Command line switches","doc":"To run only a subset of checks, you can use the `--checks` or the `-c` alias (also aliased as `--only`).\n\n```bash\n# to only run the Readability checks, use:\n\n$ mix credo --only readability\n\n# to only run Readability checks and Warnings, use:\n\n$ mix credo --only readability,warning\n```\n\nThe second example illustrates how the command takes a comma-separated list. All commands matching **any** of the passed items will be run.\n\nYou can use partial names to quickly run checks. `mix credo --only todo` will show all `# TODO` comments since `todo` will match Credo.Check.Design.Tag**TODO**. `mix credo --only inspect` will show you all calls to `IO.inspect` since it matches Credo.Check.Warning.Io**Inspect**.\n\n\n#","ref":"cli_switches.html#only-run-some-checks"},{"type":"extras","title":"Ignore some checks - Command line switches","doc":"To ignore selected checks, you can use the `--ignore-checks` or the `-i` alias (also aliased as `--ignore`).\n\n```bash\n# to ignore all Readability checks, use:\n\n$ mix credo --ignore readability\n\n# to ignore all Readability checks and the DuplicatedCode check, use:\n\n$ mix credo --ignore readability,duplicated\n```\n\nThe second example illustrates how the command takes a comma-separated list. All commands matching any of the passed items will be ignored.\n\nYou can use partial names to quickly exclude checks. `mix credo --ignore nameredec` will exclude all checks for variables/parameters having the same name as declared functions by matching Credo.Check.Warning.**NameRedeclarationBy...**\n\n\n#","ref":"cli_switches.html#ignore-some-checks"},{"type":"extras","title":"Re-enable disabled checks - Command line switches","doc":"Use `--enable-disabled-checks [pattern]` to re-enable checks that were disabled in the config using `{CheckModule, false}`. This comes in handy when using checks on a case-by-case basis\n\nAs with other check-related switches, `pattern` is a comma-delimited list of patterns:\n\n```bash\n$ mix credo info --enable-disabled-checks Credo.Check.Readability.Specs,Credo.Check.Refactor.DoubleBooleanNegation\n```\n\nOf course, we can have the same effect by choosing the pattern less explicitly:\n\n```bash\n$ mix credo info --enable-disabled-checks specs,double\n```\n\n\n#","ref":"cli_switches.html#re-enable-disabled-checks"},{"type":"extras","title":"Parsing source from STDIN - Command line switches","doc":"You can also use Credo to parse source that has been piped directly into it.\nThis is especially useful when integrating with external editors. You can use this feature by passing the `--read-from-stdin` option as follows:\n\n```bash\n$ echo 'IO.puts(\"hello world\");' | mix credo --format flycheck --read-from-stdin\n# stdin:1: C: There is no whitespace around parentheses/brackets most of the time, but here there is.\n```\n\nNotice the origin if the source is coming annotated as `stdin`, you can change this annotation by passing it along after option like so:\n\n```bash\n$ echo 'IO.puts(\"hello world\");' | mix credo --format flycheck --read-from-stdin /path/representing/the_current/source.ex\n# /path/representing/the_current/source.ex:1: C: There is no whitespace around parentheses/brackets most of the time, but here there is.\n```\n\nDo note with the passed option as filename is a stub that is just used to prefix the error and so certain editors can annotate the original file.\n\n\n#","ref":"cli_switches.html#parsing-source-from-stdin"},{"type":"extras","title":"Show all issues (including low priority ones) - Command line switches","doc":"By default, Credo's CLI output shows only the 5 most important issues per category. Using `--all`, you can show all the important issues in each category.\n\nUse the `--all-priorities` switch to include low priority issues in the output (aliased as `--strict`).","ref":"cli_switches.html#show-all-issues-including-low-priority-ones"},{"type":"extras","title":"Command line switches and config file - Command line switches","doc":"Most configuration options are also available as command line switches.\n\n```bash\n➜ mix credo suggest --help\nUsage: mix credo suggest [paths] [options]\n\nSuggests objects from every category that Credo thinks can be improved.\n\nExamples:\n  $ mix credo suggest --format json\n  $ mix credo suggest lib/**/*.ex --only consistency --all\n  $ mix credo suggest --checks-without-tag formatter --checks-without-tag controversial\n\nArrows (↑ ↗ → ↘ ↓) hint at the importance of an issue.\n\nSuggest options:\n  -a, --all                     Show all issues\n  -A, --all-priorities          Show all issues including low priority ones\n  -c, --checks                  Only include checks that match the given strings\n      --checks-with-tag         Only include checks that match the given tag (can be used multiple times)\n      --checks-without-tag      Ignore checks that match the given tag (can be used multiple times)\n      --config-file             Use the given config file\n  -C, --config-name             Use the given config instead of \"default\"\n      --enable-disabled-checks  Re-enable disabled checks that match the given strings\n      --files-included          Only include these files (accepts globs, can be used multiple times)\n      --files-excluded          Exclude these files (accepts globs, can be used multiple times)\n      --format                  Display the list in a specific format (json,flycheck,sarif,oneline)\n  -i, --ignore-checks           Ignore checks that match the given strings\n      --ignore                  Alias for --ignore-checks\n      --min-priority            Minimum priority to show issues (higher,high,normal,low,ignore or number)\n      --mute-exit-status        Exit with status zero even if there are issues\n      --only                    Alias for --checks\n      --strict                  Alias for --all-priorities\n\nGeneral options:\n      --[no-]color              Toggle colored output\n  -v, --version                 Show version\n  -h, --help                    Show this help\n\nFind advanced usage instructions and more examples here:\n  https://hexdocs.pm/credo/suggest_command.html\n\nGive feedback and open an issue here:\n  https://github.com/rrrene/credo/issues\n```\n\nSome of these are not available as configuration options in `.credo.exs`:\n\n* `--enable-disabled-checks [PATTERN]` activates disabled checks on the fly\n* `--mute-exit-status` forces Credo to exit with an exit status of `0`","ref":"cli_switches.html#command-line-switches-and-config-file"},{"type":"extras","title":"Inline Config Comments","doc":"# Inline Config Comments\n\nUsers of Credo can now disable individual lines or files for all or just\nspecific checks.\n\n```elixir\ndefp my_fun() do\n  # credo:disable-for-next-line\n  IO.inspect {:we_want_this_inspect_in_production!}\nend\n```\n\nThere are four config comments:\n\n* `# credo:disable-for-this-file` - to disable for the entire file\n* `# credo:disable-for-next-line` - to disable for the next line\n* `# credo:disable-for-previous-line` - to disable for the previous line\n* `# credo:disable-for-lines: ` - to disable for the given number of lines (negative for previous lines)\n\n\n## `credo:disable-for-this-file`\n\nThis config comment can be used to disable Credo for the current file:\n\n```elixir\n# credo:disable-for-this-file\ndefmodule MyModule do\n  # ...\nend\n```\n\nYou can also provide the name of the check you want to disable:\n\n```elixir\n# credo:disable-for-this-file Credo.Check.Warning.IoInspect\ndefmodule MyModule do\n  # ...\nend\n```\n\nLastly, you can put a regular expression (`/.+/`) instead of a check name to disable multiple checks (or if you do not want to type out the checks):\n\n```elixir\n# credo:disable-for-this-file /\\.Warning\\./\ndefmodule MyModule do\n  # ...\nend\n```\n\n## `credo:disable-for-next-line`\n\nThis config comment can be used to disable Credo for the following line of source code:\n\n```elixir\ndefp my_fun() do\n  # credo:disable-for-next-line\n  IO.inspect {:we_want_this_inspect_in_production!}\nend\n```\n\nYou can also provide the name of the check you want to disable:\n\n```elixir\ndefp my_fun() do\n  # credo:disable-for-next-line Credo.Check.Warning.IoInspect\n  IO.inspect {:we_want_this_inspect_in_production!}\nend\n```\n\nLastly, you can put a regular expression (`/.+/`) instead of a check name to disable multiple checks (or if you do not want to type out the checks):\n\n```elixir\ndefp my_fun() do\n  # credo:disable-for-next-line /\\.Warning\\./\n  IO.inspect {:we_want_this_inspect_in_production!}\nend\n```\n\n## `credo:disable-for-previous-line`\n\nThis config comment can be used to disable Credo for the preceding line of source code:\n\n```elixir\ndefp my_fun() do\n  IO.inspect {:we_want_this_inspect_in_production!}\n  # credo:disable-for-previous-line\nend\n```\n\nYou can also provide the name of the check you want to disable:\n\n```elixir\ndefp my_fun() do\n  IO.inspect {:we_want_this_inspect_in_production!}\n  # credo:disable-for-previous-line Credo.Check.Warning.IoInspect\nend\n```\n\nLastly, you can put a regular expression (`/.+/`) instead of a check name to disable multiple checks (or if you do not want to type out the checks):\n\n```elixir\ndefp my_fun() do\n  IO.inspect {:we_want_this_inspect_in_production!}\n  # credo:disable-for-previous-line /\\.Warning\\./\nend\n```\n\n## `credo:disable-for-lines: `\n\nThis config comment can be used to disable Credo for the preceding line of source code:\n\n```elixir\n# credo:disable-for-lines:3\ndefp my_fun() do\n  IO.inspect {:we_want_this_inspect_in_production!}\nend\n```\n\nYou can also provide the name of the check you want to disable:\n\n```elixir\n# credo:disable-for-lines:3 Credo.Check.Warning.IoInspect\ndefp my_fun() do\n  IO.inspect {:we_want_this_inspect_in_production!}\nend\n```\n\nLastly, you can put a regular expression (`/.+/`) instead of a check name to disable multiple checks (or if you do not want to type out the checks):\n\n```elixir\n# credo:disable-for-lines:3 /\\.Warning\\./\ndefp my_fun() do\n  IO.inspect {:we_want_this_inspect_in_production!}\nend\n```","ref":"config_comments.html"},{"type":"extras","title":"Check Parameters","doc":"# Check Parameters\n\nChecks are configured using parameters (short: params).\n\nThere are custom params for each check and a set of [general params](#general-params) that work for all checks.","ref":"check_params.html"},{"type":"extras","title":"Configuration - Check Parameters","doc":"Checks and their params are configured in [Credo's configuration file](./config_file.md):\n\n```elixir\n# .credo.exs\n%{\n  configs: [\n    %{\n      name: \"default\",\n      checks: [\n        {Credo.Check.Consistency.TabsOrSpaces, false},\n        {Credo.Check.Design.AliasUsage, if_nested_deeper_than: 2},\n      ],\n      # files etc.\n    }\n  ]\n}\n```\n\nAll checks are configured using a two-element tuple:\n\n```elixir\n{MyApp.CheckModule, params}\n```\n\n`MyApp.CheckModule` is the module representing the check to be configured ([read more about custom checks](../custom_checks/adding_checks.md)).\n\n`params` can be either `false`, to disable the check ...\n\n```elixir\n# This check won't be part of the analysis\n{Credo.Check.Consistency.TabsOrSpaces, false}\n```\n\n... or a keyword list of parameters, used to configure the check:\n\n```elixir\n{Credo.Check.Design.AliasUsage, if_nested_deeper_than: 2}\n```","ref":"check_params.html#configuration"},{"type":"extras","title":"General params - Check Parameters","doc":"While `params` are defined by each check individually, there are a couple of general params provided by Credo, which work the same for each check.\n\nThe general params available are:\n\n- [`:category`](#category)\n- [`:exit_status`](#exit_status)\n- [`:files`](#files)\n- [`:priority`](#priority)\n- [`:tags`](#tags)\n\n### `:category`\n\nOverwrites the category of the check\n\n```elixir\n{Credo.Check.Warning.IExPry, category: :readability}\n```\n\n### `:exit_status`\n\nOverwrites a custom exit status for the check\n\n```elixir\n{Credo.Check.Warning.IoInspect, exit_status: 0}\n```\n\nRead more on [exit status in Credo](../introduction/exit_statuses.md).\n\n### `:files`\n\nControls which files the check runs on.\n\nThis allows for specific files or patterns to be included/excluded for specific checks.\n\nThe syntax is the same as for the top-level `:files` key:\n\n```elixir\n# check included for Elixir files in lib/ only\n{Credo.Check.Consistency.ExceptionNames, files: %{included: [\"lib/**/*.ex\"]}},\n\n# check excluded for a specific file\n{Credo.Check.Warning.IExPry, files: %{excluded: [\"lib/debug_server.ex\"]}},\n\n# check included for all Elixir script files, but excluded for test scripts\n{Credo.Check.Warning.IoInspect, files: %{included: [\"**/*.exs\"], excluded: [\"**/*_test.exs\"]}},\n```\n\nPlease note that these params do not \"override\" the top-level config, but are applied to the result of the top-level config's resolution.\n\n### `:priority`\n\nOverwrites the priority of the check\n\n```elixir\n{Credo.Check.Warning.IoInspect, priority: :low}\n```\n\nAvailable priorities are: `:low`, `:normal`, `:high` and `:higher`.\n\n### `:tags`\n\nOverwrites or appends the tags of the check\n\n```elixir\n# Overwrite all tags for `MyApp.CheckModule`\n{MyApp.CheckModule, tags: [:my_tag]}\n\n# OR: append tags to `MyApp.CheckModule`\n{MyApp.CheckModule, tags: [:__initial__, :my_tag]}\n```\n\nTags can then be used to include or exclude checks from the analysis:\n\n```bash\n# Only run checks tagged `:my_tag` during analysis\n$ mix credo --checks-with-tag my_tag\n\n# Exclude all checks tagged `:my_tag` from analysis\n$ mix credo --checks-without-tag my_tag\n```","ref":"check_params.html#general-params"},{"type":"extras","title":"mix credo suggest","doc":"# mix credo suggest\n\n`suggest` suggests issues to fix in your code.","ref":"suggest_command.html"},{"type":"extras","title":"Examples - mix credo suggest","doc":"```bash\n$ mix credo\n$ mix credo suggest                 # same thing, since it's the default command\n$ mix credo --strict --format=json  # include low priority issues, output as JSON\n$ mix credo suggest --help          # more options\n\n$ mix credo suggest --format json\n$ mix credo suggest lib/**/*.ex --only consistency --strict\n$ mix credo suggest --checks-without-tag formatter --checks-without-tag controversial\n```","ref":"suggest_command.html#examples"},{"type":"extras","title":"Command Line Switches - mix credo suggest","doc":"| Name, shorthand                                                                                    | Description                                                              |\n| -------------------------------------------------------------------------------------------------- | ------------------------------------------------------------------------ |\n| [`--all`](#all), [`-a`](#all)                                                                      | Show all issues for each category                                        |\n| [`--all-priorities`](#all-priorities-aliased-as-strict), [`-A`](#all-priorities-aliased-as-strict) | Show all issues including low priority ones                              |\n| [`--checks`](#checks-aliased-as-only)                                                              | Only include checks that match the given comma-seperated patterns        |\n| [`--checks-with-tag`](#checks-with-tag)                                                            | Only include checks that match the given tag                             |\n| [`--checks-without-tag`](#checks-without-tag)                                                      | Ignore checks that match the given tag                                   |\n| [`--config-file`](#config-file)                                                                    | Use the given config file as Credo's config                              |\n| [`--config-name`](#config-name)                                                                    | Use the given config instead of \"default\"                                |\n| [`--enable-disabled-checks`](#enable-disabled-checks)                                              | Re-enable disabled checks that match the given comma-seperated patterns  |\n| [`--files-included`](#files-included)                                                              | Only include these files                                                 |\n| [`--files-excluded`](#files-excluded)                                                              | Exclude these files                                                      |\n| [`--format`](#format)                                                                              | Display the list in a specific format (json, flycheck, sarif or oneline) |\n| [`--ignore-checks`](#ignore-checks-aliased-as-ignore)                                              | Ignore checks that match the given comma-seperated patterns              |\n| [`--ignore`](#ignore)                                                                              | Alias for [`--ignore-checks`](#ignore-checks-aliased-as-ignore)          |\n| [`--min-priority`](#min-priority)                                                                  | Minimum priority to show issues                                          |\n| [`--mute-exit-status`](#mute-exit-status)                                                          | Exit with status zero even if there are issues                           |\n| [`--only`](#only)                                                                                  | Alias for [`--checks`](#checks-aliased-as-only)                          |\n| [`--strict`](#strict)                                                                              | Alias for [`--all-priorities`](#all-priorities-aliased-as-strict)        |\n| [`--verbose`](#verbose)                                                                            | Additionally print the check and the source code that raised the issue   |","ref":"suggest_command.html#command-line-switches"},{"type":"extras","title":"Descriptions - mix credo suggest","doc":"### `--all`\n\nShow all issues for each category\n\nBy default, Credo's report is limited to 5 issues per category.\n\n```bash\n$ mix credo --all\n```\n\n### `--all-priorities` (aliased as `--strict`)\n\nShow all issues including low priority ones\n\nBy default, Credo's report is limited to high priority issues as indicated by the arrows (↑ ↗ → ↘ ↓) next to each issue.\n\n```bash\n$ mix credo --strict\n```\n\n### `--checks` (aliased as `--only`)\n\nOnly include checks that match the given comma-seperated patterns\n\n```bash\n# Run only checks where the name matches \"readability\" or \"space\" (case-insensitive),\n# e.g. `Credo.Check.Readability.ModuleDoc` or `Credo.Check.Consistency.SpaceAroundOperators`\n$ mix credo --only readability,space\n```\n\nThe patterns are also compiled using `Regex.compile/2`, which allows for more complex queries:\n\n```bash\n# Run only checks where the name matches \"readability\" and \"space\"\n# (case-insensitive), e.g. `Credo.Check.Readability.SpaceAfterCommas`\n$ mix credo --only readability.+space\n```\n\n### `--checks-with-tag`\n\nOnly include checks that match the given tag (can be used multiple times)\n\n```bash\n$ mix credo --checks-with-tag experimental --checks-with-tag controversial\n```\n\n### `--checks-without-tag`\n\nIgnore checks that match the given tag (can be used multiple times)\n\n```bash\n$ mix credo --checks-without-tag formatter\n```\n\n### `--config-file`\n\nUse the given config file as Credo's config\n\n```bash\n$ mix credo --config-file ./path/to/credo.exs\n```\n\nThis disables [Transitive configuration files](config_file.html#transitive-configuration-files) and only the given config file is\n\n### `--config-name`\n\nUse the given config instead of \"default\"\n\n```bash\n$ mix credo --config-name special-ci-config\n```\n\n### `--enable-disabled-checks`\n\nRe-enable disabled checks that match the given comma-seperated patterns\n\n```bash\n# Enable all disabled checks where the name matches \"readability\" or \"space\" (case-insensitive),\n# e.g. `Credo.Check.Readability.ModuleDoc` or `Credo.Check.Consistency.SpaceAroundOperators`\n$ mix credo --enable-disabled-checks readability,space\n```\n\nThe patterns are also compiled using `Regex.compile/2`, which allows for more complex queries:\n\n```bash\n# Enable all previously disabled checks where the name matches \"readability\" and \"space\"\n# (case-insensitive), e.g. `Credo.Check.Readability.SpaceAfterCommas`\n$ mix credo --enable-disabled-checks readability.+space\n\n# Enable *all* disabled checks by simply using:\n$ mix credo --enable-disabled-checks .+\n```\n\n### `--files-included`\n\nOnly include these files (accepts globs, can be used multiple times)\n\n```bash\n$ mix credo --files-included \"./lib/**/*.ex\" --files-included \"./src/**/*.ex\"\n```\n\n### `--files-excluded`\n\nExclude these files (accepts globs, can be used multiple times)\n\n```bash\n$ mix credo --files-excluded \"./test/**/*.exs\"\n```\n\n### `--format`\n\nDisplay the list in a specific format (json, flycheck, sarif or oneline)\n\n```bash\n$ mix credo --format json\n```\n\n### `--ignore-checks` (aliased as `--ignore`)\n\nIgnore checks that match the given comma-seperated patterns\n\n```bash\n# Ignore checks where the name matches \"readability\" or \"space\" (case-insensitive),\n# e.g. `Credo.Check.Readability.ModuleDoc` or `Credo.Check.Consistency.SpaceAroundOperators`\n$ mix credo --ignore readability,space\n```\n\nThe patterns are also compiled using `Regex.compile/2`, which allows for more complex queries:\n\n```bash\n# Ignore checks where the name matches \"readability\" and \"space\"\n# (case-insensitive), e.g. `Credo.Check.Readability.SpaceAfterCommas`\n$ mix credo --ignore readability.+space\n```\n\n### `--ignore`\n\nAlias for [`--ignore-checks`](#ignore-checks-aliased-as-ignore)\n\n### `--min-priority`\n\nMinimum priority to show issues (higher,high,normal,low,ignore or number)\n\n```bash\n$ mix credo --min-priority high\n```\n\n### `--mute-exit-status`\n\nExit with status zero even if there are issues\n\n```bash\n$ mix credo --format json\n# ...\n\n$ echo $?\n0\n```\n\n### `--only`\n\nAlias for [`--checks`](#checks-aliased-as-only)\n\n### `--strict`\n\nAlias for [`--all-priorities`](#all-priorities-aliased-as-strict)\n\n### `--verbose`\n\nAdditionally print the check and the source code that raised the issue\n\n```bash\n$ mix credo --verbose\n\n# ...\n\n┃\n┃ [W] ↗ There should be no calls to IO.inspect/1. [Credo.Check.Warning.IoInspect]\n┃       lib/foo/bar.ex:121:6 #(Foo.Bar.run)\n┃\n┃       |> IO.inspect(label: \"Arguments given\")\n┃          ^^^^^^^^^^\n┃\n```","ref":"suggest_command.html#descriptions"},{"type":"extras","title":"mix credo diff","doc":"# mix credo diff\n\n`diff` suggests issues to fix in your code (based on git-diff).","ref":"diff_command.html"},{"type":"extras","title":"Examples - mix credo diff","doc":"Example usage:\n\n    $ mix credo diff v1.4.0\n    $ mix credo diff main\n    $ mix credo diff --from-git-ref HEAD --files-included \"lib/**/*.ex\"\n\n    $ mix credo diff --help               # more options","ref":"diff_command.html#examples"},{"type":"extras","title":"Command Line Switches - mix credo diff","doc":"There are a couple of switches to control the diff parameters:\n\n| Name, shorthand                                 | Description                                                             |\n|-------------------------------------------------|-------------------------------------------------------------------------|\n| [`--from-dir`](#from-dir)                       | Diff from the given directory                                           |\n| [`--from-git-ref`](#from-git-ref)               | Diff from the given Git ref                                             |\n| [`--from-git-merge-base`](#from-git-merge-base) | Diff from where the current HEAD branched off from the given merge base |\n| [`--since`](#since)                             | Diff from the given point in time (using Git)                           |\n\nTo adjust the analysis all [command line switches of the `suggest` command](suggest_command.html#command-line-switches) are supported.","ref":"diff_command.html#command-line-switches"},{"type":"extras","title":"Descriptions - mix credo diff","doc":"### `--from-dir`\n\nDiff from the given directory. This is a great option if you are not using `git` for version control.\n\n```bash\n$ mix credo diff --from-dir ../my-project-v1.3.1\n```\n\n### `--from-git-ref`\n\nDiff from the given Git ref by comparing Credo's analysis for the given ref with `HEAD`.\n\n```bash\n$ mix credo diff --from-git-ref v1.3.1\n```\n\nTo oversimplify, this runs Credo on the code points returned by `git diff`.\n\n### `--from-git-merge-base`\n\nDiff from where the current `HEAD` branched off from the given merge base.\n\n```bash\n$ mix credo diff --from-git-merge-base develop\n```\n\nThis runs Credo only on the things that changed on your branch after it branched off of the given ref.","ref":"diff_command.html#descriptions"},{"type":"extras","title":"mix credo explain","doc":"# mix credo explain\n\n`explain` allows you to dig deeper into an issue, by showing you details about the issue and the reasoning by it being reported.\nAs a convenience, you can just copy-paste the `filename:line_number:column` string from the report behind the Credo command to check it out.\n\nExample usage:\n\n```bash\n$ mix credo\n\n[...]\n\n┃ [C] ↗ There is no whitespace around parentheses/brackets most of the time, but here there is.\n┃       lib/my_app/server.ex:10:24 #(Credo.Code.InterpolationHelperTest)\n        ^^^^^^^^^^^^^^^^^^^^^^^^^^\n          copy this and use it with `mix credo`!\n\n[...]\n\n$ mix credo lib/my_app/server.ex:10:24          # show explanation for the issue\n```\n\nPlease note that you do *not* have to specify the `explain` command explicitly when using an issue location:\n\n```bash\n$ mix credo lib/my_app/server.ex:10:24          # short-hand without `explain`\n$ mix credo explain lib/my_app/server.ex:10:24  # identical to this\n```\n\n*Credits:* This is inspired by how you can snap the info from failed tests behind `mix test`.","ref":"explain_command.html"},{"type":"extras","title":"Command Line Switches - mix credo explain","doc":"### `--format`\n\nDisplay the explanation in a specific format (json)\n\n```bash\n$ mix credo explain lib/my_app/server.ex:10:24 --format json\n```","ref":"explain_command.html#command-line-switches"},{"type":"extras","title":"mix credo list","doc":"# mix credo list\n\n`list` suggests issues, grouping them by file and NOT limitting the list to a certain count.","ref":"list_command.html"},{"type":"extras","title":"Examples - mix credo list","doc":"Example usage:\n\n    $ mix credo list                      # show issues grouped by file\n    $ mix credo list --format oneline     # show issues grouped by file, one issue per line\n    $ mix credo list --format oneline -a  # same thing, include low priority issues\n\n    $ mix credo list --help               # more options","ref":"list_command.html#examples"},{"type":"extras","title":"Command Line Switches - mix credo list","doc":"The command line switches are identical to [command line switches of the `suggest` command](suggest_command.html#command-line-switches).","ref":"list_command.html#command-line-switches"},{"type":"extras","title":"Adding Custom Checks","doc":"# Adding Custom Checks\n\nThere comes a time when Credo does not feature the check you need or where you want to test a project- or domain-specific aspect of your codebase.\n\nThis is when you should consider implementing a Custom Check.\n\nCustom checks are simply modules implementing the `Credo.Check` behaviour, which most of the time means that it is a module with a `run/2` function returning a list of `Credo.Issue` structs:\n\n    # lib/checks/my_check.ex\n    defmodule MyProject.Checks.MyCheck do\n      use Credo.Check\n\n      def run(%SourceFile{} = source_file, params) do\n        #\n      end\n    end\n\nCheck `Credo.Check` for more technical information.\n\n#","ref":"adding_checks.html"},{"type":"extras","title":"Our first check: Policing module attributes - Adding Custom Checks","doc":"Sometimes the conventions for names of module attributes change within a development team and you want to encourage people to stop using the old naming scheme for module attributes to avoid endless bikeshedding about whether or not the new naming policy was needed in the first place.\n\nSo, let's implement a check for this completely made up scenario!\n\n#","ref":"adding_checks.html#our-first-check-policing-module-attributes"},{"type":"extras","title":"Minimal check & config - Adding Custom Checks","doc":"First, we add the necessary Elixir module for the check and, for now, just return an empty list of issues.\n\n```elixir\n# lib/my_project/checks/reject_module_attributes.ex\ndefmodule MyProject.Checks.RejectModuleAttributes do\n  # Set up the behaviour and make this module a \"check\":\n  use Credo.Check\n\n  # The minimum each check has to implement is a `run/2` function which returns the found issues:\n  def run(source_file, params \\\\ []) do\n    []\n  end\nend\n```\n\nTo run our new check, we also need to add the necessary `:requires` and `:checks` in our Credo config file:\n\n```elixir\n# .credo.exs\n%{\n  configs: [\n    %{\n      name: \"default\",\n      requires: [\"./lib/my_project/checks/**/*.ex\"],\n      checks: [\n        {MyProject.Checks.RejectModuleAttributes, []}\n      ]\n    }\n  ]\n}\n```\n\nThis tells Credo to require all files in our checks directory and to enable the `RejectModuleAttributes` check.\n\n#","ref":"adding_checks.html#minimal-check-config"},{"type":"extras","title":"Getting it working - Adding Custom Checks","doc":"For a first implementation, our check should look into all modules and report \"violations\" against a list of rejected names.\n\n```elixir\n# lib/my_project/checks/reject_module_attributes.ex\ndefmodule MyProject.Checks.RejectModuleAttributes do\n  use Credo.Check\n\n  # Let's say we want to report module attributes named `@checkdoc`\n  @rejected_names [:checkdoc]\n\n  def run(source_file, params \\\\ []) do\n    # IssueMeta helps keeping track of the source file and the check's params\n    # (technically, it's just a custom tagged tuple)\n    issue_meta = IssueMeta.for(source_file, params)\n\n    # we'll walk the `source_file`'s AST and look for module attributes matching `@rejected_names`\n    Credo.Code.prewalk(source_file, &traverse(&1, &2, @rejected_names, issue_meta))\n  end\n\n  # This matches on the AST structure of module attributes.\n  defp traverse({:@, _, [{name, meta, [_string]} | _]} = ast, issues, rejected_names, issue_meta) do\n    if Enum.member?(rejected_names, name) do\n      {ast, issues ++ [issue_for(name, meta[:line], issue_meta)]}\n    else\n      {ast, issues}\n    end\n  end\n\n  # For all AST nodes not matching the pattern above, we simply do nothing:\n  defp traverse(ast, issues, _rejected_names, _issue_meta) do\n    {ast, issues}\n  end\n\n  defp issue_for(trigger, line_no, issue_meta) do\n    format_issue(\n      issue_meta,\n      message: \"There should be no `@#{trigger}` module attributes.\",\n      trigger: \"@#{trigger}\",\n      line_no: line_no\n    )\n  end\nend\n```\n\nTraversal of the AST is done via `Credo.Code.prewalk/2`, which is a light wrapper around `Macro.prewalk/3`, taking a `Credo.SourceFile` struct instead of an AST.\n\nYou can use `Code.string_to_quoted!/1` to look at the AST structure for code snippets:\n\n```elixir\niex> Code.string_to_quoted!(\"@my_attribute 23\")\n{:@, [line: 1], [{:my_attribute, [line: 1], [23]}]}\n```\n\n#","ref":"adding_checks.html#getting-it-working"},{"type":"extras","title":"Adding configuration parameters - Adding Custom Checks","doc":"Next, we should have a config parameter which allows us to define which module attribute names are no longer allowed.\n\n```elixir\ndefmodule MyProject.Checks.RejectModuleAttributes do\n  # To add a parameter, we use the `:param_defaults` keyword with `use Credo.Check`:\n  use Credo.Check, param_defaults: [reject: [:checkdoc]]\n\n  def run(source_file, params \\\\ []) do\n    # To get a parameter, we use `Params.get/3`, which returns the given parameter from the config\n    # or the default we registered above:\n    reject = Params.get(params, :reject, __MODULE__)\n    issue_meta = IssueMeta.for(source_file, params)\n\n    Credo.Code.prewalk(source_file, &traverse(&1, &2, reject, issue_meta))\n  end\n\n  # ...\nend\n```\n\nWe can now use `.credo.exs` to configure the `:reject` param.\n\nIf the param is not declared ...\n\n    {MyProject.Checks.RejectModuleAttributes, []}\n\n... then the default from your check is used.\n\nIf the param is declared, it overwrites the default, meaning that this ...\n\n    {MyProject.Checks.RejectModuleAttributes, [reject: [:shortdoc]]}\n\n... forbids `@shortdoc`, but allows `@checkdoc` again.\n\nOur final `.credo.exs` might look something like this:\n\n```elixir\n# .credo.exs\n%{\n  configs: [\n    %{\n      name: \"default\",\n      requires: [\"./lib/my_project/checks/**/*.ex\"],\n      checks: [\n        {MyProject.Checks.RejectModuleAttributes, [reject: [:checkdoc, :other_attr]]}\n      ]\n    }\n  ]\n}\n```\n\n#","ref":"adding_checks.html#adding-configuration-parameters"},{"type":"extras","title":"Finalizing the check - Adding Custom Checks","doc":"To really make this a full-fledged Credo check, we have to configure its priority, category and describe what it does (you can find a description of the options in `Credo.Check`).\n\n```elixir\ndefmodule MyProject.Checks.RejectModuleAttributes do\n  use Credo.Check,\n    base_priority: :high,\n    category: :readability,\n    param_defaults: [reject: []],\n    explanations: [\n      check: \"\"\"\n      Look, sometimes the policies for names of module attributes change.\n      We want to make sure that all module attributes adhere to the newest standards of ACME Corp.\n\n      We do not want to discuss this policy, we just want to stop you from using the old\n      module attributes :)\n      \"\"\",\n      params: [reject: \"This check warns about module attributes with any of the given names.\"]\n    ]\n\n  # ...\nend\n```\n\nYou can now use Credo's `explain` command ...\n\n```bash\n$ mix credo explain MyProject.Checks.RejectModuleAttributes\n```\n\n... to show a description of your new check:\n\n```bash\n  MyProject.Checks.MyIExPry\n┃\n┃   [R] Category: readability\n┃    ↗  Priority: high\n┃\n┃    __ WHY IT MATTERS\n┃\n┃       Look, sometimes the policies for names of module attributes change.\n┃       We want to make sure that all module attributes adhere to the newest standards of ACME Corp.\n┃\n┃       We do not want to discuss this policy, we just want to stop you from using the old\n┃       module attributes :)\n┃\n┃    __ CONFIGURATION OPTIONS\n┃\n┃       To configure this check, use this tuple\n┃\n┃         {MyProject.Checks.RejectModuleAttributes,  }\n┃\n┃       with   being false or any combination of these keywords:\n┃\n┃         reject:                Names of module attributes that are no longer allowed\n┃                                (defaults to [])\n┃\n\n```\n\nAnd that's it. Here's the final check:\n\n```elixir\ndefmodule MyProject.Checks.RejectModuleAttributes do\n  use Credo.Check,\n    base_priority: :high,\n    category: :readability,\n    param_defaults: [reject: []],\n    explanations: [\n      check: \"\"\"\n      Look, sometimes the policies for names of module attributes change.\n      We want to make sure that all module attributes adhere to the newest standards of ACME Corp.\n\n      We do not want to discuss this policy, we just want to stop you from using the old\n      module attributes :)\n      \"\"\",\n      params: [reject: \"Names of module attributes that are no longer allowed\"]\n    ]\n\n  def run(source_file, params \\\\ []) do\n    reject = Params.get(params, :reject, __MODULE__)\n\n    issue_meta = IssueMeta.for(source_file, params)\n\n    Credo.Code.prewalk(source_file, &traverse(&1, &2, reject, issue_meta))\n  end\n\n  defp traverse({:@, _, [{name, meta, [_string]} | _]} = ast, issues, rejected_names, issue_meta) do\n    if Enum.member?(rejected_names, name) do\n      {ast, issues ++ issue_for(name, meta[:line], issue_meta)}\n    else\n      {ast, issues}\n    end\n  end\n\n  defp traverse(ast, issues, _rejected_names, _issue_meta) do\n    {ast, issues}\n  end\n\n  defp issue_for(trigger, line_no, issue_meta) do\n    format_issue(\n      issue_meta,\n      message: \"There should be no `@#{trigger}` module attributes.\",\n      trigger: trigger,\n      line_no: line_no\n    )\n  end\nend\n```\n\nNext, let's see how we can [write tests for our custom check!](./testing_checks.md)","ref":"adding_checks.html#finalizing-the-check"},{"type":"extras","title":"Testing Custom Checks","doc":"# Testing Custom Checks\n\n`Credo.Test.Case` provides conveniences for testing Credo custom checks and plugins.\n\nThis module can be used in your test cases, like this:\n\n    use Credo.Test.Case\n\nUsing this module will:\n\n* import all the functions from this module\n* make the test case `:async` by default (use `async: false` to opt out of this behaviour)\n\nIn `test_helper.exs`, make sure `credo` is started:\n\n```\n{:ok, _} = Application.ensure_all_started(:credo)\n```\n\nLet's test the `RejectModuleAttributes` check that we implemented in [Adding custom checks](./adding_checks.md).\n\n#","ref":"testing_checks.html"},{"type":"extras","title":"Basic custom check tests - Testing Custom Checks","doc":"To test a check, we can use `Credo.Test.Case` for convenience:\n\n    defmodule MyProject.Checks.RejectModuleAttributesTest do\n      use Credo.Test.Case\n\n      alias MyProject.Checks.RejectModuleAttributes\n\n      test \"it should NOT report expected code\" do\n        \"\"\"\n        defmodule CredoSampleModule do\n          @somedoc \"This is somedoc\"\n        end\n        \"\"\"\n        |> to_source_file()\n        |> run_check(RejectModuleAttributes)\n        |> refute_issues()\n      end\n\n      test \"it should report code that includes rejected module attribute names\" do\n        \"\"\"\n        defmodule CredoSampleModule do\n          @checkdoc \"This is checkdoc\"\n        end\n        \"\"\"\n        |> to_source_file()\n        |> run_check(RejectModuleAttributes)\n        |> assert_issues()\n      end\n    end\n\nWe have two tests: one for the good case, one for the bad case.\nIn each, we create a source file representation from a heredoc, run our custom check and assert/refute the issues\nwe expect.\n\n#","ref":"testing_checks.html#basic-custom-check-tests"},{"type":"extras","title":"Using custom params in tests - Testing Custom Checks","doc":"We can (and should) also test the params of our check by passing them to `run_check/2`:\n\n    defmodule MyProject.Checks.RejectModuleAttributesTest do\n      use Credo.Test.Case\n\n      alias MyProject.Checks.RejectModuleAttributes\n\n      # ...\n\n      test \"it should NOT report code that includes default rejected module attribute names when a custom set of rejected names is provided\" do\n        \"\"\"\n        defmodule CredoSampleModule do\n          @checkdoc \"This is checkdoc\"\n        end\n        \"\"\"\n        |> to_source_file()\n        |> run_check(RejectModuleAttributes, reject: [:somedoc])\n        |> refute_issues()\n      end\n\n      test \"it should report expected code when a custom set of rejected names is provided\" do\n        \"\"\"\n        defmodule CredoSampleModule do\n          @somedoc \"This is somedoc\"\n        end\n        \"\"\"\n        |> to_source_file()\n        |> run_check(RejectModuleAttributes, reject: [:somedoc])\n        |> assert_issues()\n      end\n    end\n\n#","ref":"testing_checks.html#using-custom-params-in-tests"},{"type":"extras","title":"Asserting found issues - Testing Custom Checks","doc":"Once we get to know domain a little better, we can add more tests, typically testing for other bad cases in which\nour check should produce issues.\n\nNote that there are two assertion functions for this: `assert_issue/2` and `assert_issues/2`, where the first one\nensures that there is a single issue and the second asserts that there are at least two issues.\n\nBoth functions take an optional `callback` as their second parameter, which is called with the `issue` or the\nlist of `issues` found, which makes it convenient  to check for the issues properties ...\n\n    \"\"\"\n    # ... any Elixir code ...\n    \"\"\"\n    |> to_source_file()\n    |> run_check(RejectModuleAttributes)\n    |> assert_issue(fn issue -> assert issue.trigger == \"@checkdoc\" end)\n\n... or properties of the list of issues:\n\n    \"\"\"\n    # ... any Elixir code ...\n    \"\"\"\n    |> to_source_file()\n    |> run_check(RejectModuleAttributes)\n    |> assert_issues(fn issues -> assert Enum.count(issues) == 3 end)\n\n#","ref":"testing_checks.html#asserting-found-issues"},{"type":"extras","title":"Testing checks that analyse multiple source files - Testing Custom Checks","doc":"For checks that analyse multiple source files, like Credo's consistency checks, we can use `to_source_files/1` to\ncreate\n\n    [\n      \"\"\"\n      # source file 1\n      \"\"\",\n      \"\"\"\n      # source file 2\n      \"\"\"\n    ]\n    |> to_source_files()\n    |> run_check(RejectModuleAttributes)\n    |> refute_issues()\n\nIf our check needs named source files, we can always use `to_source_file/2` to create individually named source\nfiles and combine them into a list:\n\n    source_file1 =\n      \"\"\"\n      # source file 1\n      \"\"\"\n      |> to_source_file(\"foo.ex\")\n\n    source_file2 =\n      \"\"\"\n      # source file 2\n      \"\"\"\n      |> to_source_file(\"bar.ex\")\n\n    [source_file1, source_file2]\n    |> run_check(RejectModuleAttributes)\n    |> assert_issue(fn issue -> assert issue.filename == \"foo.ex\" end)","ref":"testing_checks.html#testing-checks-that-analyse-multiple-source-files"},{"type":"extras","title":"Using Plugins","doc":"# Using Plugins\n\n#","ref":"using_plugins.html"},{"type":"extras","title":"Installing plugins - Using Plugins","doc":"Plugins are just modules.\n\nPlugins can be included by listing them under the `:plugins` field `.credo.exs`:\n\n```elixir\n%{\n  configs: [\n    %{\n      name: \"default\",\n      plugins: [\n        {CredoDemoPlugin, []}\n      ]\n    }\n  ]\n}\n```\n\nMost of the time, a Credo plugin will be published on Hex. You include it as a dependency in `mix.exs` like any other dependency:\n\n```elixir\n{:credo_demo_plugin, \"~> 0.1.0\"},\n```\n\nPlugins, like checks, are just modules and functions. They are enabled in Credo's configuration file `.credo.exs`, which you can generate via `mix credo gen.config`:\n\n```bash\n$ mix credo gen.config\n* creating .credo.exs\n```\n\nThe demo plugin adds a command called \"demo\":\n\n```bash\n$ mix credo demo\nBy the power of !\n```\n\nIt seems like there's something missing before the `!` ...\n\n#","ref":"using_plugins.html#installing-plugins"},{"type":"extras","title":"Configuring plugins - Using Plugins","doc":"Plugins can be configured via params, just like checks.\n\nEach entry consists of a two-element tuple: the plugin's module and a keyword list of parameters, which can be used to configure the plugin itself.\n\n```elixir\n%{\n  configs: [\n    %{\n      name: \"default\",\n      plugins: [\n        {CredoDemoPlugin, [castle: \"Grayskull\"]}\n      ]\n    }\n  ]\n}\n```\n\n```bash\n$ mix credo demo\nBy the power of Grayskull!\n```\n\nJust in case, Plugins can be deactivated by setting the second tuple element to `false`.\n\n```elixir\n%{\n  configs: [\n    %{\n      name: \"default\",\n      plugins: [\n        {CredoDemoPlugin, false} # <-- don't load this for now\n      ]\n    }\n  ]\n}\n```\n\nThe demo plugin used in the docs can be found on GitHub and Hex:\n\n- https://github.com/rrrene/credo_demo_plugin\n- https://hex.pm/packages/credo_demo_plugin","ref":"using_plugins.html#configuring-plugins"},{"type":"extras","title":"Creating Plugins","doc":"# Creating Plugins\n\nA plugin is basically just a module that provides an `init/1` callback, taking a `Credo.Execution` struct as its only parameter.\n\n```elixir\ndefmodule CredoDemoPlugin do\n  def init(exec) do\n    # but what do we do here??\n    exec\n  end\nend\n```\n\nThe `Credo.Plugin` module provides a number of functions for extending Credo's core features.\n\n```elixir\ndefmodule CredoDemoPlugin do\n  @config_file File.read!(\".credo.exs\")\n\n  import Credo.Plugin\n\n  def init(exec) do\n    exec\n    |> register_default_config(@config_file)\n    |> register_command(\"demo\", CredoDemoPlugin.DemoCommand)\n    |> register_cli_switch(:castle, :string, :X)\n    |> append_task(:convert_cli_options_to_config, CredoDemoPlugin.ConvertCliSwitchesToPluginParams)\n    |> prepend_task(:set_default_command, CredoDemoPlugin.SetDemoAsDefaultCommand)\n  end\nend\n```\n\nYou can find more information on `Credo.Plugin` and the functions imported:\n\n- `Credo.Plugin.register_default_config/2`\n- `Credo.Plugin.register_command/3`\n- `Credo.Plugin.register_cli_switch/5`\n- `Credo.Plugin.append_task/3`\n- `Credo.Plugin.append_task/4`\n- `Credo.Plugin.prepend_task/3`\n- `Credo.Plugin.prepend_task/4`\n\nThe demo plugin used in the docs can be found on GitHub and Hex:\n\n- https://github.com/rrrene/credo_demo_plugin\n- https://hex.pm/packages/credo_demo_plugin","ref":"creating_plugins.html"}],"content_type":"text/markdown"}